@@ -1,7 +1,7 @@
 /* alarms.c
  * A module of J-Pilot http://jpilot.org
  * 
- * Copyright (C) 2000-2001 by Judd Montgomery
+ * Copyright (C) 2000-2002 by Judd Montgomery
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -98,27 +98,23 @@ int alarms_find_next(struct tm *date1, struct tm *date2, int soonest_only);
  * Start of Dialog window code
  */
 static void cb_dialog_button(GtkWidget *widget,
-			       gpointer   data)
+			     gpointer   data)
 {
    struct alarm_dialog_data *Pdata;
    GtkWidget *w;
-   int i;
 
-   for (w=widget, i=10; w && (i>0); w=w->parent, i--) {
-      if (GTK_IS_WINDOW(w)) {
-	 Pdata = gtk_object_get_data(GTK_OBJECT(w), "alarm");
-	 if (Pdata) {
-	    Pdata->button_hit = GPOINTER_TO_INT(data);
-	 }
-	 gtk_widget_destroy(GTK_WIDGET(w));
-      }
+   w=gtk_widget_get_toplevel(widget);
+   Pdata = gtk_object_get_data(GTK_OBJECT(w), "alarm");
+   if (Pdata) {
+      Pdata->button_hit = GPOINTER_TO_INT(data);
    }
+   gtk_widget_destroy(GTK_WIDGET(w));
 }
 
 static gboolean cb_destroy_dialog(GtkWidget *widget)
 {
    struct alarm_dialog_data *Pdata;
-   char *entry;
+   const char *entry;
    time_t ltime;
    time_t advance;
    time_t remind;
@@ -134,13 +130,13 @@ static gboolean cb_destroy_dialog(GtkWidget *widget)
    entry = gtk_entry_get_text(GTK_ENTRY(Pdata->remind_entry));
    if (Pdata->button_hit==DIALOG_SAID_2) {
       remind = atoi(entry);
-      jpilot_logf(LOG_DEBUG, "remind entry = [%s]\n", entry);
-      set_pref(PREF_REMIND_IN, 0, entry);
+      jp_logf(JP_LOG_DEBUG, "remind entry = [%s]\n", entry);
+      set_pref(PREF_REMIND_IN, 0, entry, TRUE);
       if (GTK_TOGGLE_BUTTON(Pdata->radio1)->active) {
-	 set_pref(PREF_REMIND_UNITS, 0, NULL);
+	 set_pref(PREF_REMIND_UNITS, 0, NULL, TRUE);
 	 remind *= 60;
       } else {
-	 set_pref(PREF_REMIND_UNITS, 1, NULL);
+	 set_pref(PREF_REMIND_UNITS, 1, NULL, TRUE);
 	 remind *= 3600;
       }
       time(&ltime);
@@ -284,6 +280,22 @@ int dialog_alarm(char *title, char *frame_text,
  * End Alarm GUI
  */
 
+time_t mktime_dst_adj(struct tm *tm)
+{
+   struct tm t;
+
+   memcpy(&t, tm, sizeof(t));
+   t.tm_isdst=-1;
+   return mktime(&t);
+}
+
+time_t tm_copy_with_dst_adj(struct tm *dest, struct tm *src)
+{
+   memcpy(dest, src, sizeof(struct tm));
+   dest->tm_isdst=-1;
+   return mktime(dest);
+}
+
 #ifdef ALARMS_DEBUG
 const char *print_date(const time_t t1)
 {
@@ -291,7 +303,7 @@ const char *print_date(const time_t t1)
    static char str[100];
 
    Pnow = localtime(&t1);
-   strftime(str, 80, "%B %d, %Y %H:%M:%S", Pnow);
+   strftime(str, sizeof(str), "%B %d, %Y %H:%M:%S", Pnow);
    return str;
 }
 const char *print_type(AlarmType type)
@@ -334,7 +346,7 @@ void alarms_add_to_list(unsigned int unique_id,
 
    temp_alarm = malloc(sizeof(struct jp_alarms));
    if (!temp_alarm) {
-      jpilot_logf(LOG_WARN, "alarms_add_to_list: Out of memory\n");
+      jp_logf(JP_LOG_WARN, "alarms_add_to_list: Out of memory\n");
       return;
    }
    temp_alarm->unique_id = unique_id;
@@ -414,26 +426,24 @@ void alarms_write_file(void)
    time_t ltime;
    struct tm *now;
 
-   jpilot_logf(LOG_DEBUG, "alarms_write_file()\n");
+   jp_logf(JP_LOG_DEBUG, "alarms_write_file()\n");
 
    time(&ltime);
    now = localtime(&ltime);
 
-   out=jp_open_home_file("jpilot.alarms.tmp", "w");
+   out=jp_open_home_file(EPN".alarms.tmp", "w");
    if (!out) {
-      jpilot_logf(LOG_WARN, "Could not open jpilot.alarms.tmp file\n");
+      jp_logf(JP_LOG_WARN, "Could not open "EPN".alarms.tmp file\n");
       return;
    }
    fail=0;
-   strncpy(line, 
-	   "# This file was generated by jpilot, changes will be lost\n",
-	   255);
+   g_snprintf(line, sizeof(line), "%s",
+	   "# This file was generated by "EPN", changes will be lost\n");
    n = fwrite(line, strlen(line), 1, out);
    if (n<1) fail=1;
 
-   strncpy(line, 
-	   "# This is the last time that jpilot was ran\n",
-	   255);
+   g_snprintf(line, sizeof(line), "%s",
+	   "# This is the last time that "EPN" was ran\n");
    n = fwrite(line, strlen(line), 1, out);
    if (n<1) fail=1;
 
@@ -450,9 +460,9 @@ void alarms_write_file(void)
    fclose(out);
 
    if (fail) {
-      unlink_file("jpilot.alarms.tmp");
+      unlink_file(EPN".alarms.tmp");
    } else {
-      rename_file("jpilot.alarms.tmp", "jpilot.alarms");
+      rename_file(EPN".alarms.tmp", EPN".alarms");
    }
 }
 
@@ -484,6 +494,8 @@ int alarms_do_one(struct Appointment *a,
 		  AlarmType type)
 {
    struct tm *Pnow;
+   struct tm begin;
+   struct tm end;
    char time_str[255];
    char desc_str[255];
    char note_str[255];
@@ -523,19 +535,21 @@ int alarms_do_one(struct Appointment *a,
 
    Pnow = localtime(&t_alarm);
 
-   strftime(date_str, 50, pref_date, Pnow);
-   strftime(time1_str, 50, pref_time, Pnow);
-   strftime(time2_str, 50, pref_time, &(a->end));
-   sprintf(time_str, "%s %s-%s\n", date_str, time1_str, time2_str);
+   strftime(date_str, sizeof(date_str), pref_date, Pnow);
+   tm_copy_with_dst_adj(&begin, &(a->begin));
+   strftime(time1_str, sizeof(time1_str), pref_time, &begin);
+   tm_copy_with_dst_adj(&end, &(a->end));
+   strftime(time2_str, sizeof(time2_str), pref_time, &end);
+   g_snprintf(time_str, sizeof(time_str), "%s %s-%s\n", date_str, time1_str, time2_str);
    desc_str[0]='\0';
    note_str[0]='\0';
    if (a->description) {
-      strncpy(desc_str, a->description, 200);
-      desc_str[200]='\0';
+      strncpy(desc_str, a->description, sizeof(desc_str));
+      desc_str[sizeof(desc_str)-1]='\0';
    }
    if (a->note) {
-      strncpy(note_str, a->note, 200);
-      note_str[200]='\0';
+      strncpy(note_str, a->note, sizeof(note_str));
+      note_str[sizeof(note_str)-1]='\0';
    }
 
    get_pref(PREF_ALARM_COMMAND, &ivalue, &pref_command);
@@ -543,7 +557,7 @@ int alarms_do_one(struct Appointment *a,
 #ifdef ALARMS_DEBUG
    printf("pref_command = [%s]\n", pref_command);
 #endif
-   bzero(command, 1024);
+   memset(command, 0, sizeof(command));
    if (do_command) {
       command[0]='\0';
       for (i=0; i<MAX_PREF_VALUE-1; i++) {
@@ -554,30 +568,30 @@ int alarms_do_one(struct Appointment *a,
 	 if (c1=='%') {
 	    if (c2=='t') {
 	       i++;
-	       strncat(command, time1_str, 1022-len);
+	       strncat(command, time1_str, sizeof(command)-2-len);
 	       continue;
 	    }
 	    /* expand '%d' */
 	    if (c2=='d') {
 	       i++;
-	       strncat(command, date_str, 1022-len);
+	       strncat(command, date_str, sizeof(command)-2-len);
 	       continue;
 	    }
-#ifdef ALARM_SHELL_DESC_NOTE
+#ifdef ENABLE_ALARM_SHELL_DANGER
 	    /* expand '%D' */
 	    if (c2=='D') {
 	       i++;
-	       strncat(command, desc_str, 1022-len);
+	       strncat(command, desc_str, sizeof(command)-2-len);
 	       continue;
 	    }
 	    if (c2=='N') {
 	       i++;
-	       strncat(command, note_str, 1022-len);
+	       strncat(command, note_str, sizeof(command)-2-len);
 	       continue;
 	    }
 #endif
 	 }
-	 if (len<1020) {
+	 if (len<sizeof(command)-4) {
 	    command[len++]=c1;
 	    command[len]='\0';
 	 }
@@ -585,10 +599,10 @@ int alarms_do_one(struct Appointment *a,
 	    break;
 	 }
       }
-      command[1022]='\0';
+      command[sizeof(command)-2]='\0';
 
       make_command_safe(command);
-      jpilot_logf(LOG_STDOUT|LOG_FILE, "executing command = [%s]\n", command);
+      jp_logf(JP_LOG_STDOUT|JP_LOG_FILE, "executing command = [%s]\n", command);
       system(command);
    }
 
@@ -637,7 +651,7 @@ gint cb_timer_alarms(gpointer data)
 	 }
       }
       if (a_list==NULL) {
-	 get_days_appointments2(&a_list, NULL, 0, 0, 1);
+	 get_days_appointments2(&a_list, NULL, 0, 0, 1, NULL);
       }
 #ifdef ALARMS_DEBUG
       printf("unique_id=%d\n", temp_alarm->unique_id);
@@ -666,7 +680,7 @@ gint cb_timer_alarms(gpointer data)
       diff = next_alarm->event_time - t - next_alarm->alarm_advance;
       if (diff <= ALARM_INTERVAL/2) {
 	 if (a_list==NULL) {
-	    get_days_appointments2(&a_list, NULL, 0, 0, 1);
+	    get_days_appointments2(&a_list, NULL, 0, 0, 1, NULL);
 	 }
 	 for (temp_alarm=next_alarm; temp_alarm; temp_alarm=ta_next) {
 	    for (temp_al = a_list; temp_al; temp_al=temp_al->next) {
@@ -706,6 +720,123 @@ gint cb_timer_alarms(gpointer data)
    return TRUE;
 }
 
+/*
+ * This routine takes time (t) and either advances t to the next
+ * occurence of a repeating appointment, or the previous occurence
+ * 
+ * a is the appointment passed in
+ * t is an in/out param.
+ * fdom is first day of month (Sunday being 0)
+ * ndim is the number of days in the month
+ * forward_or_backward should be -1 for backward or 1 for forward.
+ */
+int forward_backward_in_appt_time(const struct Appointment *a,
+				  struct tm *t,
+				  int fdom, int ndim,
+				  int forward_or_backward)
+{
+   int count, dow, freq;
+
+   freq = a->repeatFrequency;
+
+   /* Go forward in time */
+   if (forward_or_backward==1) {
+      switch (a->repeatType) {
+       case repeatNone:
+	 break;
+       case repeatDaily:
+	 break;
+       case repeatWeekly:
+	 for (count=0, dow=t->tm_wday; count<14; count++) {
+	    add_days_to_date(t, 1);
+#ifdef ALARMS_DEBUG      
+	    printf("fpn: weekly forward t.tm_wday=%d, freq=%d\n", t->tm_wday, freq);
+#endif
+	    dow++;
+	    if (dow==7) {
+#ifdef ALARMS_DEBUG      
+	       printf("fpn: dow==7\n");
+#endif
+	       add_days_to_date(t, (freq-1)*7);
+	       dow=0;
+	    }
+	    if (a->repeatDays[dow]) {
+#ifdef ALARMS_DEBUG      
+	       printf("fpn: repeatDay[dow] dow=%d\n", dow);
+#endif
+	       break;
+	    }
+	 }
+	 break;
+       case repeatMonthlyByDay:
+	 add_months_to_date(t, freq);
+	 get_month_info(t->tm_mon, 1, t->tm_year, &fdom, &ndim);
+	 t->tm_mday=((a->repeatDay+7-fdom)%7) - ((a->repeatDay)%7) + a->repeatDay + 1;
+	 if (t->tm_mday > ndim-1) {
+	    t->tm_mday -= 7;
+	 }
+	 break;
+       case repeatMonthlyByDate:
+	 t->tm_mday=a->begin.tm_mday;
+	 add_months_to_date(t, freq);
+	 break;
+       case repeatYearly:
+	 t->tm_mday=a->begin.tm_mday;
+	 add_years_to_date(t, freq);
+	 break;
+      }/*switch */
+      return 0;
+   }
+   /* Go back in time */
+   if (forward_or_backward==-1) {
+      switch (a->repeatType) {
+       case repeatNone:
+	 break;
+       case repeatDaily:
+	 break;
+       case repeatWeekly:
+	 for (count=0, dow=t->tm_wday; count<14; count++) {
+	    sub_days_from_date(t, 1);
+#ifdef ALARMS_DEBUG
+	    printf("fpn: weekly backward t.tm_wday=%d, freq=%d\n", t->tm_wday, freq);
+#endif
+	    dow--;
+	    if (dow==-1) {
+#ifdef ALARMS_DEBUG      
+	       printf("fpn: dow==-1\n");
+#endif
+	       sub_days_from_date(t, (freq-1)*7);
+	       dow=6;
+	    }
+	    if (a->repeatDays[dow]) {
+#ifdef ALARMS_DEBUG      
+	       printf("fpn: repeatDay[dow] dow=%d\n", dow);
+#endif
+	       break;
+	    }
+	 }
+	 break;
+       case repeatMonthlyByDay:
+	 sub_months_from_date(t, freq);
+	 get_month_info(t->tm_mon, 1, t->tm_year, &fdom, &ndim);
+	 t->tm_mday=((a->repeatDay+7-fdom)%7) - ((a->repeatDay)%7) + a->repeatDay + 1;
+	 if (t->tm_mday > ndim-1) {
+	    t->tm_mday -= 7;
+	 }
+	 break;
+       case repeatMonthlyByDate:
+	 t->tm_mday=a->begin.tm_mday;
+	 sub_months_from_date(t, freq);
+	 break;
+       case repeatYearly:
+	 t->tm_mday=a->begin.tm_mday;
+	 sub_years_from_date(t, freq);
+	 break;
+      }/*switch */
+   }
+   return 0;
+}
+
 static int find_prev_next(struct Appointment *a,
 			  int adv,
 			  struct tm *date1,
@@ -729,12 +860,12 @@ static int find_prev_next(struct Appointment *a,
    int freq;
    int found;
    int count;
-   int dow;
    int i;
    int safety_counter;
    int fdom, ndim;
    long fdow;
    int days, begin_days;
+   int found_exception;
 
 #ifdef ALARMS_DEBUG
    printf("fpn: entered find_previous_next\n");
@@ -742,13 +873,13 @@ static int find_prev_next(struct Appointment *a,
    *prev_found=*next_found=0;
    forward=backward=1;
 
-   t1=mktime(date1);
-   t2=mktime(date2);
+   t1=mktime_dst_adj(date1);
+   t2=mktime_dst_adj(date2);
 
-   bzero(tm_prev, sizeof(struct tm));
-   bzero(tm_next, sizeof(struct tm));
+   memset(tm_prev, 0, sizeof(*tm_prev));
+   memset(tm_next, 0, sizeof(*tm_next));
 
-   bzero(&t, sizeof(struct tm));
+   memset(&t, 0, sizeof(t));
    t.tm_year=a->begin.tm_year;
    t.tm_mon=a->begin.tm_mon;
    t.tm_mday=a->begin.tm_mday;
@@ -756,13 +887,21 @@ static int find_prev_next(struct Appointment *a,
    t.tm_min=a->begin.tm_min;
    t.tm_isdst=-1;
 
+   mktime(&t);
+#ifdef ALARMS_DEBUG
+	{
+	   char str[100];
+	   strftime(str, sizeof(str), "%B %d, %Y %H:%M", &t);
+	   printf("fpn: debug=%s\n", str);
+	}
+#endif
    freq = 0;
    switch (a->repeatType) {
     case repeatNone:
 #ifdef ALARMS_DEBUG
       printf("fpn: repeatNone\n");
 #endif
-      t_alarm=mktime(&(a->begin)) - adv;
+      t_alarm=mktime_dst_adj(&(a->begin)) - adv;
       if ((t_alarm < t2) && (t_alarm > t1)) {
 	 memcpy(tm_prev, &(a->begin), sizeof(struct tm));
 	 *prev_found=1;
@@ -785,7 +924,7 @@ static int find_prev_next(struct Appointment *a,
       freq = a->repeatFrequency;
       t_interval = a->repeatFrequency * 86400;
       if (t_interval==0) t_interval=1;
-      t_alarm = mktime(&t);
+      t_alarm = mktime_dst_adj(&t);
       if ((t2 + adv) > t_alarm) {
 	 t_past = ((t2 + adv - t_alarm) / t_interval) *t_interval + t_alarm;
 	 t_future = (((t2 + adv - t_alarm) / t_interval) + 1) *t_interval + t_alarm;
@@ -799,12 +938,38 @@ static int find_prev_next(struct Appointment *a,
       Pnow = localtime(&t_future);
       memcpy(tm_next, Pnow, sizeof(struct tm));
       forward=backward=0;
+      /* since the code above disregarged DST in the above calcs,
+       * we have to try to correct for it here */
+      if (tm_prev->tm_hour<a->begin.tm_hour) {
+	 t_past+=3600; /* 1 hour for dst */
+	 Pnow = localtime(&t_past);
+	 memcpy(tm_prev, Pnow, sizeof(struct tm));
+      }
+      if (tm_prev->tm_hour>a->begin.tm_hour) {
+	 t_past-=3600; /* 1 hour for dst */
+	 Pnow = localtime(&t_past);
+	 memcpy(tm_prev, Pnow, sizeof(struct tm));
+      }
+      if (tm_next->tm_hour<a->begin.tm_hour) {
+	 t_future+=3600; /* 1 hour for dst */
+	 Pnow = localtime(&t_future);
+	 memcpy(tm_next, Pnow, sizeof(struct tm));
+      }
+      if (tm_next->tm_hour>a->begin.tm_hour) {
+	 t_future-=3600; /* 1 hour for dst */
+	 Pnow = localtime(&t_future);
+	 memcpy(tm_next, Pnow, sizeof(struct tm));
+      }
+      if (t_future<t1) {
+  	 *next_found=0;
+	 forward=1;
+      }
 #ifdef ALARMS_DEBUG
 	{
 	   char str[100];
-	   strftime(str, 80, "%B %d, %Y %H:%M", tm_prev);
+	   strftime(str, sizeof(str), "%B %d, %Y %H:%M", tm_prev);
 	   printf("fpn: daily tm_prev=%s\n", str);
-	   strftime(str, 80, "%B %d, %Y %H:%M", tm_next);
+	   strftime(str, sizeof(str), "%B %d, %Y %H:%M", tm_next);
 	   printf("fpn: daily tm_next=%s\n", str);
 	}
 #endif
@@ -814,9 +979,16 @@ static int find_prev_next(struct Appointment *a,
       printf("fpn: repeatWeekly\n");
 #endif
       freq = a->repeatFrequency;
-      t.tm_year=date2->tm_year;
-      t.tm_mon=date2->tm_mon;
-      t.tm_mday=date2->tm_mday;
+      /* Check for future start date */
+      begin_days = dateToDays(&(a->begin));
+      days = dateToDays(&t);
+      if (days > begin_days) {
+	 /* appt started in past, get to current date sooner */
+	 t.tm_year=date2->tm_year;
+	 t.tm_mon=date2->tm_mon;
+	 t.tm_mday=date2->tm_mday;
+      }
+      t.tm_isdst=-1;
       mktime(&t);
       begin_days = dateToDays(&(a->begin));
       days = dateToDays(&t);
@@ -844,7 +1016,7 @@ static int find_prev_next(struct Appointment *a,
 #ifdef ALARMS_DEBUG
 	      {
 		 char str[100];
-		 strftime(str, 80, "%B %d, %Y %H:%M", &t);
+		 strftime(str, sizeof(str), "%B %d, %Y %H:%M", &t);
 		 printf("fpn: initial weekly=%s\n", str);
 	      }
 #endif
@@ -865,8 +1037,14 @@ static int find_prev_next(struct Appointment *a,
 #ifdef ALARMS_DEBUG
       printf("fpn: repeatMonthlyByDay\n");
 #endif
-      t.tm_mon=date2->tm_mon;
-      t.tm_year=date2->tm_year;
+      /* Check for future start date */
+      begin_days = dateToDays(&(a->begin));
+      days = dateToDays(&t);
+      if (days > begin_days) {
+	 /* appt started in past, get to current date sooner */
+	 t.tm_mon=date2->tm_mon;
+	 t.tm_year=date2->tm_year;
+      }
       freq = a->repeatFrequency;
 #ifdef ALARMS_DEBUG
       printf("fpn: freq=%d\n", freq);
@@ -888,10 +1066,10 @@ static int find_prev_next(struct Appointment *a,
       if (t.tm_mday > ndim) {
 	 t.tm_mday -= 7;
       }
-#ifdef ALARMS_DEBUG      
+#ifdef ALARMS_DEBUG
 	{
 	   char str[100];
-	   strftime(str, 80, "%B %d, %Y %H:%M", &t);
+	   strftime(str, sizeof(str), "%B %d, %Y %H:%M", &t);
 	   printf("fpn: initial monthly by day=%s\n", str);
 	}
 #endif
@@ -900,8 +1078,15 @@ static int find_prev_next(struct Appointment *a,
 #ifdef ALARMS_DEBUG
       printf("fpn: repeatMonthlyByDate\n");
 #endif
-      t.tm_mon=date2->tm_mon;
-      t.tm_year=date2->tm_year;
+      /* Check for future start date */
+      begin_days = dateToDays(&(a->begin));
+      days = dateToDays(&t);
+      if (days > begin_days) {
+	 /* appt started in past, get to current date sooner */
+	 t.tm_mon=date2->tm_mon;
+	 t.tm_year=date2->tm_year;
+      }
+
       freq = a->repeatFrequency;
       offset = ((t.tm_year - a->begin.tm_year)*12 +
 		(t.tm_mon - a->begin.tm_mon))%(a->repeatFrequency);
@@ -916,7 +1101,12 @@ static int find_prev_next(struct Appointment *a,
 #ifdef ALARMS_DEBUG
       printf("fpn: repeatYearly\n");
 #endif
-      t.tm_year=date2->tm_year;
+      begin_days = dateToDays(&(a->begin));
+      days = dateToDays(&t);
+      if (days > begin_days) {
+	 /* appt started in past, get to current date sooner */
+	 t.tm_year=date2->tm_year;
+      }
       freq = a->repeatFrequency;
       offset = (t.tm_year - a->begin.tm_year)%(a->repeatFrequency);
 #ifdef ALARMS_DEBUG      
@@ -930,42 +1120,65 @@ static int find_prev_next(struct Appointment *a,
 	 sub_years_from_date(&t, offset);
       }
       break;
-   }/*switch */
+   }
 
    safety_counter=0;
    while(forward || backward) {
       safety_counter++;
-      if (safety_counter > 20) {
-	 jpilot_logf(LOG_STDOUT|LOG_FILE, "find_prev_next():infinite loop, breaking\n");
+      if (safety_counter > 3000) {
+	 jp_logf(JP_LOG_STDOUT|JP_LOG_FILE, "find_prev_next():infinite loop, breaking\n");
 	 if (a->description) {
-	    jpilot_logf(LOG_STDOUT|LOG_FILE, "desc=[%s]\n", a->description);
+	    jp_logf(JP_LOG_STDOUT|JP_LOG_FILE, "desc=[%s]\n", a->description);
 	 }
 	 break;
       }
+      t_temp = mktime_dst_adj(&t);
+      t.tm_isdst=-1;
       t_temp = mktime(&t);
 #ifdef ALARMS_DEBUG
 	{
 	   char str[100];
-	   strftime(str, 80, "%B %d, %Y %H:%M", &t);
+	   strftime(str, sizeof(str), "%B %d, %Y %H:%M", &t);
 	   printf("fpn: trying with=%s\n", str);
 	}
 #endif
-      /* See that we aren't before then begin date */
-      t_begin = mktime(&(a->begin));
+      /* Check for exceptions */
+      found_exception=0;
+      for (i=0; i<a->exceptions; i++) {
+	 if ((t.tm_mday==a->exception[i].tm_mday) &&
+	     (t.tm_mon==a->exception[i].tm_mon) &&
+	     (t.tm_year==a->exception[i].tm_year)
+	     ) {
+	    found_exception=1;
+	    break;
+	 }
+      }
+      if (found_exception) {
+	 if (forward) {
+	    forward_backward_in_appt_time(a, &t, fdom, ndim, 1);
+	    continue;
+	 }
+	 if (backward) {
+	    forward_backward_in_appt_time(a, &t, fdom, ndim, -1);
+	    continue;
+	 }
+      }
+      /* See that we aren't before the begin date */
+      t_begin = mktime_dst_adj(&(a->begin));
       if (t_temp < t_begin - adv) {
 #ifdef ALARMS_DEBUG      
-	 printf("fpn:1\n");
+	 printf("fpn: before begin date\n");
 #endif
 	 backward=0;
       }
       /* If the appointment has an end date, see that we are not past it */
       if (!(a->repeatForever)) {
-	 t_end = mktime(&(a->repeatEnd));
+	 t_end = mktime_dst_adj(&(a->repeatEnd));
 	 if (t_temp > t_end) {
-	    forward=0;
-#ifdef ALARMS_DEBUG      
-	    printf("fpn: 2\n");
+#ifdef ALARMS_DEBUG
+	    printf("fpn: after end date\n");
 #endif
+	    forward=0;
 	 }
       }
       t_temp-=adv;
@@ -984,99 +1197,12 @@ static int find_prev_next(struct Appointment *a,
 	 printf("fpn: prev_found\n");
 #endif
       }
-      /* increment or decrement time */
       if (forward) {
-	 switch (a->repeatType) {
-	  case repeatNone:
-	    break;
-	  case repeatDaily:
-	    break;
-	  case repeatWeekly:
-	    for (count=0, dow=t.tm_wday; count<14; count++) {
-	       add_days_to_date(&t, 1);
-#ifdef ALARMS_DEBUG      
-	       printf("fpn: weekly forward t.tm_wday=%d, freq=%d\n", t.tm_wday, freq);
-#endif
-	       dow++;
-	       if (dow==7) {
-#ifdef ALARMS_DEBUG      
-		  printf("fpn: dow==7\n");
-#endif
-		  add_days_to_date(&t, (freq-1)*7);
-		  dow=0;
-	       }
-	       if (a->repeatDays[dow]) {
-#ifdef ALARMS_DEBUG      
-		  printf("fpn: repeatDay[dow] dow=%d\n", dow);
-#endif
-		  break;
-	       }
-	    }
-	    break;
-	  case repeatMonthlyByDay:
-	    add_months_to_date(&t, freq);
-	    get_month_info(t.tm_mon, 1, t.tm_year, &fdom, &ndim);
-	    t.tm_mday=((a->repeatDay+7-fdom)%7) - ((a->repeatDay)%7) + a->repeatDay + 1;
-	    if (t.tm_mday > ndim-1) {
-	       t.tm_mday -= 7;
-	    }
-	    break;
-	  case repeatMonthlyByDate:
-	    t.tm_mday=a->begin.tm_mday;
-	    add_months_to_date(&t, freq);
-	    break;
-	  case repeatYearly:
-	    t.tm_mday=a->begin.tm_mday;
-	    add_years_to_date(&t, freq);
-	    break;
-	 }/*switch */
+	 forward_backward_in_appt_time(a, &t, fdom, ndim, 1);
 	 continue;
       }
       if (backward) {
-	 switch (a->repeatType) {
-	  case repeatNone:
-	    break;
-	  case repeatDaily:
-	    break;
-	  case repeatWeekly:
-	    for (count=0, dow=t.tm_wday; count<14; count++) {
-	       sub_days_from_date(&t, 1);
-#ifdef ALARMS_DEBUG      
-	       printf("fpn: weekly backward t.tm_wday=%d, freq=%d\n", t.tm_wday, freq);
-#endif
-	       dow--;
-	       if (dow==-1) {
-#ifdef ALARMS_DEBUG      
-		  printf("fpn: dow==-1\n");
-#endif
-		  sub_days_from_date(&t, (freq-1)*7);
-		  dow=6;
-	       }
-	       if (a->repeatDays[dow]) {
-#ifdef ALARMS_DEBUG      
-		  printf("fpn: repeatDay[dow] dow=%d\n", dow);
-#endif
-		  break;
-	       }
-	    }
-	    break;
-	  case repeatMonthlyByDay:
-	    sub_months_from_date(&t, freq);
-	    get_month_info(t.tm_mon, 1, t.tm_year, &fdom, &ndim);
-	    t.tm_mday=((a->repeatDay+7-fdom)%7) - ((a->repeatDay)%7) + a->repeatDay + 1;
-	    if (t.tm_mday > ndim-1) {
-	       t.tm_mday -= 7;
-	    }
-	    break;
-	  case repeatMonthlyByDate:
-	    t.tm_mday=a->begin.tm_mday;
-	    sub_months_from_date(&t, freq);
-	    break;
-	  case repeatYearly:
-	    t.tm_mday=a->begin.tm_mday;
-	    sub_years_from_date(&t, freq);
-	    break;
-	 }/*switch */
+	 forward_backward_in_appt_time(a, &t, fdom, ndim, -1);
 	 continue;
       }
    }
@@ -1110,7 +1236,7 @@ int alarms_find_next(struct tm *date1_in, struct tm *date2_in, int soonest_only)
    int prev_found, next_found;
    int add_a_next;
 
-   jpilot_logf(LOG_DEBUG, "alarms_find_next()\n");
+   jp_logf(JP_LOG_DEBUG, "alarms_find_next()\n");
 
    if (glob_skip_all_alarms) return 0;
 
@@ -1120,7 +1246,7 @@ int alarms_find_next(struct tm *date1_in, struct tm *date2_in, int soonest_only)
    } else {
       tm_temp=date1_in;
    }
-   bzero(&date1, sizeof(date1));
+   memset(&date1, 0, sizeof(date1));
    date1.tm_year=tm_temp->tm_year;
    date1.tm_mon=tm_temp->tm_mon;
    date1.tm_mday=tm_temp->tm_mday;
@@ -1135,7 +1261,7 @@ int alarms_find_next(struct tm *date1_in, struct tm *date2_in, int soonest_only)
    } else {
       tm_temp=date2_in;
    }
-   bzero(&date2, sizeof(date2));
+   memset(&date2, 0, sizeof(date2));
    date2.tm_year=tm_temp->tm_year;
    date2.tm_mon=tm_temp->tm_mon;
    date2.tm_mday=tm_temp->tm_mday;
@@ -1144,19 +1270,19 @@ int alarms_find_next(struct tm *date1_in, struct tm *date2_in, int soonest_only)
    date2.tm_sec=tm_temp->tm_sec;
    date2.tm_isdst=tm_temp->tm_isdst;
 
-   t1=mktime(&date1);
-   t2=mktime(&date2);
+   t1=mktime_dst_adj(&date1);
+   t2=mktime_dst_adj(&date2);
 
 #ifdef ALARMS_DEBUG
      {
 	char str[100];
 	struct tm *Pnow;
-	strftime(str, 80, "%B %d, %Y %H:%M", &date1);
+	strftime(str, sizeof(str), "%B %d, %Y %H:%M", &date1);
 	printf("date1=%s\n", str);
-	strftime(str, 80, "%B %d, %Y %H:%M", &date2);
+	strftime(str, sizeof(str), "%B %d, %Y %H:%M", &date2);
 	printf("date2=%s\n", str);
 	Pnow = localtime(&t1);
-	strftime(str, 80, "%B %d, %Y %H:%M", Pnow);
+	strftime(str, sizeof(str), "%B %d, %Y %H:%M", Pnow);
 	printf("^date1=%s\n", str);
      }
 #endif
@@ -1168,7 +1294,7 @@ int alarms_find_next(struct tm *date1_in, struct tm *date2_in, int soonest_only)
    }
 
    a_list=NULL;
-   get_days_appointments2(&a_list, NULL, 0, 0, 1);
+   get_days_appointments2(&a_list, NULL, 0, 0, 1, NULL);
 
    t_soonest=0;
 
@@ -1187,7 +1313,7 @@ int alarms_find_next(struct tm *date1_in, struct tm *date2_in, int soonest_only)
        * and is a non repeating appointment
        */
       if (temp_al->ma.a.repeatType == repeatNone) {
-	 t_alarm = mktime(&(temp_al->ma.a.begin));
+	 t_alarm = mktime_dst_adj(&(temp_al->ma.a.begin));
 	 if (t_alarm < t1) {
 #ifdef ALARMS_DEBUG      
 	    printf("non repeat before t1, t_alarm<t1, %ld<%ld\n",t_alarm,t1);
@@ -1198,10 +1324,10 @@ int alarms_find_next(struct tm *date1_in, struct tm *date2_in, int soonest_only)
 
       /* If the appointment has an end date, see that we are not past it */
       if (!(temp_al->ma.a.repeatForever)) {
-	 t_end = mktime(&(temp_al->ma.a.repeatEnd));
+	 t_end = mktime_dst_adj(&(temp_al->ma.a.repeatEnd));
 	 /* We need to add 24 hours to the end date to make it inclusive */
 	 t_end += 86400;
-	 t_begin = mktime(&(temp_al->ma.a.begin));
+	 t_begin = mktime_dst_adj(&(temp_al->ma.a.begin));
 	 if (t_end < t2) {
 #ifdef ALARMS_DEBUG      
 	    printf("past end date\n");
@@ -1248,8 +1374,8 @@ int alarms_find_next(struct tm *date1_in, struct tm *date2_in, int soonest_only)
 		     &tm_next,
 		     &prev_found,
 		     &next_found);
-      t_prev=mktime(&tm_prev);
-      t_future=mktime(&tm_next);
+      t_prev=mktime_dst_adj(&tm_prev);
+      t_future=mktime_dst_adj(&tm_next);
 
 #ifdef ALARMS_DEBUG      
       printf("adv = %ld\n", adv);
@@ -1274,7 +1400,7 @@ int alarms_find_next(struct tm *date1_in, struct tm *date2_in, int soonest_only)
       }
       if (next_found) {
 	 if (!(temp_al->ma.a.repeatForever)) {
-	    t_end = mktime(&(temp_al->ma.a.repeatEnd));
+	    t_end = mktime_dst_adj(&(temp_al->ma.a.repeatEnd));
 	    if (t_future > t_end) {
 #ifdef ALARMS_DEBUG      
 	       printf("failed future is after t_end\n");
@@ -1348,7 +1474,7 @@ int alarms_init(unsigned char skip_past_alarms,
    int found_uptodate;
    int year, mon, day, hour, min, n;
 
-   jpilot_logf(LOG_DEBUG, "alarms_init()\n");
+   jp_logf(JP_LOG_DEBUG, "alarms_init()\n");
 
    alarm_list=NULL;
    Plast_alarm_list=NULL;
@@ -1366,29 +1492,29 @@ int alarms_init(unsigned char skip_past_alarms,
    }
 
    found_uptodate=0;
-   in=jp_open_home_file("jpilot.alarms", "r");
+   in=jp_open_home_file(EPN".alarms", "r");
    if (!in) {
-      jpilot_logf(LOG_WARN, "Could not open jpilot.alarms file\n");
+      jp_logf(JP_LOG_WARN, "Could not open "EPN".alarms file\n");
       return -1;
    }
 
    while (!feof(in)) {
       line[0]='\0';
-      fgets(line, 255, in);
-      line[255] = '\0';
+      fgets(line, sizeof(line)-1, in);
+      line[sizeof(line)-1] = '\0';
       if (line[0]=='#') continue;
       if (!strncmp(line, "UPTODATE ", 9)) {
 	 n = sscanf(line+9, "%d %d %d %d %d\n", &year, &mon, &day, &hour, &min);
 	 if (n==5) {
 	    found_uptodate=1;
 	 }
-	 jpilot_logf(LOG_DEBUG, "UPTODATE %d %d %d %d %d\n", year, mon, day, hour, min);
+	 jp_logf(JP_LOG_DEBUG, "UPTODATE %d %d %d %d %d\n", year, mon, day, hour, min);
       }
    }
 
    time(&ltime);
    Pnow = localtime(&ltime);
-   bzero(&now, sizeof(now));
+   memset(&now, 0, sizeof(now));
    now.tm_year=Pnow->tm_year;
    now.tm_mon=Pnow->tm_mon;
    now.tm_mday=Pnow->tm_mday;
@@ -1406,7 +1532,7 @@ int alarms_init(unsigned char skip_past_alarms,
       min = now.tm_min;
    }
 
-   bzero(&tm1, sizeof(tm1));
+   memset(&tm1, 0, sizeof(tm1));
    tm1.tm_year=year-1900;
    tm1.tm_mon=mon-1;
    tm1.tm_mday=day;
