@@ -1,7 +1,7 @@
 /* dat.c
  * A module of J-Pilot http://jpilot.org
  * 
- * Copyright (C) 2001 by Judd Montgomery
+ * Copyright (C) 2001-2002 by Judd Montgomery
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,6 +35,7 @@
 #endif
 #include "log.h"
 #include "utils.h"
+#include "i18n.h"
 
 #define JPILOT_DEBUGno
 
@@ -107,7 +108,7 @@ static int get_CString(FILE *in, char **PStr)
 
 static int get_categories(FILE *in, struct CategoryAppInfo *ai)
 {
-   char str_long[4];
+   unsigned char str_long[4];
    char *PStr;
    long count;
    int i;
@@ -165,7 +166,7 @@ int get_repeat(FILE *in, struct Appointment *a)
 #endif
    if (a) {
       a->exception=NULL;
-      bzero(&(a->repeatEnd), sizeof(struct tm));
+      memset(&(a->repeatEnd), 0, sizeof(a->repeatEnd));
    }
 
    if (a) {
@@ -173,7 +174,7 @@ int get_repeat(FILE *in, struct Appointment *a)
       if (s>0) {
 	 a->exception=malloc(sizeof(struct tm) * s);
 	 if (!(a->exceptions)) {
-	    jpilot_logf(LOG_WARN, "get_repeat(): Out of Memory\n");
+	    jp_logf(JP_LOG_WARN, "get_repeat(): Out of Memory\n");
 	 }
       }
    }
@@ -353,6 +354,7 @@ int get_repeat(FILE *in, struct Appointment *a)
 #ifdef JPILOT_DEBUG
       printf("unknown repeat type2 %d\n", l);
 #endif
+      break;
    }
 
    return 0;
@@ -368,7 +370,7 @@ static int print_date(int palm_date)
 
    t = palm_date;/* - 20828448800; */
    now = localtime(&t);
-   strftime(text, 255, "%02m/%02d/%Y %02H:%02M:%02S", now);
+   strftime(text, sizeof(text), "%02m/%02d/%Y %02H:%02M:%02S", now);
    printf("%s\n", text);
 
    return 0;
@@ -410,45 +412,43 @@ int print_field(struct field *f)
 
 int get_field(FILE *in, struct field *f)
 {
-   char str_long[4];
+   unsigned char str_long[4];
    long type;
    char *PStr;
 
    fread(str_long, 4, 1, in);
    type = x86_long(str_long);
-   f->type=0;
+   f->type=type;
    f->str=NULL;
 
    switch(type) {
     case DAT_TYPE_INTEGER:
-      f->type=type;
       fread(str_long, 4, 1, in);
       f->i = x86_long(str_long);
       break;
     case DAT_TYPE_CSTRING:
-      f->type=type;
       /* padding */
       fseek(in, 4, SEEK_CUR);
       get_CString(in, &PStr);
       f->str = PStr;
       break;
     case DAT_TYPE_BOOLEAN:
-      f->type=type;
       fread(str_long, 4, 1, in);
       f->i = x86_long(str_long);
       break;
     case DAT_TYPE_DATE:
-      f->type=type;
       fread(str_long, 4, 1, in);
       f->date = x86_long(str_long);
       break;
+    case DAT_TYPE_BITFLAG:
+      /* I currently do not know how to read this datatype */
+      break;
     case DAT_TYPE_REPEAT:
-      f->type=type;
       /* The calling function needs to call this */
       /* get_repeat(in, NULL); */
       break;
     default:
-      jpilot_logf(LOG_WARN, "get_field(): unknown type = %ld\n", type);
+      jp_logf(JP_LOG_WARN, "get_field(): unknown type = %ld\n", type);
       break;
    }
 
@@ -459,12 +459,12 @@ int dat_check_if_dat_file(FILE *in)
 {
    char version[6];
 
-   bzero(version, 4);
+   memset(version, 0, sizeof(version));
    fseek(in, 0, SEEK_SET);
    /* Version */
    fread(version, 4, 1, in);
    fseek(in, 0, SEEK_SET);
-   jpilot_logf(LOG_DEBUG, "dat_check_if_dat_file(): version = [%c%c%d%d]\n", version[3],version[2],version[1],version[0]);
+   jp_logf(JP_LOG_DEBUG, "dat_check_if_dat_file(): version = [%c%c%d%d]\n", version[3],version[2],version[1],version[0]);
    if ((version[3]=='D') && (version[2]=='B') &&
        (version[1]==1) && (version[0]==0)) {
       return DAT_DATEBOOK_FILE;
@@ -491,25 +491,25 @@ int dat_read_header(FILE *in,
 		    int *schema_count, int *field_count, long *rec_count)
 {
    int i;
-   char filler[100];
+   unsigned char filler[100];
    char version[4];
    char *PStr;
-   char str_long[4];
+   unsigned char str_long[4];
 
    fseek(in, 0, SEEK_SET);
 
    /* Version */
    fread(version, 4, 1, in);
-   jpilot_logf(LOG_DEBUG, "version = [%c%c%d%d]\n", version[3],version[2],version[1],version[0]);
+   jp_logf(JP_LOG_DEBUG, "version = [%c%c%d%d]\n", version[3],version[2],version[1],version[0]);
 
    /* Full file path name */
    get_CString(in, &PStr);
-   jpilot_logf(LOG_DEBUG, "path:[%s]\n",PStr);
+   jp_logf(JP_LOG_DEBUG, "path:[%s]\n",PStr);
    free(PStr);
 
    /* Show Header */
    get_CString(in, &PStr);
-   jpilot_logf(LOG_DEBUG, "show header:[%s]\n",PStr);
+   jp_logf(JP_LOG_DEBUG, "show header:[%s]\n",PStr);
    free(PStr);
 
    /* Next free category ID */
@@ -529,7 +529,7 @@ int dat_read_header(FILE *in,
    fread(filler, 4, 1, in);
    *field_count=x86_long(filler);
    if (*field_count != expected_field_count) {
-      jpilot_logf(LOG_WARN, "fields per row count != %d, unknown format\n",
+      jp_logf(JP_LOG_WARN, "fields per row count != %d, unknown format\n",
 		  expected_field_count);
       return -1;
    }
@@ -543,7 +543,7 @@ int dat_read_header(FILE *in,
    fread(filler, 2, 1, in);
    *field_count = x86_short(filler);
    if (*field_count != expected_field_count) {
-      jpilot_logf(LOG_WARN, "field count != %d, unknown format\n",
+      jp_logf(JP_LOG_WARN, "field count != %d, unknown format\n",
 		  expected_field_count);
       return -1;
    }
@@ -551,17 +551,17 @@ int dat_read_header(FILE *in,
    /* Schema fields */
    fread(filler, (*field_count)*2, 1, in);
    if (memcmp(filler, schema, (*field_count)*2)) {
-      jpilot_logf(LOG_WARN, "unknown format, file has wrong schema\n");
-      jpilot_logf(LOG_WARN, "File schema is:");
+      jp_logf(JP_LOG_WARN, "unknown format, file has wrong schema\n");
+      jp_logf(JP_LOG_WARN, "File schema is:");
       for (i=0; i<(*field_count)*2; i++) {
-	 jpilot_logf(LOG_WARN, " %02d", (char)filler[i]);
+	 jp_logf(JP_LOG_WARN, " %02d", (char)filler[i]);
       }
-      jpilot_logf(LOG_WARN, "\n");
-      jpilot_logf(LOG_WARN, "It should be:  ");
+      jp_logf(JP_LOG_WARN, "\n");
+      jp_logf(JP_LOG_WARN, "It should be:  ");
       for (i=0; i<(*field_count)*2; i++) {
-	 jpilot_logf(LOG_WARN, " %02d", (char)schema[i]);
+	 jp_logf(JP_LOG_WARN, " %02d", (char)schema[i]);
       }
-      jpilot_logf(LOG_WARN, "\n");
+      jp_logf(JP_LOG_WARN, "\n");
       return -1;
    }
 
@@ -590,7 +590,7 @@ int dat_get_appointments(FILE *in, AppointmentList **alist, struct CategoryAppIn
    AppointmentList *last_alist;
    time_t t;
    struct tm *now;
-   /* Should be 1, 1, 1, 3, 3, 5, 1, 5, 6, 6, 1, 6, 1, 1, 8 */
+   /* Should be 1, 1, 1, 3, 1, 5, 1, 5, 6, 6, 1, 6, 1, 1, 8 */
    char schema[30]={
       DAT_TYPE_INTEGER,0,
 	DAT_TYPE_INTEGER,0,
@@ -630,7 +630,7 @@ int dat_get_appointments(FILE *in, AppointmentList **alist, struct CategoryAppIn
    };
 #endif
 
-   jpilot_logf(LOG_DEBUG, "dat_get_appointments\n");
+   jp_logf(JP_LOG_DEBUG, "dat_get_appointments\n");
 
    if (!alist) return 0;
    *alist=NULL;
@@ -648,22 +648,15 @@ int dat_get_appointments(FILE *in, AppointmentList **alist, struct CategoryAppIn
    for (i=0; i<rec_count; i++) {
       temp_alist = malloc(sizeof(AppointmentList));
       if (!temp_alist) {
-	 jpilot_logf(LOG_WARN, "dat_get_appointments(): Out of memory\n");
+	 jp_logf(JP_LOG_WARN, "dat_get_appointments(): Out of memory\n");
 	 return i;
       }
-      if (last_alist) {
-	 last_alist->next=temp_alist;
-	 last_alist=temp_alist;
-      } else {
-	 last_alist=temp_alist;
-	 *alist=last_alist;
-      }
-      last_alist->next=NULL;
-      last_alist->app_type=DATEBOOK;
 #ifdef JPILOT_DEBUG
       printf("----- record %d -----\n", i+1);
 #endif
-      bzero(&(last_alist->ma.a), sizeof(struct Appointment));
+      memset(&(temp_alist->ma.a), 0, sizeof(temp_alist->ma.a));
+      temp_alist->next=NULL;
+      temp_alist->app_type=DATEBOOK;
 
       /* Record ID */
       /* Status Field */
@@ -674,7 +667,9 @@ int dat_get_appointments(FILE *in, AppointmentList **alist, struct CategoryAppIn
 	 printf("rec field %d %s: ", j, rec_fields[j]); print_field(&(fa[j]));
 #endif
 	 if (fa[j].type!=schema[j*2]) {
-	    jpilot_logf(LOG_WARN, "Invalid schema ID record %d, field %d\n", i+1, j+3);
+	    jp_logf(JP_LOG_WARN, "%s:%d Record %d, field %d: Invalid type.  Expected %d, found %d\n", __FILE__, __LINE__, i+1, j+3, schema[j*2], fa[j].type);
+	    jp_logf(JP_LOG_WARN, _("read of file terminated\n"));
+	    free(temp_alist);
 	    return 0;
 	 }
       }
@@ -690,56 +685,67 @@ int dat_get_appointments(FILE *in, AppointmentList **alist, struct CategoryAppIn
 	 }
 #endif
 	 if (fa[j].type!=schema[j*2+6]) {
-	    jpilot_logf(LOG_WARN, "Invalid schema ID record %d, field %d\n", i+1, j+3);
+	    jp_logf(JP_LOG_WARN, "%s:%d Record %d, field %d: Invalid type.  Expected %d, found %d\n", __FILE__, __LINE__, i+1, j+3, schema[j*2+6], fa[j].type);
+	    jp_logf(JP_LOG_WARN, _("read of file terminated\n"));
+	    free(temp_alist);
 	    return 0;
 	 }
 	 if (fa[j].type==DAT_TYPE_REPEAT) {
-	    get_repeat(in, &(last_alist->ma.a));
+	    get_repeat(in, &(temp_alist->ma.a));
 	 }
       }
       /* Start Time */
       t = fa[0].date;
       now = localtime(&t);
-      memcpy(&(last_alist->ma.a.begin), now, sizeof(struct tm));
+      memcpy(&(temp_alist->ma.a.begin), now, sizeof(struct tm));
       /* End Time */
       t = fa[1].i;
       now = localtime(&t);
-      memcpy(&(last_alist->ma.a.end), now, sizeof(struct tm));
+      memcpy(&(temp_alist->ma.a.end), now, sizeof(struct tm));
       /* Description */
       if (fa[2].str) {
-	 last_alist->ma.a.description=fa[2].str;
+	 temp_alist->ma.a.description=fa[2].str;
       } else {
-	 last_alist->ma.a.description=strdup("");
+	 temp_alist->ma.a.description=strdup("");
       }
       /* Duration */
       /* what is duration? (repeatForever?) */
       /* Note */
       if (fa[4].str) {
-	 last_alist->ma.a.note=fa[4].str;
+	 temp_alist->ma.a.note=fa[4].str;
       } else {
-	 last_alist->ma.a.note=strdup("");
+	 temp_alist->ma.a.note=strdup("");
       }
       /* Untimed */
-      last_alist->ma.a.event=fa[5].i;
+      temp_alist->ma.a.event=fa[5].i;
       /* Private */
-      last_alist->ma.attrib = 0;
+      temp_alist->ma.attrib = 0;
       if (fa[6].i) {
-	 last_alist->ma.attrib |= DAT_STATUS_PRIVATE;
+	 temp_alist->ma.attrib |= DAT_STATUS_PRIVATE;
       }
       /* Category */
-      last_alist->ma.unique_id = fa[7].i;
-      if (last_alist->ma.unique_id > 15) {
-	 last_alist->ma.unique_id = 15;
+      temp_alist->ma.unique_id = fa[7].i;
+      if (temp_alist->ma.unique_id > 15) {
+	 temp_alist->ma.unique_id = 15;
       }
-      if (last_alist->ma.unique_id < 0) {
-	 last_alist->ma.unique_id = 0;
+      if (temp_alist->ma.unique_id < 0) {
+	 temp_alist->ma.unique_id = 0;
       }	 
       /* Alarm Set */
-      last_alist->ma.a.alarm=fa[8].i;
+      temp_alist->ma.a.alarm=fa[8].i;
       /* Alarm Advance Units */
-      last_alist->ma.a.advance=fa[9].i;
+      temp_alist->ma.a.advance=fa[9].i;
       /* Alarm Advance Type */
-      last_alist->ma.a.advanceUnits=fa[10].i;
+      temp_alist->ma.a.advanceUnits=fa[10].i;
+
+      /* Append onto the end of the list */
+      if (last_alist) {
+	 last_alist->next=temp_alist;
+	 last_alist=temp_alist;
+      } else {
+	 last_alist=temp_alist;
+	 *alist=last_alist;
+      }
    }
    return 0;
 }
@@ -826,7 +832,7 @@ int dat_get_addresses(FILE *in, AddressList **addrlist, struct CategoryAppInfo *
    };
 #endif
 
-   jpilot_logf(LOG_DEBUG, "dat_get_addresses\n");
+   jp_logf(JP_LOG_DEBUG, "dat_get_addresses\n");
 
    if (!addrlist) return 0;
    *addrlist=NULL;
@@ -844,18 +850,11 @@ int dat_get_addresses(FILE *in, AddressList **addrlist, struct CategoryAppInfo *
    for (i=0; i<rec_count; i++) {
       temp_addrlist = malloc(sizeof(AddressList));
       if (!temp_addrlist) {
-	 jpilot_logf(LOG_WARN, "dat_get_addresses(): Out of memory\n");
+	 jp_logf(JP_LOG_WARN, "dat_get_addresses(): Out of memory\n");
 	 return i;
       }
-      if (last_addrlist) {
-	 last_addrlist->next=temp_addrlist;
-	 last_addrlist=temp_addrlist;
-      } else {
-	 last_addrlist=temp_addrlist;
-	 *addrlist=last_addrlist;
-      }
-      last_addrlist->next=NULL;
-      last_addrlist->app_type=ADDRESS;
+      temp_addrlist->next=NULL;
+      temp_addrlist->app_type=ADDRESS;
 #ifdef JPILOT_DEBUG
       printf("----- record %d -----\n", i+1);
 #endif
@@ -868,7 +867,9 @@ int dat_get_addresses(FILE *in, AddressList **addrlist, struct CategoryAppInfo *
 	 printf("rec field %d %s: ", j, rec_fields[j]); print_field(&(fa[j]));
 #endif
 	 if (fa[j].type!=schema[j*2]) {
-	    jpilot_logf(LOG_WARN, "Invalid schema ID record %d, field %d\n", i+1, j+3);
+	    jp_logf(JP_LOG_WARN, "%s:%d Record %d, field %d: Invalid type.  Expected %d, found %d\n", __FILE__, __LINE__, i+1, j+3, schema[j*2], fa[j].type);
+	    jp_logf(JP_LOG_WARN, _("read of file terminated\n"));
+	    free(temp_addrlist);
 	    return 0;
 	 }
       }
@@ -879,38 +880,48 @@ int dat_get_addresses(FILE *in, AddressList **addrlist, struct CategoryAppInfo *
 	 printf("field %d %s: ", j, field_names[j]); print_field(&(fa[j]));
 #endif
 	 if (fa[j].type!=schema[j*2+6]) {
-	    jpilot_logf(LOG_WARN, "Invalid schema ID record %d, field %d\n", i+1, j+3);
+	    jp_logf(JP_LOG_WARN, "%s:%d Record %d, field %d: Invalid type.  Expected %d, found %d\n", __FILE__, __LINE__, i+1, j+3, schema[j*2+6], fa[j].type);
+	    jp_logf(JP_LOG_WARN, _("read of file terminated\n"));
+	    free(temp_addrlist);
 	    return 0;
 	 }
       }
       for (k=0; k<19; k++) {
-	 last_addrlist->ma.a.entry[k]=fa[dat_order[k]].str;
+	 temp_addrlist->ma.a.entry[k]=fa[dat_order[k]].str;
       }
-      last_addrlist->ma.a.phoneLabel[0] = fa[4].i;
-      last_addrlist->ma.a.phoneLabel[1] = fa[6].i;
-      last_addrlist->ma.a.phoneLabel[2] = fa[8].i;
-      last_addrlist->ma.a.phoneLabel[3] = fa[10].i;
-      last_addrlist->ma.a.phoneLabel[4] = fa[12].i;
+      temp_addrlist->ma.a.phoneLabel[0] = fa[4].i;
+      temp_addrlist->ma.a.phoneLabel[1] = fa[6].i;
+      temp_addrlist->ma.a.phoneLabel[2] = fa[8].i;
+      temp_addrlist->ma.a.phoneLabel[3] = fa[10].i;
+      temp_addrlist->ma.a.phoneLabel[4] = fa[12].i;
       /* Private */
-      last_addrlist->ma.attrib = 0;
+      temp_addrlist->ma.attrib = 0;
       if (fa[20].i) {
-	 last_addrlist->ma.attrib |= DAT_STATUS_PRIVATE;
+	 temp_addrlist->ma.attrib |= DAT_STATUS_PRIVATE;
       }
       /* Category */
-      last_addrlist->ma.unique_id = fa[21].i;
-      if (last_addrlist->ma.unique_id > 15) {
-	 last_addrlist->ma.unique_id = 15;
+      temp_addrlist->ma.unique_id = fa[21].i;
+      if (temp_addrlist->ma.unique_id > 15) {
+	 temp_addrlist->ma.unique_id = 15;
       }
-      if (last_addrlist->ma.unique_id < 0) {
-	 last_addrlist->ma.unique_id = 0;
+      if (temp_addrlist->ma.unique_id < 0) {
+	 temp_addrlist->ma.unique_id = 0;
       }	 
       /* Show phone in list */
-      last_addrlist->ma.a.showPhone = fa[26].i - 1;
+      temp_addrlist->ma.a.showPhone = fa[26].i - 1;
       for (k=0; k<19; k++) {
-	 if (last_addrlist->ma.a.entry[k]==NULL) {
-	    last_addrlist->ma.a.entry[k]=strdup("");
+	 if (temp_addrlist->ma.a.entry[k]==NULL) {
+	    temp_addrlist->ma.a.entry[k]=strdup("");
 	 }
       }
+      /* Append onto the end of the list */
+      if (temp_addrlist) {
+	 last_addrlist->next=temp_addrlist;
+	 last_addrlist=temp_addrlist;
+      } else {
+	 last_addrlist=temp_addrlist;
+	 *addrlist=last_addrlist;
+      }
    }
    return 0;
 }
@@ -955,7 +966,7 @@ int dat_get_todos(FILE *in, ToDoList **todolist, struct CategoryAppInfo *ai)
    };
 #endif
 
-   jpilot_logf(LOG_DEBUG, "dat_get_todos\n");
+   jp_logf(JP_LOG_DEBUG, "dat_get_todos\n");
 
    if (!todolist) return 0;
    *todolist=NULL;
@@ -973,18 +984,13 @@ int dat_get_todos(FILE *in, ToDoList **todolist, struct CategoryAppInfo *ai)
    for (i=0; i<rec_count; i++) {
       temp_todolist = malloc(sizeof(ToDoList));
       if (!temp_todolist) {
-	 jpilot_logf(LOG_WARN, "dat_get_todos(): Out of memory\n");
+	 jp_logf(JP_LOG_WARN, "dat_get_todos(): Out of memory\n");
+	 jp_logf(JP_LOG_WARN, _("read of file terminated\n"));
+	 free(temp_todolist);
 	 return i;
       }
-      if (last_todolist) {
-	 last_todolist->next=temp_todolist;
-	 last_todolist=temp_todolist;
-      } else {
-	 last_todolist=temp_todolist;
-	 *todolist=last_todolist;
-      }
-      last_todolist->next=NULL;
-      last_todolist->app_type=TODO;
+      temp_todolist->next=NULL;
+      temp_todolist->app_type=TODO;
 #ifdef JPILOT_DEBUG
       printf("----- record %d -----\n", i+1);
 #endif
@@ -997,7 +1003,9 @@ int dat_get_todos(FILE *in, ToDoList **todolist, struct CategoryAppInfo *ai)
 	 printf("rec field %d %s: ", j, rec_fields[j]); print_field(&(fa[j]));
 #endif
 	 if (fa[j].type!=schema[j*2]) {
-	    jpilot_logf(LOG_WARN, "Invalid schema ID record %d, field %d\n", i+1, j+3);
+	    jp_logf(JP_LOG_WARN, "%s:%d Record %d, field %d: Invalid type.  Expected %d, found %d\n", __FILE__, __LINE__, i+1, j+3, schema[j*2], fa[j].type);
+	    jp_logf(JP_LOG_WARN, _("read of file terminated\n"));
+	    free(temp_todolist);
 	    return 0;
 	 }
       }
@@ -1008,36 +1016,38 @@ int dat_get_todos(FILE *in, ToDoList **todolist, struct CategoryAppInfo *ai)
 	 printf("field %d %s: ", j, field_names[j]); print_field(&(fa[j]));
 #endif
 	 if (fa[j].type!=schema[j*2+6]) {
-	    jpilot_logf(LOG_WARN, "Invalid schema ID record %d, field %d\n", i+1, j+3);
+	    jp_logf(JP_LOG_WARN, "%s:%d Record %d, field %d: Invalid type.  Expected %d, found %d\n", __FILE__, __LINE__, i+1, j+3, schema[j*2+6], fa[j].type);
+	    jp_logf(JP_LOG_WARN, _("read of file terminated\n"));
+	    free(temp_todolist);
 	    return 0;
 	 }
       }
       /* Description */
       if (fa[0].str) {
-	 last_todolist->mtodo.todo.description=fa[0].str;
+	 temp_todolist->mtodo.todo.description=fa[0].str;
       } else {
-	 last_todolist->mtodo.todo.description=strdup("");
+	 temp_todolist->mtodo.todo.description=strdup("");
       }
       /* Due Date */
       if (fa[1].date==0x749E77BF) {
-	 last_todolist->mtodo.todo.indefinite=1;
-	 bzero(&(last_todolist->mtodo.todo.due), sizeof(struct tm));
+	 temp_todolist->mtodo.todo.indefinite=1;
+	 memset(&(temp_todolist->mtodo.todo.due), 0, sizeof(temp_todolist->mtodo.todo.due));
       } else {
 	 t = fa[1].date;
 	 now = localtime(&t);
-	 memcpy(&(last_todolist->mtodo.todo.due), now, sizeof(struct tm));
-	 last_todolist->mtodo.todo.indefinite=0;
+	 memcpy(&(temp_todolist->mtodo.todo.due), now, sizeof(struct tm));
+	 temp_todolist->mtodo.todo.indefinite=0;
       }
       /* Completed */
-      last_todolist->mtodo.todo.complete = (fa[2].i==0) ? 0 : 1;
+      temp_todolist->mtodo.todo.complete = (fa[2].i==0) ? 0 : 1;
       /* Priority */
       if (fa[3].i < 0) fa[3].i=0;
       if (fa[3].i > 5) fa[3].i=5;
-      last_todolist->mtodo.todo.priority = fa[3].i;
+      temp_todolist->mtodo.todo.priority = fa[3].i;
       /* Private */
-      last_todolist->mtodo.attrib = 0;
+      temp_todolist->mtodo.attrib = 0;
       if (fa[4].i) {
-	 last_todolist->mtodo.attrib |= DAT_STATUS_PRIVATE;
+	 temp_todolist->mtodo.attrib |= DAT_STATUS_PRIVATE;
       }
       /* Category */
       /* Normally the category would go into 4 bits of the attrib.
@@ -1046,12 +1056,20 @@ int dat_get_todos(FILE *in, ToDoList **todolist, struct CategoryAppInfo *ai)
        */
       if (fa[5].i < 0) fa[5].i=0;
       if (fa[5].i > 15) fa[5].i=15;
-      last_todolist->mtodo.unique_id = fa[5].i;
+      temp_todolist->mtodo.unique_id = fa[5].i;
       /* Note */
       if (fa[6].str) {
-	 last_todolist->mtodo.todo.note=fa[6].str;
+	 temp_todolist->mtodo.todo.note=fa[6].str;
       } else {
-	 last_todolist->mtodo.todo.note=strdup("");
+	 temp_todolist->mtodo.todo.note=strdup("");
+      }
+      /* Append onto the end of the list */
+      if (last_todolist) {
+	 last_todolist->next=temp_todolist;
+	 last_todolist=temp_todolist;
+      } else {
+	 last_todolist=temp_todolist;
+	 *todolist=last_todolist;
       }
    }
 
@@ -1087,7 +1105,7 @@ int dat_get_memos(FILE *in, MemoList **memolist, struct CategoryAppInfo *ai)
    };
 #endif
 
-   jpilot_logf(LOG_DEBUG, "dat_get_memos\n");
+   jp_logf(JP_LOG_DEBUG, "dat_get_memos\n");
 
    if (!memolist) return 0;
    *memolist=NULL;
@@ -1105,18 +1123,11 @@ int dat_get_memos(FILE *in, MemoList **memolist, struct CategoryAppInfo *ai)
    for (i=0; i<rec_count; i++) {
       temp_memolist = malloc(sizeof(MemoList));
       if (!temp_memolist) {
-	 jpilot_logf(LOG_WARN, "dat_get_memos(): Out of memory\n");
+	 jp_logf(JP_LOG_WARN, "dat_get_memos(): Out of memory\n");
 	 return i;
       }
-      if (last_memolist) {
-	 last_memolist->next=temp_memolist;
-	 last_memolist=temp_memolist;
-      } else {
-	 last_memolist=temp_memolist;
-	 *memolist=last_memolist;
-      }
-      last_memolist->next=NULL;
-      last_memolist->app_type=MEMO;
+      temp_memolist->next=NULL;
+      temp_memolist->app_type=MEMO;
 #ifdef JPILOT_DEBUG
       printf("----- record %d -----\n", i+1);
 #endif
@@ -1129,7 +1140,9 @@ int dat_get_memos(FILE *in, MemoList **memolist, struct CategoryAppInfo *ai)
 	 printf("rec field %d %s: ", j, rec_fields[j]); print_field(&(fa[j]));
 #endif
 	 if (fa[j].type!=schema[j*2]) {
-	    jpilot_logf(LOG_WARN, "Invalid schema ID record %d, field %d\n", i+1, j+3);
+	    jp_logf(JP_LOG_WARN, "%s:%d Record %d, field %d: Invalid type.  Expected %d, found %d\n", __FILE__, __LINE__, i+1, j+3, schema[j*2], fa[j].type);
+	    jp_logf(JP_LOG_WARN, _("read of file terminated\n"));
+	    free(temp_memolist);
 	    return 0;
 	 }
       }
@@ -1140,16 +1153,18 @@ int dat_get_memos(FILE *in, MemoList **memolist, struct CategoryAppInfo *ai)
 	 printf("field %d %s: ", j, field_names[j]); print_field(&(fa[j]));
 #endif
 	 if (fa[j].type!=schema[j*2+6]) {
-	    jpilot_logf(LOG_WARN, "Invalid schema ID record %d, field %d\n", i+1, j+3);
+	    jp_logf(JP_LOG_WARN, "%s:%d Record %d, field %d: Invalid type.  Expected %d, found %d\n", __FILE__, __LINE__, i+1, j+3, schema[j*2+6], fa[j].type);
+	    jp_logf(JP_LOG_WARN, _("read of file terminated\n"));
+	    free(temp_memolist);
 	    return 0;
 	 }
       }
       /* Memo */
-      last_memolist->mmemo.memo.text=fa[0].str;
+      temp_memolist->mmemo.memo.text=fa[0].str;
       /* Private */
-      last_memolist->mmemo.attrib = 0;
+      temp_memolist->mmemo.attrib = 0;
       if (fa[1].i) {
-	 last_memolist->mmemo.attrib |= DAT_STATUS_PRIVATE;
+	 temp_memolist->mmemo.attrib |= DAT_STATUS_PRIVATE;
       }
       /* Category */
       /* Normally the category would go into 4 bits of the attrib.
@@ -1158,7 +1173,16 @@ int dat_get_memos(FILE *in, MemoList **memolist, struct CategoryAppInfo *ai)
        */
       if (fa[2].i < 0) fa[2].i=0;
       if (fa[2].i > 15) fa[2].i=15;
-      last_memolist->mmemo.unique_id = fa[2].i;
+      temp_memolist->mmemo.unique_id = fa[2].i;
+      /* Append onto the end of the list */
+
+      if (last_memolist) {
+	 last_memolist->next=temp_memolist;
+	 last_memolist=temp_memolist;
+      } else {
+	 last_memolist=temp_memolist;
+	 *memolist=last_memolist;
+      }
    }
 
    return 0;
