@@ -1,7 +1,7 @@
 /* datebook.c
  * A module of J-Pilot http://jpilot.org
  *
- * Copyright (C) 1999-2001 by Judd Montgomery
+ * Copyright (C) 1999-2002 by Judd Montgomery
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -59,6 +59,10 @@ int datebook_compare(const void *v1, const void *v2)
    a1=&((*al1)->ma.a);
    a2=&((*al2)->ma.a);
 
+   if ((a1->event) || (a2->event)) {
+      return a2->event-a1->event;
+   }
+
    /* Jim Rees pointed out my sorting error */
    /* return ((a1->begin.tm_hour*60 + a1->begin.tm_min) > */
    return ((a1->begin.tm_hour*60 + a1->begin.tm_min) -
@@ -84,7 +88,7 @@ static int datebook_sort(AppointmentList **al)
    /* Allocate an array to be qsorted */
    sort_al = calloc(count, sizeof(AppointmentList *));
    if (!sort_al) {
-      jpilot_logf(LOG_WARN, "datebook_sort(): Out of Memory\n");
+      jp_logf(JP_LOG_WARN, "datebook_sort(): Out of Memory\n");
       return 0;
    }
 
@@ -109,7 +113,7 @@ static int datebook_sort(AppointmentList **al)
    return 0;
 }
 
-#ifdef USE_DB3
+#ifdef ENABLE_DATEBK
 int db3_hack_date(struct Appointment *a, struct tm *today)
 {
    int t1, t2;
@@ -160,70 +164,188 @@ int db3_hack_date(struct Appointment *a, struct tm *today)
    return 0;
 }
 
-/* Returns a bitmask
- * 0 if not a floating OR
- * bitmask:
- *  1 if float,
- *  2 if completed float
- *  16 if float has a note
- */
-int db3_is_float(struct Appointment *a, int *category)
+/* Note should be pretty much validated by now */
+void db3_fill_struct(char *note, int type, struct db4_struct *db4)
 {
-   int len, mask=0;
+   /* jp_logf(JP_LOG_WARN, "db3_fill_struct()\n"); */
+   switch (note[2]) {
+    case 'c':
+      db4->floating_event=DB3_FLOAT_COMPLETE;
+      break;
+    case 'd':
+      db4->floating_event=DB3_FLOAT_DONE;
+      break;
+    case 'f':
+      db4->floating_event=DB3_FLOAT;
+      break;
+   }
+   if (type==DB3_TAG_TYPE_DBplus) {
+      return;
+   }
+   switch (note[3]) {
+    case 'b':
+      db4->custom_font=DB3_FONT_BOLD;
+      break;
+    case 'l':
+      db4->custom_font=DB3_FONT_LARGE;
+      break;
+    case 'L':
+      db4->custom_font=DB3_FONT_LARGE_BOLD;
+      break;
+   }
+   db4->category = (note[4] - '@') & 0x0F;
+   db4->icon = (note[5] - '@');
+   if (note[6] == 's') {
+      db4->spans_midnight = DB3_SPAN_MID_YES;
+   }
+   /* bytes 8,9 I don't understand yet */
+   if (type==DB3_TAG_TYPE_DB3) {
+      return;
+   }
+   if (note[9] == 'l') {
+      db4->link = DB3_LINK_YES;
+   }
+   /* bytes 11-14 I don't understand yet */
+   /* bytes 15-18 lower 6 bits make 24 bit numer (not sure of byte order) */
+   db4->custom_alarm_sound = ((note[14] & 0x3F) << 18) +
+     ((note[15] & 0x3F) << 12) +
+     ((note[16] & 0x3F) << 6) +
+     ((note[17] & 0x3F) << 0);
+   db4->color = (note[18] - '@') & 0x0F;
+   /* Byte 19 is a carriage return */
+}
 
-   *category=0;
-   if (!a->note) {
+/*
+ * Parses the note tag and looks for db3 or db4 tags.
+ * returns -1: error, 0: false, 1: true
+ * for true: will in db4
+ * 
+ * db4 can be passed in NULL for just checking for datebk tags.
+ */
+int db3_parse_tag(char *note, int *type, struct db4_struct *db4)
+{
+   /* Characters allowed to exist in each character of a db3/4 tag */
+   /* NULL means any character is allowed */
+   char *allowed[]={
+      "#", "#", /* First 2 characters are # */
+	"@fcd", /* f floating, c completed, d done */
+	"@blL", /* b bold, l large, L large bold */
+	"@ABCDEFGHIJKLMNO", /* Category */
+	"@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_abcdefghijklmnopqrst", /* icon */
+	"@s", /* Spans midnight */ 
+	NULL, /* Lower 2 digits of next 2 chars are time zone */
+	NULL,
+	"@l\n", /* l link, EOL for datebk3 tags */
+	NULL, /* Lower 6 bits of next 4 chars make a 24 bit number (advance) */
+	NULL, /* I don't understand this yet and have not coded it */
+	NULL,
+	NULL,
+	NULL, /* Lower 6 bits of next 4 chars make a 24 bit number */
+	NULL, /*  which is the custom sound */
+	NULL,
+	NULL,
+	"@ABCDEFGHIJKLMNO", /* Color */
+	"\n", /* EOL for datebk4 tags */
+   };
+   int len, i;
+
+   /* We need to guarantee these to be set upon return */
+   if (db4) {
+      memset(db4, 0, sizeof(*db4));
+   }
+   *type=DB3_TAG_TYPE_NONE;
+
+   if (!note) {
       return 0;
    }
-   len = strlen(a->note);
-   if (len > 8) {
-      if ((a->note[0]=='#') && (a->note[1]=='#')) {
-	 *category = a->note[4] - '@';
-	 if (len > 10) {
-	    mask=mask | DB3_FLOAT_HAS_NOTE;
+
+   len = strlen(note);
+
+   for (i=0; i<20; i++) {
+      if (i+1>len) {
+	 return 0;
+      }
+      if ((i==3) && (note[i]=='\n')) {
+	 /* If we made it this far and CR then its a db+ tag */
+	 *type=DB3_TAG_TYPE_DBplus;
+	 if (note[i+1]) {
+	    if (db4) {
+	       db4->note=&(note[i+1]);
+	    }
 	 }
-	 if (a->note[2]=='f') {
-	    mask=mask | DB3_FLOAT;
-	    return mask;
+	 if (db4) {
+	    db3_fill_struct(note, DB3_TAG_TYPE_DBplus, db4);
 	 }
-	 if (a->note[2]=='c') {
-	    mask=mask | DB3_FLOAT_COMPLETE;
-	    return mask;
+	 return 1;
+      }
+      if ((i==9) && (note[i]=='\n')) {
+	 /* If we made it this far and CR then its a db3 tag */
+	 *type=DB3_TAG_TYPE_DB3;
+	 if (note[i+1]) {
+	    if (db4) {
+	       db4->note=&(note[i+1]);
+	    }
 	 }
-      } else if (a->note[0] != '\0') {
-	 mask=mask | DB3_FLOAT_HAS_NOTE;
+	 if (db4) {
+	    db3_fill_struct(note, DB3_TAG_TYPE_DB3, db4);
+	 }
+	 return 1;
+      }
+      if (allowed[i]==NULL) continue;
+      if (!strchr(allowed[i], note[i])) {
+	 return 0;	 
       }
    }
-   return mask;
+   
+   /* If we made it this far then its a db4 tag */
+   *type=DB3_TAG_TYPE_DB4;
+   if (note[i]) {
+      if (db4) {
+	 db4->note=&(note[i]);
+      }
+   }
+   if (db4) {
+      db3_fill_struct(note, DB3_TAG_TYPE_DB4, db4);
+   }
+   return 1;
 }
 #endif
 
-int pc_datebook_write(struct Appointment *a, PCRecType rt, unsigned char attrib)
+int pc_datebook_write(struct Appointment *a, PCRecType rt,
+		      unsigned char attrib, unsigned int *unique_id)
 {
-   char record[65536];
+   unsigned char record[65536];
    int rec_len;
    buf_rec br;
    long char_set;
 
    get_pref(PREF_CHAR_SET, &char_set, NULL);
-   if (char_set != CHAR_SET_ENGLISH) {
-      if (a->description) charset_j2p(a->description, strlen(a->description)+1, char_set);
-      if (a->note) charset_j2p(a->note, strlen(a->note)+1, char_set);
+   if (char_set != CHAR_SET_LATIN1) {
+      if (a->description) charset_j2p((unsigned char *)a->description, strlen(a->description)+1, char_set);
+      if (a->note) charset_j2p((unsigned char *)a->note, strlen(a->note)+1, char_set);
    }
 
    rec_len = pack_Appointment(a, record, 65535);
    if (!rec_len) {
       PRINT_FILE_LINE;
-      jpilot_logf(LOG_WARN, "pack_Appointment %s\n", _("error"));
+      jp_logf(JP_LOG_WARN, "pack_Appointment %s\n", _("error"));
       return -1;
    }
    br.rt=rt;
    br.attrib = attrib;
    br.buf = record;
    br.size = rec_len;
+   /* Keep unique ID intact */
+   if ((unique_id) && (*unique_id!=0)) {
+      br.unique_id = *unique_id;
+   } else {
+      br.unique_id = 0;
+   }
 
    jp_pc_write("DatebookDB", &br);
-   /* *unique_id = br.unique_id;*/
+   if (unique_id) {
+      *unique_id = br.unique_id;
+   }
 
    return 0;
 }
@@ -247,14 +369,14 @@ int datebook_copy_appointment(struct Appointment *a1,
 {
    *a2=malloc(sizeof(struct Appointment));
    if (!(*a2)) {
-      jpilot_logf(LOG_WARN, "datebook_copy_appointment(): Out of memory\n");
+      jp_logf(JP_LOG_WARN, "datebook_copy_appointment(): Out of memory\n");
       return -1;
    }
    memcpy(*a2, a1, sizeof(struct Appointment));
 
    (*a2)->exception = (struct tm *)malloc(a1->exceptions * sizeof(struct tm));
    if (!(*a2)->exception) {
-      jpilot_logf(LOG_WARN, "datebook_copy_appointment(): Out of memory 2\n");
+      jp_logf(JP_LOG_WARN, "datebook_copy_appointment(): Out of memory 2\n");
       return -1;
    }
    memcpy((*a2)->exception, a1->exception, a1->exceptions * sizeof(struct tm));
@@ -284,7 +406,7 @@ int datebook_add_exception(struct Appointment *a, int year, int mon, int day)
 
    new_exception = malloc((a->exceptions + 1) * sizeof(struct tm));
    if (!new_exception) {
-      jpilot_logf(LOG_WARN, "datebook_add_exception(): Out of memory\n");
+      jp_logf(JP_LOG_WARN, "datebook_add_exception(): Out of memory\n");
       return -1;
    }
    memcpy(new_exception, a->exception, (a->exceptions) * sizeof(struct tm));
@@ -383,7 +505,7 @@ unsigned int isApptOnDate(struct Appointment *a, struct tm *date)
    static int days, begin_days;
    static struct tm cached_date;
 
-   /* jpilot_logf(LOG_DEBUG, "isApptOnDate\n"); */
+   /* jp_logf(JP_LOG_DEBUG, "isApptOnDate\n"); */
 
    ret = FALSE;
 
@@ -532,7 +654,7 @@ unsigned int isApptOnDate(struct Appointment *a, struct tm *date)
       }   
       break;
     default:
-      jpilot_logf(LOG_WARN, "unknown repeatType (%d) found in DatebookDB\n",
+      jp_logf(JP_LOG_WARN, "unknown repeatType (%d) found in DatebookDB\n",
 	   a->repeatType);
       ret = FALSE;
    }/*switch */
@@ -541,11 +663,11 @@ unsigned int isApptOnDate(struct Appointment *a, struct tm *date)
       /* Check for exceptions */
       for (i=0; i<a->exceptions; i++) {
 #ifdef JPILOT_DEBUG
-	 jpilot_logf(LOG_DEBUG, "exception %d mon %d\n", i, a->exception[i].tm_mon);
-	 jpilot_logf(LOG_DEBUG, "exception %d day %d\n", i, a->exception[i].tm_mday);
-	 jpilot_logf(LOG_DEBUG, "exception %d year %d\n", i, a->exception[i].tm_year);
-	 jpilot_logf(LOG_DEBUG, "exception %d yday %d\n", i, a->exception[i].tm_yday);
-	 jpilot_logf(LOG_DEBUG, "today is yday %d\n", date->tm_yday);
+	 jp_logf(JP_LOG_DEBUG, "exception %d mon %d\n", i, a->exception[i].tm_mon);
+	 jp_logf(JP_LOG_DEBUG, "exception %d day %d\n", i, a->exception[i].tm_mday);
+	 jp_logf(JP_LOG_DEBUG, "exception %d year %d\n", i, a->exception[i].tm_year);
+	 jp_logf(JP_LOG_DEBUG, "exception %d yday %d\n", i, a->exception[i].tm_yday);
+	 jp_logf(JP_LOG_DEBUG, "today is yday %d\n", date->tm_yday);
 #endif
 	 exception_days = dateToDays(&(a->exception[i]));
 	 if (!days) {
@@ -564,32 +686,37 @@ unsigned int isApptOnDate(struct Appointment *a, struct tm *date)
 int get_datebook_app_info(struct AppointmentAppInfo *ai)
 {
    int num,i;
-   unsigned int rec_size;
+   int rec_size;
    unsigned char *buf;
    long char_set;
 
-   bzero(ai, sizeof(*ai));
+   memset(ai, 0, sizeof(*ai));
+   /* Put at least one entry in there */
+   strcpy(ai->category.name[0], "Unfiled");
 
    jp_get_app_info("DatebookDB", &buf, &rec_size);
    num = unpack_AppointmentAppInfo(ai, buf, rec_size);
    if (buf) {
       free(buf);
    }
-   if (num <= 0) {
-      jpilot_logf(LOG_WARN, _("Error reading"), "DatebookDB.pdb");
+   if ((num<0) || (rec_size<=0)) {
+      jp_logf(JP_LOG_WARN, _("Error reading %s\n"), "DatebookDB.pdb");
       return -1;
    }
    get_pref(PREF_CHAR_SET, &char_set, NULL);
-   if (char_set != CHAR_SET_ENGLISH) {
+   if (char_set != CHAR_SET_LATIN1) {
       for (i = 0; i < 16; i++) {
-	 if (ai->category.name[i][0] != '\0') charset_p2j(ai->category.name[i], 16, char_set);
+	 if (ai->category.name[i][0] != '\0') {
+	    charset_p2j((unsigned char *)ai->category.name[i], 16, char_set);
+	 }
       }
    }
 
    return 0;
 }
 
-int weed_datebook_list(AppointmentList **al, int mon, int year, int *mask)
+int weed_datebook_list(AppointmentList **al, int mon, int year,
+		       int skip_privates, int *mask)
 {
    struct tm tm_fdom;
    struct tm tm_ldom;
@@ -600,7 +727,7 @@ int weed_datebook_list(AppointmentList **al, int mon, int year, int *mask)
    int ret;
    int trash_it;
 
-   bzero(&tm_fdom, sizeof(tm_fdom));
+   memset(&tm_fdom, 0, sizeof(tm_fdom));
    tm_fdom.tm_hour=11;
    tm_fdom.tm_mday=1;
    tm_fdom.tm_mon=mon;
@@ -625,6 +752,10 @@ int weed_datebook_list(AppointmentList **al, int mon, int year, int *mask)
     * search though it ~30 times.
     */
    for (prev_al=NULL, tal=*al; tal; tal = next_al) {
+      if (skip_privates && (tal->ma.attrib & dlpRecAttrSecret)) {
+	 next_al=tal->next;
+	 continue;
+      }
       trash_it=0;
       /* See if the appointment starts after the last day of the month */
       r = compareTimesToDay(&(tal->ma.a.begin), &tm_ldom);
@@ -742,21 +873,29 @@ int appointment_on_day_list(int mon, int year, int *mask)
    AppointmentList *tal, *al;
    int dow, ndim, num;
    int bit;
+   int show_priv;
+   int skip_privates;
 
-   bzero(&tm_dom, sizeof(tm_dom));
+   memset(&tm_dom, 0, sizeof(tm_dom));
    tm_dom.tm_hour=11;
    tm_dom.tm_mday=1;
    tm_dom.tm_mon=mon;
    tm_dom.tm_year=year;
 
    al = NULL;
-   num = get_days_appointments2(&al, NULL, 2, 2, 2);
+   /* Get private records back
+    * We want to highlight a day with a private record if we are showing or
+    * masking private records */
+   num = get_days_appointments2(&al, NULL, 2, 2, 1, NULL);
+
+   show_priv = show_privates(GET_PRIVATES);
+   skip_privates = (show_priv==HIDE_PRIVATES);
 
    get_month_info(mon, 1, year, &dow, &ndim);
 
    *mask = 0;
 
-   weed_datebook_list(&al, mon, year, mask);
+   weed_datebook_list(&al, mon, year, skip_privates, mask);
 
    for (tm_dom.tm_mday=1, bit=1; tm_dom.tm_mday<=ndim; tm_dom.tm_mday++, bit=bit<<1) {
       if (*mask & bit) {
@@ -765,6 +904,7 @@ int appointment_on_day_list(int mon, int year, int *mask)
       mktime(&tm_dom);
 
       for (tal=al; tal; tal = tal->next) {
+	 if (skip_privates && (tal->ma.attrib & dlpRecAttrSecret)) continue;
 	 if (isApptOnDate(&(tal->ma.a), &tm_dom)) {
 	    *mask = *mask | bit;
 	    break;
@@ -776,16 +916,18 @@ int appointment_on_day_list(int mon, int year, int *mask)
    return 0;
 }
 
-int get_days_appointments(AppointmentList **appointment_list, struct tm *now)
+int get_days_appointments(AppointmentList **appointment_list, struct tm *now,
+			  int *total_records)
 {
-   return get_days_appointments2(appointment_list, now, 1, 1, 1);
+   return get_days_appointments2(appointment_list, now, 1, 1, 1, total_records);
 }
 /*
  * If Null is passed in for date, then all appointments will be returned
  * modified, deleted and private, 0 for no, 1 for yes, 2 for use prefs
  */
 int get_days_appointments2(AppointmentList **appointment_list, struct tm *now,
-			   int modified, int deleted, int privates)
+			   int modified, int deleted, int privates,
+			   int *total_records)
 {
    GList *records;
    GList *temp_list;
@@ -796,13 +938,15 @@ int get_days_appointments2(AppointmentList **appointment_list, struct tm *now,
    int keep_priv;
    buf_rec *br;
    long char_set;
-#ifdef USE_DB3
+   char *buf;
+   size_t len;
+#ifdef ENABLE_DATEBK
    long use_db3_tags;
    time_t ltime;
    struct tm *Ptoday, today;
 #endif
 
-#ifdef USE_DB3
+#ifdef ENABLE_DATEBK
    time(&ltime);
    Ptoday = localtime(&ltime);
    /* Copy into stable memory */
@@ -810,7 +954,7 @@ int get_days_appointments2(AppointmentList **appointment_list, struct tm *now,
    get_pref(PREF_USE_DB3, &use_db3_tags, NULL);
 #endif
 
-   jpilot_logf(LOG_DEBUG, "get_days_appointments()\n");
+   jp_logf(JP_LOG_DEBUG, "get_days_appointments()\n");
 
    if (modified==2) {
       get_pref(PREF_SHOW_MODIFIED, &keep_modified, NULL);
@@ -823,7 +967,7 @@ int get_days_appointments2(AppointmentList **appointment_list, struct tm *now,
       keep_deleted = deleted;
    }
    if (privates==2) {
-      keep_priv = show_privates(GET_PRIVATES, NULL);
+      keep_priv = show_privates(GET_PRIVATES);
    } else {
       keep_priv = privates;
    }
@@ -832,6 +976,7 @@ int get_days_appointments2(AppointmentList **appointment_list, struct tm *now,
    recs_returned = 0;
 
    num = jp_read_DB_files("DatebookDB", &records);
+   if (total_records) *total_records = num;
    /* Go to first entry in the list */
    for (temp_list = records; temp_list; temp_list = temp_list->prev) {
       records = temp_list;
@@ -864,7 +1009,7 @@ int get_days_appointments2(AppointmentList **appointment_list, struct tm *now,
 	 continue;
       }
 
-#ifdef USE_DB3
+#ifdef ENABLE_DATEBK
       if (use_db3_tags) {
 	 db3_hack_date(&a, &today);
       }
@@ -877,12 +1022,37 @@ int get_days_appointments2(AppointmentList **appointment_list, struct tm *now,
       }
 
       get_pref(PREF_CHAR_SET, &char_set, NULL);
-      if (a.description) charset_p2j(a.description, strlen(a.description)+1, char_set);
-      if (a.note) charset_p2j(a.note, strlen(a.note)+1, char_set);
+
+      if (a.description) {
+	 if ((buf = (char *)malloc((len = strlen(a.description)*2+1))) != NULL) {
+	    multibyte_safe_strncpy(buf, a.description, len);
+	    charset_p2j((unsigned char *)buf, len, char_set);
+	    if (strlen(buf) > strlen(a.description)) {
+	       free(a.description);
+	       a.description = strdup(buf);
+	    } else {
+	       multibyte_safe_strncpy(a.description, buf, strlen(a.description)+1);
+	    }
+	    free(buf);
+	 }
+      }
+      if (a.note) {
+	 if ((buf = (char *) malloc((len = strlen(a.note)*2+1))) != NULL) {
+	    multibyte_safe_strncpy(buf, a.note, len);
+	    charset_p2j((unsigned char *)buf, len, char_set);
+	    if (strlen(buf) > strlen(a.note)) {
+	       free(a.note);
+	       a.note = strdup(buf);
+	    } else {
+	       multibyte_safe_strncpy(a.note, buf, strlen(a.note)+1);
+	    }
+	    free(buf);
+	 }
+      }
 
       temp_a_list = malloc(sizeof(AppointmentList));
       if (!temp_a_list) {
-	 jpilot_logf(LOG_WARN, "get_days_appointments(): Out of memory\n");
+	 jp_logf(JP_LOG_WARN, "get_days_appointments(): Out of memory\n");
 	 free_Appointment(&a);
 	 break;
       }
@@ -900,7 +1070,7 @@ int get_days_appointments2(AppointmentList **appointment_list, struct tm *now,
 
    datebook_sort(appointment_list);
 
-   jpilot_logf(LOG_DEBUG, "Leaving get_days_appointments()\n");
+   jp_logf(JP_LOG_DEBUG, "Leaving get_days_appointments()\n");
 
    return recs_returned;
 }
