@@ -1,7 +1,7 @@
 /* datebook.h
  * A module of J-Pilot http://jpilot.org
  * 
- * Copyright (C) 1999-2001 by Judd Montgomery
+ * Copyright (C) 1999-2002 by Judd Montgomery
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,32 +23,79 @@
 #include <pi-datebook.h>
 #include "utils.h"
 
-#ifdef USE_DB3
-#define DB3_FLOAT           1
-#define DB3_FLOAT_COMPLETE  2
-#define DB3_FLOAT_HAS_NOTE 16
+#ifdef ENABLE_DATEBK
+
+/* These defines depend on the defaults being zero so that the structure
+   being zeroed out sets the defaults (0) */
+# define DB3_TAG_TYPE_NONE   0
+# define DB3_TAG_TYPE_DBplus 1
+# define DB3_TAG_TYPE_DB3    3
+# define DB3_TAG_TYPE_DB4    4
+
+# define DB3_REGULAR         0
+# define DB3_FLOAT           1
+# define DB3_FLOAT_COMPLETE  2
+# define DB3_FLOAT_DONE      3
+
+# define DB3_FONT_NONE       0
+# define DB3_FONT_BOLD       1
+# define DB3_FONT_LARGE      2
+# define DB3_FONT_LARGE_BOLD 3
+
+# define DB3_SPAN_MID_NO     0
+# define DB3_SPAN_MID_YES    1
+
+# define DB3_LINK_NO         0
+# define DB3_LINK_YES        1
+
+struct db4_struct {
+   int floating_event;
+   /* The following are db3 and greater only tags */
+   int custom_font;
+   int category;
+   int icon;
+   int spans_midnight;
+   int time_zone;
+   /* The following are db4 and greater only tags */
+   int link;
+   int advance; /* I don't understand this and next field yet */
+   int last_date; /* This may need to be a struct tm (not sure yet) */
+   int custom_alarm_sound;
+   int color;
+   /* This isn't datebk specific */
+   char *note; /* points to where the note starts after the tag (or null) */
+};
 #endif
 
 int datebook_print(int type);
 int datebook_import(GtkWidget *window);
 int datebook_cleanup();
-int pc_datebook_write(struct Appointment *a, PCRecType rt, unsigned char attrib);
+int pc_datebook_write(struct Appointment *a, PCRecType rt,
+		      unsigned char attrib, unsigned int *unique_id);
 void free_AppointmentList(AppointmentList **al);
 
 /*
- * If Null is passed in for date, then all appointments will be returned
+ * If Null is passed in for date, then all appointments will be returned.
+ * total_records returns total records in database (can pass in NULL).
+ * returns number of records in list.
  */
-int get_days_appointments(AppointmentList **al_out, struct tm *date);
+int get_days_appointments(AppointmentList **al_out, struct tm *date,
+			  int *total_records);
 /*
  * If Null is passed in for date, then all appointments will be returned
- * modified, deleted and private, 0 for no, 1 for yes, 2 for use prefs
+ * modified, deleted and private, 0 for no, 1 for yes, 2 for use prefs.
+ * total_records returns total records in database (can pass in NULL).
+ * returns number of records in list.
  */
 int get_days_appointments2(AppointmentList **appointment_list, struct tm *now,
-			   int modified, int deleted, int privates);
+			   int modified, int deleted, int privates,
+			   int *total_records);
 
 /* This funtion removes appointments from the list that obviously will not
- * occur in this month */
-int weed_datebook_list(AppointmentList **al, int mon, int year, int *mask);
+ * occur in this month.  Skip privates will ignore private records.
+ */
+int weed_datebook_list(AppointmentList **al, int mon, int year,
+		       int skip_privates, int *mask);
 
 /* Year is years since 1900 */
 /* Mon is 0-11 */
@@ -75,7 +122,7 @@ int dateToDays(struct tm *tm1);
 
 int compareTimesToDay(struct tm *tm1, struct tm *tm2);
 
-#ifdef USE_DB3
+#ifdef ENABLE_DATEBK
 /* Returns a bitmask
  * 0 if not a floating OR
  * bitmask:
@@ -83,7 +130,9 @@ int compareTimesToDay(struct tm *tm1, struct tm *tm2);
  *  2 if completed float
  *  16 if float has a note
  */
-int db3_is_float(struct Appointment *a, int *category);
+int db3_parse_tag(char *note, int *type, struct db4_struct *db4);
+int db3_has_tags(struct Appointment *a);
+int db3_which_icon(struct  Appointment *a);
 #endif
 
 int datebook_import(GtkWidget *window);
