@@ -1,6 +1,7 @@
 /* i18n.h
+ * A module of J-Pilot http://jpilot.org
  * 
- * Copyright (C) 1999 by Judd Montgomery
+ * Copyright (C) 1999-2002 by Judd Montgomery
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,5 +35,8 @@
 #ifndef gettext_noop
 #define	gettext_noop(str) str
 #endif
+#ifndef N_
+#define N_(str) str
+#endif
 
 #endif
