@@ -1,7 +1,7 @@
 /* import_gui.c
  * A module of J-Pilot http://jpilot.org
  *
- * Copyright (C) 1999-2001 by Judd Montgomery
+ * Copyright (C) 1999-2002 by Judd Montgomery
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,6 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
+
 #include "config.h"
 #include "i18n.h"
 #include <gtk/gtk.h>
@@ -33,7 +34,7 @@ static int line_selected;
 static GtkWidget *filew=NULL;
 static GtkWidget *import_record_ask_window=NULL;
 static int glob_import_record_ask_button_pressed;
-int (*glob_import_callback)(GtkWidget *parent_window, char *file_path, int type);
+int (*glob_import_callback)(GtkWidget *parent_window, const char *file_path, int type);
 int glob_type_selected;
 
 
@@ -93,7 +94,11 @@ int read_csv_field(FILE *in, char *text, int size, int new_line)
 	 }
       }
       text[n++]=c;
-      if (n+1>size) return n;
+      if (n+1>=size)
+      {
+	  text[n++]='\0';
+	  return n;
+      }
    }
    text[n++]='\0';
    return n;
@@ -116,7 +121,7 @@ char *str_type(int type)
    return "?";
 }
 
-int guess_file_type(char *path)
+int guess_file_type(const char *path)
 {
    FILE *in;
    char text[256];
@@ -149,11 +154,11 @@ static void
 cb_quit(GtkWidget *widget,
 	gpointer   data)
 {
-   char *sel;
+   const char *sel;
    char dir[MAX_PREF_VALUE+2];
    int i;
 
-   jpilot_logf(LOG_DEBUG, "Quit\n");
+   jp_logf(JP_LOG_DEBUG, "Quit\n");
 
    sel = gtk_file_selection_get_filename(GTK_FILE_SELECTION(filew));
    strncpy(dir, sel, MAX_PREF_VALUE);
@@ -169,7 +174,7 @@ cb_quit(GtkWidget *widget,
       }
    }
 
-   set_pref(PREF_MEMO_IMPORT_PATH, 0, dir);
+   set_pref(PREF_MEMO_IMPORT_PATH, 0, dir, TRUE);
 
    filew = NULL;
    gtk_widget_destroy(data);
@@ -186,20 +191,20 @@ static void
 cb_import(GtkWidget *widget,
 	  gpointer   filesel)
 {
-   char *sel;
+   const char *sel;
    struct stat statb;
 
-   jpilot_logf(LOG_DEBUG, "cb_import\n");
+   jp_logf(JP_LOG_DEBUG, "cb_import\n");
    sel = gtk_file_selection_get_filename(GTK_FILE_SELECTION(filesel));
-   jpilot_logf(LOG_DEBUG, "file selected [%s]\n", sel);
+   jp_logf(JP_LOG_DEBUG, "file selected [%s]\n", sel);
 
    /*Check to see if its a regular file */
    if (stat(sel, &statb)) {
-      jpilot_logf(LOG_DEBUG, "File selected was not stat-able\n");
+      jp_logf(JP_LOG_DEBUG, "File selected was not stat-able\n");
       return;
    }
    if (!S_ISREG(statb.st_mode)) {
-      jpilot_logf(LOG_DEBUG, "File selected was not a regular file\n");
+      jp_logf(JP_LOG_DEBUG, "File selected was not a regular file\n");
       return;
    }
    glob_import_callback(filesel, sel, glob_type_selected);
@@ -211,21 +216,21 @@ static void cb_import_select_row(GtkWidget *file_clist,
 				 GdkEventButton *bevent,
 				 gpointer data)
 {
-   char *sel;
+   const char *sel;
    struct stat statb;
    int guessed_type;
    int i;
 
-   jpilot_logf(LOG_DEBUG, "cb_import_select_row\n");
+   jp_logf(JP_LOG_DEBUG, "cb_import_select_row\n");
    sel = gtk_file_selection_get_filename(GTK_FILE_SELECTION(filew));
 
    /*Check to see if its a regular file */
    if (stat(sel, &statb)) {
-      jpilot_logf(LOG_DEBUG, "File selected was not stat-able\n");
+      jp_logf(JP_LOG_DEBUG, "File selected was not stat-able\n");
       return;
    }
    if (!S_ISREG(statb.st_mode)) {
-      jpilot_logf(LOG_DEBUG, "File selected was not a regular file\n");
+      jp_logf(JP_LOG_DEBUG, "File selected was not a regular file\n");
       return;
    }
 
@@ -244,7 +249,7 @@ static void cb_import_select_row(GtkWidget *file_clist,
 void import_gui(GtkWidget *main_window, GtkWidget *main_pane,
 		char *type_desc[], int type_int[],
 		int (*import_callback)(GtkWidget *parent_window,
-				       char *file_path, int type))
+				       const char *file_path, int type))
 {
    GtkWidget *button;
    GtkWidget *vbox, *hbox;
@@ -264,19 +269,26 @@ void import_gui(GtkWidget *main_window, GtkWidget *main_pane,
    gdk_window_get_size(main_window->window, &pw, &ph);
    gdk_window_get_root_origin(main_window->window, &px, &py);
 
+#ifdef ENABLE_GTK2
+   pw = gtk_paned_get_position(GTK_PANED(main_pane));
+#else
    pw = GTK_PANED(main_pane)->handle_xpos;
+#endif
    px+=40;
 
-   g_snprintf(temp, 255, "%s %s", PN, _("Import"));
-   temp[255]='\0';
+   g_snprintf(temp, sizeof(temp), "%s %s", PN, _("Import"));
 
    filew = gtk_widget_new(GTK_TYPE_FILE_SELECTION,
-			  "type", GTK_WINDOW_DIALOG,
-			  "x", px, "y", py,
-			  "width", pw, "height", ph,
+			  "type", GTK_WINDOW_TOPLEVEL,
 			  "title", temp,
 			  NULL);
 
+   gtk_window_set_default_size(GTK_WINDOW(filew), pw, ph);
+   gtk_widget_set_uposition(filew, px, py);
+
+   gtk_window_set_modal(GTK_WINDOW(filew), TRUE);
+   gtk_window_set_transient_for(GTK_WINDOW(filew), GTK_WINDOW(main_window));
+
    get_pref(PREF_MEMO_IMPORT_PATH, NULL, &svalue);
    if (svalue && svalue[0]) {
       gtk_file_selection_set_filename(GTK_FILE_SELECTION(filew), svalue);
@@ -341,8 +353,13 @@ void import_gui(GtkWidget *main_window, GtkWidget *main_pane,
 
    /* This callback is for a file guess algorithm and to pre-push
     * the type buttons */
+#ifdef ENABLE_GTK2
+   gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filew)->file_list),
+		      "cursor_changed", GTK_SIGNAL_FUNC(cb_import_select_row), NULL);
+#else
    gtk_signal_connect(GTK_OBJECT(GTK_CLIST(GTK_FILE_SELECTION(filew)->file_list)),
 		      "select_row", GTK_SIGNAL_FUNC(cb_import_select_row), NULL);
+#endif
 
    gtk_widget_show_all(vbox);
 
@@ -379,16 +396,19 @@ int import_record_ask(GtkWidget *main_window, GtkWidget *pane,
    GtkWidget *vbox;
    GtkWidget *temp_hbox;
    GtkWidget *textw;
+#ifdef ENABLE_GTK2
+   GObject   *textw_buffer;
+#endif
    GtkWidget *label;
+#ifdef ENABLE_GTK2
+   GtkWidget *scrolled_window;
+#else
    GtkWidget *vscrollbar;
+#endif
    int pw, ph;
    gint px, py;
    char str[100];
 
-   if (import_record_ask_window) {
-      return 0;
-   }
-
    /* There is no support yet for changing the suggested category */
    /* Like a menu for selecting cat to be imported into, etc. */
    *new_cat_num = suggested_cat_num;
@@ -399,15 +419,22 @@ int import_record_ask(GtkWidget *main_window, GtkWidget *pane,
 
    gdk_window_get_root_origin(main_window->window, &px, &py);
 
+#ifdef ENABLE_GTK2
+   pw = gtk_paned_get_position(GTK_PANED(pane));
+#else
    pw = GTK_PANED(pane)->handle_xpos;
+#endif
    px+=40;
 
    import_record_ask_window = gtk_widget_new(GTK_TYPE_WINDOW,
-					     "type", GTK_WINDOW_DIALOG,
-					     "x", px, "y", py,
-					     "width", pw, "height", ph,
+					     "type", GTK_WINDOW_TOPLEVEL,
 					     "title", _("Import"),
 					     NULL);
+   gtk_window_set_default_size(GTK_WINDOW(import_record_ask_window), pw, ph);
+   gtk_widget_set_uposition(GTK_WIDGET(import_record_ask_window), px, py);
+
+   gtk_window_set_modal(GTK_WINDOW(import_record_ask_window), TRUE);
+   gtk_window_set_transient_for(GTK_WINDOW(import_record_ask_window), GTK_WINDOW(main_window));
 
    gtk_container_set_border_width(GTK_CONTAINER(import_record_ask_window), 5);
 
@@ -420,22 +447,22 @@ int import_record_ask(GtkWidget *main_window, GtkWidget *pane,
 
    /* Private */
    if (priv) {
-      g_snprintf(str, 50, _("Record was marked as private"));
+      g_snprintf(str, sizeof(str), _("Record was marked as private"));
    } else {
-      g_snprintf(str, 50, _("Record was not marked as private"));
+      g_snprintf(str, sizeof(str), _("Record was not marked as private"));
    }
    label = gtk_label_new(str);
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
    gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
 
 
-   g_snprintf(str, 60, "Category before import was: [%s]", old_cat_name);
+   g_snprintf(str, sizeof(str), "Category before import was: [%s]", old_cat_name);
    label = gtk_label_new(str);
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
    gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
 
 
-   g_snprintf(str, 60, "Record will be put in category [%s]",
+   g_snprintf(str, sizeof(str), "Record will be put in category [%s]",
 	      cai->name[suggested_cat_num]);
    label = gtk_label_new(str);
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
@@ -446,15 +473,34 @@ int import_record_ask(GtkWidget *main_window, GtkWidget *pane,
    temp_hbox = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(vbox), temp_hbox, TRUE, TRUE, 0);
 
+#ifdef ENABLE_GTK2
+   textw = gtk_text_view_new();
+   textw_buffer = G_OBJECT(gtk_text_view_get_buffer(GTK_TEXT_VIEW(textw)));
+   gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(textw), FALSE);
+   gtk_text_view_set_editable(GTK_TEXT_VIEW(textw), FALSE);
+   gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(textw), GTK_WRAP_WORD);
+
+   scrolled_window = gtk_scrolled_window_new(NULL, NULL);
+   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
+				  GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
+   gtk_container_set_border_width(GTK_CONTAINER(scrolled_window), 1);
+   gtk_container_add(GTK_CONTAINER(scrolled_window), textw);
+   gtk_box_pack_start_defaults(GTK_BOX(temp_hbox), scrolled_window);
+#else
    textw = gtk_text_new(NULL, NULL);
    gtk_text_set_editable(GTK_TEXT(textw), FALSE);
    gtk_text_set_word_wrap(GTK_TEXT(textw), TRUE);
    vscrollbar = gtk_vscrollbar_new(GTK_TEXT(textw)->vadj);
    gtk_box_pack_start(GTK_BOX(temp_hbox), textw, TRUE, TRUE, 0);
    gtk_box_pack_start(GTK_BOX(temp_hbox), vscrollbar, FALSE, FALSE, 0);
+#endif
 
    if (text) {
+#ifdef ENABLE_GTK2
+      gtk_text_buffer_set_text(GTK_TEXT_BUFFER(textw_buffer), text, -1);
+#else
       gtk_text_insert(GTK_TEXT(textw), NULL, NULL, NULL, text, -1);
+#endif
    }
 
    /* Box for buttons  */
@@ -491,8 +537,6 @@ int import_record_ask(GtkWidget *main_window, GtkWidget *pane,
 
    gtk_widget_show_all(import_record_ask_window);
 
-   gtk_window_set_modal(GTK_WINDOW(import_record_ask_window), TRUE);
-
    gtk_main();
 
    return glob_import_record_ask_button_pressed;
