@@ -1,7 +1,7 @@
 /* install_gui.c
  * A module of J-Pilot http://jpilot.org
  *
- * Copyright (C) 1999-2001 by Judd Montgomery
+ * Copyright (C) 1999-2002 by Judd Montgomery
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 #include <sys/stat.h>
 #include <unistd.h>
 #include "utils.h"
+#include "prefs.h"
 #include "log.h"
 
 
@@ -44,16 +45,16 @@ int install_remove_line(int deleted_line)
    char *Pc;
    int r, line_count;
 
-   in = jp_open_home_file("jpilot_to_install", "r");
+   in = jp_open_home_file(EPN"_to_install", "r");
    if (!in) {
-      jpilot_logf(LOG_DEBUG, "failed opening install_file\n");
+      jp_logf(JP_LOG_DEBUG, "failed opening install_file\n");
       return -1;
    }
 
-   out = jp_open_home_file("jpilot_to_install.tmp", "w");
+   out = jp_open_home_file(EPN"_to_install.tmp", "w");
    if (!out) {
       fclose(in);
-      jpilot_logf(LOG_DEBUG, "failed opening install_file.tmp\n");
+      jp_logf(JP_LOG_DEBUG, "failed opening install_file.tmp\n");
       return -1;
    }
 
@@ -74,17 +75,17 @@ int install_remove_line(int deleted_line)
    fclose(in);
    fclose(out);
 
-   rename_file("jpilot_to_install.tmp", "jpilot_to_install");
+   rename_file(EPN"_to_install.tmp", EPN"_to_install");
 
    return 0;
 }
 
-int install_append_line(char *line)
+int install_append_line(const char *line)
 {
    FILE *out;
    int r;
 
-   out = jp_open_home_file("jpilot_to_install", "a");
+   out = jp_open_home_file(EPN"_to_install", "a");
    if (!out) {
       return -1;
    }
@@ -105,15 +106,40 @@ int install_append_line(char *line)
 static gboolean cb_destroy(GtkWidget *widget)
 {
    filew = NULL;
-   return FALSE;
+
+   gtk_main_quit();
+
+   return TRUE;
 }
 
 static void
   cb_quit(GtkWidget *widget,
 	   gpointer   data)
 {
-   jpilot_logf(LOG_DEBUG, "Quit\n");
+   const char *sel;
+   char dir[MAX_PREF_VALUE+2];
+   int i;
+
+   jp_logf(JP_LOG_DEBUG, "Quit\n");
+
+   sel = gtk_file_selection_get_filename(GTK_FILE_SELECTION(data));
+   strncpy(dir, sel, MAX_PREF_VALUE);
+   dir[MAX_PREF_VALUE]='\0';
+   i=strlen(dir)-1;
+   if (i<0) i=0;
+   if (dir[i]!='/') {
+      for (i=strlen(dir); i>=0; i--) {
+	 if (dir[i]=='/') {
+	    dir[i+1]='\0';
+	    break;
+	 }
+      }
+   }
+
+   set_pref(PREF_INSTALL_PATH, 0, dir, TRUE);
+
    filew = NULL;
+
    gtk_widget_destroy(data);
 }
 
@@ -121,20 +147,20 @@ static void
   cb_add(GtkWidget *widget,
 	 gpointer   data)
 {
-   char *sel;
+   const char *sel;
    struct stat statb;
 
-   jpilot_logf(LOG_DEBUG, "Add\n");
+   jp_logf(JP_LOG_DEBUG, "Add\n");
    sel = gtk_file_selection_get_filename(GTK_FILE_SELECTION(data));
-   jpilot_logf(LOG_DEBUG, "file selected [%s]\n", sel);
+   jp_logf(JP_LOG_DEBUG, "file selected [%s]\n", sel);
 
    /*Check to see if its a regular file */
    if (stat(sel, &statb)) {
-      jpilot_logf(LOG_DEBUG, "File selected was not stat-able\n");
+      jp_logf(JP_LOG_DEBUG, "File selected was not stat-able\n");
       return;
    }
    if (!S_ISREG(statb.st_mode)) {
-      jpilot_logf(LOG_DEBUG, "File selected was not a regular file\n");
+      jp_logf(JP_LOG_DEBUG, "File selected was not a regular file\n");
       return;
    }
 
@@ -149,7 +175,7 @@ static void
    if (line_selected < 0) {
       return;
    }
-   jpilot_logf(LOG_DEBUG, "Remove line %d\n", line_selected);
+   jp_logf(JP_LOG_DEBUG, "Remove line %d\n", line_selected);
    install_remove_line(line_selected);
    update_clist();
 }
@@ -163,13 +189,14 @@ static int
    char *new_line[2];
    int kept_line_selected;
    int count;
+   int len;
 
    new_line[0]=line;
    new_line[1]=NULL;
 
    kept_line_selected = line_selected;
 
-   in = jp_open_home_file("jpilot_to_install", "r");
+   in = jp_open_home_file(EPN"_to_install", "r");
    if (!in) {
       return -1;
    }
@@ -178,11 +205,14 @@ static int
    gtk_clist_clear(GTK_CLIST(clist));
 
    for (count=0; (!feof(in)); count++) {
-       line[0]='\0';
-       Pc = fgets(line, 1000, in);
-       if (!Pc) {
-	    break;
-       }
+      line[0]='\0';
+      Pc = fgets(line, 1000, in);
+      if (!Pc) {
+	 break;
+      }
+      len=strlen(line);
+      if ((line[len-1]=='\n') || (line[len-1]=='\r')) line[len-1]='\0';
+      if ((line[len-2]=='\n') || (line[len-2]=='\r')) line[len-2]='\0';
       gtk_clist_append(GTK_CLIST(clist), new_line);
    }
    if (kept_line_selected > count -1) {
@@ -206,12 +236,13 @@ static void cb_clist_selection(GtkWidget      *clist,
    return;
 }
 
-int install_gui(int w, int h, int x, int y)
+int install_gui(GtkWidget *main_window, int w, int h, int x, int y)
 {
    GtkWidget *scrolled_window;
    GtkWidget *button;
    GtkWidget *label;
    char temp[256];
+   const char *svalue;
    gchar *titles[] = {"Files to be installed"
    };
 
@@ -221,15 +252,23 @@ int install_gui(int w, int h, int x, int y)
 
    line_selected = -1;
 
-   g_snprintf(temp, 255, "%s %s", PN, _("Install"));
-   temp[255]='\0';
+   g_snprintf(temp, sizeof(temp), "%s %s", PN, _("Install"));
    filew = gtk_widget_new(GTK_TYPE_FILE_SELECTION,
-			  "type", GTK_WINDOW_DIALOG,
-			  "x", x, "y", y,
-			  "width", w, "height", h,
+			  "type", GTK_WINDOW_TOPLEVEL,
 			  "title", temp,
 			  NULL);
 
+   gtk_window_set_default_size(GTK_WINDOW(filew), w, h);
+   gtk_widget_set_uposition(filew, x, y);
+
+   gtk_window_set_modal(GTK_WINDOW(filew), TRUE);
+   gtk_window_set_transient_for(GTK_WINDOW(filew), GTK_WINDOW(main_window));
+
+   get_pref(PREF_INSTALL_PATH, NULL, &svalue);
+   if (svalue && svalue[0]) {
+      gtk_file_selection_set_filename(GTK_FILE_SELECTION(filew), svalue);
+   }
+
    gtk_file_selection_hide_fileop_buttons((gpointer) filew);
 
    gtk_widget_hide((GTK_FILE_SELECTION(filew)->cancel_button));
@@ -243,8 +282,8 @@ int install_gui(int w, int h, int x, int y)
 		      "clicked", GTK_SIGNAL_FUNC(cb_add), filew);
 
 
-   strncpy(temp, _("Files to be installed"), 255);
-   temp[255]='\0';
+   strncpy(temp, _("Files to be installed"), sizeof(temp));
+   temp[sizeof(temp)-1]='\0';
    titles[0]=temp;
    clist = gtk_clist_new_with_titles(1, titles);
    gtk_widget_set_usize(GTK_WIDGET(clist), 0, 166);
@@ -297,5 +336,7 @@ int install_gui(int w, int h, int x, int y)
 
    update_clist();
 
+   gtk_main();
+
    return 0;
 }
