@@ -1,7 +1,7 @@
 /* jpilot-sync.c
  * A module of J-Pilot http://jpilot.org
  * 
- * Copyright (C) 1999-2001 by Judd Montgomery
+ * Copyright (C) 1999-2002 by Judd Montgomery
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,7 +30,7 @@
 
 /* this is a hack for now until I clean up the code */
 int *glob_date_label;
-int pipe_in, pipe_out;
+int pipe_to_parent, pipe_from_parent;
 pid_t glob_child_pid;
 GtkWidget *glob_dialog;
 pid_t glob_child_pid;
@@ -43,7 +43,7 @@ void cb_app_button(GtkWidget *widget, gpointer data)
    return;
 }
 
-#define USAGE_STRING "\njpilot-sync [ -v || -h || [-d] [-P] [-b] [-l] [-p] port]\n"\
+#define USAGE_STRING "\n"EPN"-sync [ -v || -h || [-d] [-P] [-b] [-l] [-p] port]\n"\
 " J-Pilot preferences are read to get port, rate, number of backups, etc.\n"\
 " -v = version\n"\
 " -h = help\n"\
@@ -80,12 +80,16 @@ int main(int argc, char *argv[])
    pref_init();
    pref_read_rc_file();
 
-   pipe_in=STDIN_FILENO;
-   pipe_out=STDOUT_FILENO;
+   pipe_from_parent=STDIN_FILENO;
+   pipe_to_parent=STDOUT_FILENO;
+   glob_log_stdout_mask = JP_LOG_INFO | JP_LOG_WARN | JP_LOG_FATAL |
+     JP_LOG_STDOUT | JP_LOG_GUI;
 
    for (i=1; i<argc; i++) {
       if (!strncasecmp(argv[i], "-v", 2)) {
-	 printf("%s\n", VERSION_STRING);
+	 char options[1024];
+	 get_compile_options(options, sizeof(options));
+	 printf("\n%s\n", options);
 	 exit(0);
       }
       if ( (!strncmp(argv[i], "-h", 2)) || (!strncasecmp(argv[1], "-?", 2))
@@ -96,7 +100,7 @@ int main(int argc, char *argv[])
       if (!strncasecmp(argv[i], "-d", 2)) {
 	 glob_log_stdout_mask = 0xFFFF;
 	 glob_log_file_mask = 0xFFFF;
-	 jpilot_logf(LOG_DEBUG, "Debug messages on.\n");
+	 jp_logf(JP_LOG_DEBUG, "Debug messages on.\n");
       }
       if (!strncmp(argv[i], "-b", 2)) {
 	 flags |= SYNC_FULL_BACKUP;
@@ -107,7 +111,7 @@ int main(int argc, char *argv[])
       if (!strncmp(argv[i], "-P", 2)) {
 	 skip_plugins = 1;
 	 flags |= SYNC_NO_PLUGINS;
-	 jpilot_logf(LOG_INFO, "Not loading plugins.\n");
+	 jp_logf(JP_LOG_INFO, "Not loading plugins.\n");
       }
       if (!strncmp(argv[i], "-p", 2)) {
 	 i++;
@@ -115,7 +119,7 @@ int main(int argc, char *argv[])
 	    strncpy(port, argv[i], MAX_PREF_VALUE);
 	    port[MAX_PREF_VALUE-1]='\0';
 	    /* Prefs are not saved, so this is not persistent */
-	    set_pref(PREF_PORT, 0, port);
+	    set_pref(PREF_PORT, 0, port, FALSE);
 	 }
       }
    }
@@ -130,7 +134,7 @@ int main(int argc, char *argv[])
 
    for (temp_list = plugin_list; temp_list; temp_list = temp_list->next) {
       plugin = (struct plugin_s *)temp_list->data;
-      jpilot_logf(LOG_DEBUG, "plugin: [%s] was loaded\n", plugin->name);
+      jp_logf(JP_LOG_DEBUG, "plugin: [%s] was loaded\n", plugin->name);
    }
 
    for (temp_list = plugin_list; temp_list; temp_list = temp_list->next) {
@@ -138,7 +142,7 @@ int main(int argc, char *argv[])
       if (plugin) {
 	 if (plugin->plugin_startup) {
 	    info.base_dir = strdup(BASE_DIR);
-	    jpilot_logf(LOG_DEBUG, "calling plugin_startup for [%s]\n", plugin->name);
+	    jp_logf(JP_LOG_DEBUG, "calling plugin_startup for [%s]\n", plugin->name);
 	    plugin->plugin_startup(&info);
 	    if (info.base_dir) {
 	       free(info.base_dir);
@@ -160,33 +164,49 @@ int main(int argc, char *argv[])
        case 0:
 	 break;
        case SYNC_ERROR_BIND:
+	 printf("\n");
 	 printf("Error: connecting to serial port\n");
 	 break;
        case SYNC_ERROR_LISTEN:
+	 printf("\n");
 	 printf("Error: pi_listen\n");
 	 break;
        case SYNC_ERROR_OPEN_CONDUIT:
+	 printf("\n");
 	 printf("Error: opening conduit to Palm\n");
 	 break;
        case SYNC_ERROR_PI_ACCEPT:
+	 printf("\n");
 	 printf("Error: pi_accept\n");
 	 break;
        case SYNC_ERROR_NOT_SAME_USER:
+	 printf("\n");
 	 printf("Error: this palm has a different User Name than the last sync.\n");
 	 printf(" Syncing with different palms into the same directory could cross data.\n");
-	 printf(" Use JPILOT_HOME to sync different palms under the same unix user.\n");
+#ifdef ENABLE_PROMETHEON
+	 printf(" COPILOT_HOME can be used to sync different palms under the same unix user.\n");
+#else
+	 printf(" JPILOT_HOME can be used to sync different palms under the same unix user.\n");
+#endif
 	 break;
        case SYNC_ERROR_NOT_SAME_USERID:
+	 printf("\n");
 	 printf("Error: this palm has a different ID Name than the last sync.\n");
 	 printf(" Syncing with different palms into the same directory could cross data.\n");
-	 printf(" Use JPILOT_HOME to sync different palms under the same unix user.\n");
+#ifdef ENABLE_PROMETHEON
+	 printf(" COPILOT_HOME can be used to sync different palms under the same unix user.\n");
+#else
+	 printf(" JPILOT_HOME can be used to sync different palms under the same unix user.\n");
+#endif
 	 break;
        case SYNC_ERROR_NULL_USERID:
+	 printf("\n");
 	 printf("Error: this palm has a NULL user ID.\n");
 	 printf(" use \"install-user /dev/pilot name numeric_id\"\n");
 	 break;
        default:
-	 printf("sync returned error %d\n", r);
+	 printf("\n");
+	 printf("Error: sync returned error %d\n", r);
       }
       sleep(1);
    } while(loop);
@@ -201,7 +221,7 @@ static void sig_handler(int sig)
    GList *plugin_list, *temp_list;
 #endif
 
-   jpilot_logf(LOG_DEBUG, "caught signal %d\n", sig);
+   jp_logf(JP_LOG_DEBUG, "caught signal %d\n", sig);
 
 #ifdef ENABLE_PLUGINS
    plugin_list = get_plugin_list();
@@ -210,7 +230,7 @@ static void sig_handler(int sig)
       plugin = (struct plugin_s *)temp_list->data;
       if (plugin) {
 	 if (plugin->plugin_exit_cleanup) {
-	    jpilot_logf(LOG_DEBUG, "calling plugin_exit_cleanup\n");
+	    jp_logf(JP_LOG_DEBUG, "calling plugin_exit_cleanup\n");
 	    plugin->plugin_exit_cleanup();
 	 }
       }
