@@ -1,7 +1,7 @@
 /* libplugin.c
  * A module of J-Pilot http://jpilot.org
  *
- * Copyright (C) 1999-2001 by Judd Montgomery
+ * Copyright (C) 1999-2002 by Judd Montgomery
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -42,15 +42,15 @@ const char *jp_strstr(const char *haystack, const char *needle, int case_sense)
    register const char *Ps1;
    char *r;
 
+   if (!haystack) {
+      return NULL;
+   }
+   if (!needle) {
+      return haystack;
+   }
    if (case_sense) {
       return strstr(haystack, needle);
    } else {
-      if (!haystack) {
-	 return NULL;
-      }
-      if (!needle) {
-	 return haystack;
-      }
       needle2 = malloc(strlen(needle)+2);
       haystack2 = malloc(strlen(haystack)+2);
 
@@ -82,7 +82,7 @@ const char *jp_strstr(const char *haystack, const char *needle, int case_sense)
    }
 }
 
-static int pack_header(PC3RecordHeader *header, char *packed_header)
+static int pack_header(PC3RecordHeader *header, unsigned char *packed_header)
 {
    unsigned char *p;
    unsigned long l;
@@ -161,7 +161,7 @@ static int unpack_header(PC3RecordHeader *header, unsigned char *packed_header)
    p+=sizeof(l);
 
    if (header->header_version > 2) {
-      jp_logf(LOG_WARN, "Unknown PC header version = %d\n", header->header_version);
+      jp_logf(JP_LOG_WARN, "Unknown PC header version = %d\n", header->header_version);
    }
 
    memcpy(&l, p, sizeof(l));
@@ -198,8 +198,8 @@ int read_header(FILE *pc_in, PC3RecordHeader *header)
    }
    memcpy(packed_header, &l, sizeof(l));
    len=ntohl(l);
-   if (len > 255) {
-      jp_logf(LOG_WARN, "read_header() error\n");
+   if (len > sizeof(packed_header)-1) {
+      jp_logf(JP_LOG_WARN, "read_header() error\n");
       return -1;
    }
    num = fread(packed_header+sizeof(l), len-sizeof(l), 1, pc_in);
@@ -235,6 +235,24 @@ int write_header(FILE *pc_out, PC3RecordHeader *header)
    return len;
 }
 
+int get_home_file_name(char *file, char *full_name, int max_size);
+int jp_get_home_file_name(char *file, char *full_name, int max_size)
+{
+   return get_home_file_name(file, full_name, max_size);
+}
+
+int pdb_file_write_app_block(char *DB_name, void *bufp, int size_in);
+int jp_pdb_file_write_app_block(char *DB_name, void *bufp, int size_in)
+{
+   return pdb_file_write_app_block(DB_name, bufp, size_in);
+}
+
+int edit_cats(GtkWidget *widget, char *db_name, struct CategoryAppInfo *cai);
+int jp_edit_cats(GtkWidget *widget, char *db_name, struct CategoryAppInfo *cai)
+{
+   return edit_cats(widget, db_name, cai);
+}
+
 /*
  * file must not be open elsewhere when this is called 
  * the first line in file is 0
@@ -247,22 +265,22 @@ int jp_install_remove_line(int deleted_line)
    char *Pc;
    int r, line_count;
 
-   in = jp_open_home_file("jpilot_to_install", "r");
+   in = jp_open_home_file(EPN"_to_install", "r");
    if (!in) {
-      jp_logf(LOG_DEBUG, "failed opening install_file\n");
+      jp_logf(JP_LOG_DEBUG, "failed opening install_file\n");
       return -1;
    }
 
-   out = jp_open_home_file("jpilot_to_install.tmp", "w");
+   out = jp_open_home_file(EPN"_to_install.tmp", "w");
    if (!out) {
       fclose(in);
-      jp_logf(LOG_DEBUG, "failed opening install_file.tmp\n");
+      jp_logf(JP_LOG_DEBUG, "failed opening install_file.tmp\n");
       return -1;
    }
 
    for (line_count=0; (!feof(in)); line_count++) {
       line[0]='\0';
-      Pc = fgets(line, 1000, in);
+      Pc = fgets(line, sizeof(line), in);
       if (!Pc) {
 	 break;
       }
@@ -277,7 +295,7 @@ int jp_install_remove_line(int deleted_line)
    fclose(in);
    fclose(out);
 
-   rename_file("jpilot_to_install.tmp", "jpilot_to_install");
+   rename_file(EPN"_to_install.tmp", EPN"_to_install");
 
    return 0;
 }
@@ -287,7 +305,7 @@ int jp_install_append_line(char *line)
    FILE *out;
    int r;
 
-   out = jp_open_home_file("jpilot_to_install", "a");
+   out = jp_open_home_file(EPN"_to_install", "a");
    if (!out) {
       return -1;
    }
@@ -366,7 +384,7 @@ int jp_free_DB_records(GList **br_list)
    return 0;
 }
 
-/*These next 2 functions were copied from pi-file.c in the pilot-link app */
+/* These next 2 functions were copied from pi-file.c in the pilot-link app */
 /* Exact value of "Jan 1, 1970 0:00:00 GMT" - "Jan 1, 1904 0:00:00 GMT" */
 #define PILOT_TIME_DELTA (unsigned)(2082844800)
 
@@ -427,10 +445,10 @@ int jp_get_app_info(char *DB_name, unsigned char **buf, int *buf_size)
 {
    FILE *in;
    int num;
-   unsigned int rec_size;
+   int rec_size;
    RawDBHeader rdbh;
    DBHeader dbh;
-   char PDB_name[256];
+   char PDB_name[FILENAME_MAX];
 
    if ((!buf_size) || (!buf)) {
       return -1;
@@ -438,16 +456,16 @@ int jp_get_app_info(char *DB_name, unsigned char **buf, int *buf_size)
    *buf = NULL;
    *buf_size=0;
 
-   g_snprintf(PDB_name, 255, "%s.pdb", DB_name);
+   g_snprintf(PDB_name, sizeof(PDB_name), "%s.pdb", DB_name);
    in = jp_open_home_file(PDB_name, "r");
    if (!in) {
-      jp_logf(LOG_WARN, "Error opening %s\n", PDB_name);
+      jp_logf(JP_LOG_WARN, "%s:%d Error opening %s\n", __FILE__, __LINE__, PDB_name);
       return -1;
    }
    num = fread(&rdbh, sizeof(RawDBHeader), 1, in);
    if (num != 1) {
       if (ferror(in)) {
-	 jp_logf(LOG_WARN, "Error reading %s\n", PDB_name);
+	 jp_logf(JP_LOG_WARN, "%s:%d Error reading %s\n", __FILE__, __LINE__, PDB_name);
 	 fclose(in);
 	 return -1;
       }
@@ -467,7 +485,7 @@ int jp_get_app_info(char *DB_name, unsigned char **buf, int *buf_size)
    fseek(in, dbh.app_info_offset, SEEK_SET);
    *buf=malloc(rec_size);
    if (!(*buf)) {
-      jp_logf(LOG_WARN, "jp_get_app_info(): Out of memory\n");
+      jp_logf(JP_LOG_WARN, "jp_get_app_info(): Out of memory\n");
       fclose(in);
       return -1;
    }
@@ -476,7 +494,7 @@ int jp_get_app_info(char *DB_name, unsigned char **buf, int *buf_size)
       if (ferror(in)) {
 	 fclose(in);
 	 free(*buf);
-	 jp_logf(LOG_WARN, "Error reading %s\n", PDB_name);
+	 jp_logf(JP_LOG_WARN, "%s:%d Error reading %s\n", __FILE__, __LINE__, PDB_name);
 	 return -1;
       }
    }
@@ -494,59 +512,63 @@ int jp_delete_record(char *DB_name, buf_rec *br, int flag)
 {
    FILE *pc_in;
    PC3RecordHeader header;
-   char PC_name[256];
+   char PC_name[FILENAME_MAX];
 
    if (br==NULL) {
       return -1;
    }
 
-   g_snprintf(PC_name, 255, "%s.pc3", DB_name);
+   g_snprintf(PC_name, sizeof(PC_name), "%s.pc3", DB_name);
 
    if ((br->rt==DELETED_PALM_REC) || (br->rt==MODIFIED_PALM_REC)) {
-      jp_logf(LOG_INFO, "This record is already deleted.\n"
+      jp_logf(JP_LOG_INFO, "This record is already deleted.\n"
 		  "It is scheduled to be deleted from the Palm on the next sync.\n");
       return 0;
    }
    switch (br->rt) {
     case NEW_PC_REC:
+    case REPLACEMENT_PALM_REC:
       pc_in=jp_open_home_file(PC_name, "r+");
       if (pc_in==NULL) {
-	 jp_logf(LOG_WARN, "Couldn't open PC records file\n");
+	 jp_logf(JP_LOG_WARN, "Couldn't open PC records file\n");
 	 return -1;
       }
       while(!feof(pc_in)) {
 	 read_header(pc_in, &header);
 	 if (feof(pc_in)) {
-	    jp_logf(LOG_WARN, "couldn't find record to delete\n");
+	    jp_logf(JP_LOG_WARN, "couldn't find record to delete\n");
 	    fclose(pc_in);
 	    return -1;
 	 }
 	 if (header.header_version==2) {
-	    if (header.unique_id==br->unique_id) {
+	    /* Keep unique ID intact */
+	    if ((header.unique_id==br->unique_id) && 
+		((header.rt==NEW_PC_REC) || (header.rt==REPLACEMENT_PALM_REC))
+		) {
 	       if (fseek(pc_in, -header.header_len, SEEK_CUR)) {
-		  jp_logf(LOG_WARN, "fseek failed\n");
+		  jp_logf(JP_LOG_WARN, "fseek failed\n");
 	       }
 	       header.rt=DELETED_PC_REC;
 	       write_header(pc_in, &header);
-	       jp_logf(LOG_DEBUG, "record deleted\n");
+	       jp_logf(JP_LOG_DEBUG, "record deleted\n");
 	       fclose(pc_in);
 	       return 0;
 	    }
 	 } else {
-	    jp_logf(LOG_WARN, "unknown header version %d\n", header.header_version);
+	    jp_logf(JP_LOG_WARN, "unknown header version %d\n", header.header_version);
 	 }
 	 if (fseek(pc_in, header.rec_len, SEEK_CUR)) {
-	    jp_logf(LOG_WARN, "fseek failed\n");
+	    jp_logf(JP_LOG_WARN, "fseek failed\n");
 	 }
       }
       fclose(pc_in);
       return -1;
 
     case PALM_REC:
-      jp_logf(LOG_DEBUG, "Deleteing Palm ID %d\n", br->unique_id);
+      jp_logf(JP_LOG_DEBUG, "Deleteing Palm ID %d\n", br->unique_id);
       pc_in=jp_open_home_file(PC_name, "a");
       if (pc_in==NULL) {
-	 jp_logf(LOG_WARN, "Couldn't open PC records file\n");
+	 jp_logf(JP_LOG_WARN, "Couldn't open PC records file\n");
 	 return -1;
       }
       header.unique_id=br->unique_id;
@@ -558,15 +580,15 @@ int jp_delete_record(char *DB_name, buf_rec *br, int flag)
 
       header.rec_len = br->size;
 
-      jp_logf(LOG_DEBUG, "writing header to pc file\n");
+      jp_logf(JP_LOG_DEBUG, "writing header to pc file\n");
       write_header(pc_in, &header);
       /*todo write the real appointment from palm db */
       /*Right now I am just writing an empty record */
       /*This will be used for making sure that the palm record hasn't changed */
       /*before we delete it */
-      jp_logf(LOG_DEBUG, "writing record to pc file, %d bytes\n", header.rec_len);
+      jp_logf(JP_LOG_DEBUG, "writing record to pc file, %d bytes\n", header.rec_len);
       fwrite(br->buf, header.rec_len, 1, pc_in);
-      jp_logf(LOG_DEBUG, "record deleted\n");
+      jp_logf(JP_LOG_DEBUG, "record deleted\n");
       fclose(pc_in);
       break;
     default:
@@ -576,34 +598,40 @@ int jp_delete_record(char *DB_name, buf_rec *br, int flag)
    return 0;
 }
 
+/*
+ * if buf_rec->unique_id==0 then the palm assigns an ID, else
+ *  use buf_rec->unique_id.
+ */
 int jp_pc_write(char *DB_name, buf_rec *br)
 {
    PC3RecordHeader header;
    FILE *out;
    unsigned int next_unique_id;
-   char packed_header[256];
+   unsigned char packed_header[256];
    int len;
-   char PC_name[256];
+   char PC_name[FILENAME_MAX];
 
-   g_snprintf(PC_name, 255, "%s.pc3", DB_name);
-   PC_name[255]='\0';
-
-   get_next_unique_pc_id(&next_unique_id);
+   g_snprintf(PC_name, sizeof(PC_name), "%s.pc3", DB_name);
+   if (br->unique_id==0) {
+      get_next_unique_pc_id(&next_unique_id);
+      header.unique_id=next_unique_id;
+      br->unique_id=next_unique_id;
+   } else {
+      header.unique_id=br->unique_id;
+   }
 #ifdef JPILOT_DEBUG
-   jp_logf(LOG_DEBUG, "next unique id = %d\n",next_unique_id);
+   jp_logf(JP_LOG_DEBUG, "br->unique id = %d\n",br->unique_id);
 #endif
 
    out = jp_open_home_file(PC_name, "a");
    if (!out) {
-      jp_logf(LOG_WARN, "Error opening %s\n", PC_name);
+      jp_logf(JP_LOG_WARN, "Error opening %s\n", PC_name);
       return -1;
    }
 
    header.rec_len=br->size;
    header.rt=br->rt;
    header.attrib=br->attrib;
-   header.unique_id=next_unique_id;
-   br->unique_id=next_unique_id;
 
    len = pack_header(&header, packed_header);
    write_header(out, &header);
@@ -626,7 +654,7 @@ static int pc_read_next_rec(FILE *in, buf_rec *br)
    num = read_header(in, &header);
    if (num < 1) {
       if (ferror(in)) {
-	 jp_logf(LOG_WARN, "Error reading pc file 1\n");
+	 jp_logf(JP_LOG_WARN, "Error reading pc file 1\n");
 	 return JPILOT_EOF;
       }
       if (feof(in)) {
@@ -636,13 +664,13 @@ static int pc_read_next_rec(FILE *in, buf_rec *br)
    rec_len = header.rec_len;
    record = malloc(rec_len);
    if (!record) {
-      jp_logf(LOG_WARN, "pc_read_next_rec(): Out of memory\n");
+      jp_logf(JP_LOG_WARN, "pc_read_next_rec(): Out of memory\n");
       return JPILOT_EOF;
    }
    num = fread(record, rec_len, 1, in);
    if (num != 1) {
       if (ferror(in)) {
-	 jp_logf(LOG_WARN, "Error reading pc file 2\n");
+	 jp_logf(JP_LOG_WARN, "Error reading pc file 2\n");
 	 free(record);
 	 return JPILOT_EOF;
       }
@@ -675,29 +703,28 @@ int jp_read_DB_files(char *DB_name, GList **records)
    RawDBHeader rdbh;
    DBHeader dbh;
    buf_rec *temp_br;
-   char PDB_name[256];
-   char PC_name[256];
+   int temp_br_used;
+   char PDB_name[FILENAME_MAX];
+   char PC_name[FILENAME_MAX];
 
-   jp_logf(LOG_DEBUG, "Entering jp_read_DB_files\n");
+   jp_logf(JP_LOG_DEBUG, "Entering jp_read_DB_files: %s\n", DB_name);
 
    mem_rh = last_mem_rh = NULL;
    *records = end_of_list = NULL;
    recs_returned = 0;
 
-   g_snprintf(PDB_name, 255, "%s.pdb", DB_name);
-   PDB_name[255]='\0';
-   g_snprintf(PC_name, 255, "%s.pc3", DB_name);
-   PC_name[255]='\0';
+   g_snprintf(PDB_name, sizeof(PDB_name), "%s.pdb", DB_name);
+   g_snprintf(PC_name, sizeof(PC_name), "%s.pc3", DB_name);
    in = jp_open_home_file(PDB_name, "r");
    if (!in) {
-      jp_logf(LOG_WARN, "Error opening %s\n", PDB_name);
+      jp_logf(JP_LOG_WARN, "Error opening %s\n", PDB_name);
       return -1;
    }
    /*Read the database header */
    num = fread(&rdbh, sizeof(RawDBHeader), 1, in);
    if (num != 1) {
       if (ferror(in)) {
-	 jp_logf(LOG_WARN, "Error reading %s\n", PDB_name);
+	 jp_logf(JP_LOG_WARN, "Error reading %s\n", PDB_name);
 	 fclose(in);
 	 return -1;
       }
@@ -707,9 +734,9 @@ int jp_read_DB_files(char *DB_name, GList **records)
    }
    raw_header_to_header(&rdbh, &dbh);
 #ifdef JPILOT_DEBUG
-   jp_logf(LOG_DEBUG, "db_name = %s\n", dbh.db_name);
-   jp_logf(LOG_DEBUG, "num records = %d\n", dbh.number_of_records);
-   jp_logf(LOG_DEBUG, "app info offset = %d\n", dbh.app_info_offset);
+   jp_logf(JP_LOG_DEBUG, "db_name = %s\n", dbh.db_name);
+   jp_logf(JP_LOG_DEBUG, "num records = %d\n", dbh.number_of_records);
+   jp_logf(JP_LOG_DEBUG, "app info offset = %d\n", dbh.app_info_offset);
 #endif
 
    /*Read each record entry header */
@@ -721,7 +748,7 @@ int jp_read_DB_files(char *DB_name, GList **records)
       num = fread(&rh, sizeof(record_header), 1, in);
       if (num != 1) {
 	 if (ferror(in)) {
-	    jp_logf(LOG_WARN, "Error reading %s\n", PDB_name);
+	    jp_logf(JP_LOG_WARN, "Error reading %s\n", PDB_name);
 	    break;
 	 }
 	 if (feof(in)) {
@@ -737,14 +764,14 @@ int jp_read_DB_files(char *DB_name, GList **records)
       prev_offset = offset;
 
 #ifdef JPILOT_DEBUG
-      jp_logf(LOG_DEBUG, "record header %u offset = %u\n",i, offset);
-      jp_logf(LOG_DEBUG, "       attrib 0x%x\n",rh.attrib);
-      jp_logf(LOG_DEBUG, "    unique_ID %d %d %d = ",rh.unique_ID[0],rh.unique_ID[1],rh.unique_ID[2]);
-      jp_logf(LOG_DEBUG, "%d\n",(rh.unique_ID[0]*256+rh.unique_ID[1])*256+rh.unique_ID[2]);
+      jp_logf(JP_LOG_DEBUG, "record header %u offset = %u\n",i, offset);
+      jp_logf(JP_LOG_DEBUG, "       attrib 0x%x\n",rh.attrib);
+      jp_logf(JP_LOG_DEBUG, "    unique_ID %d %d %d = ",rh.unique_ID[0],rh.unique_ID[1],rh.unique_ID[2]);
+      jp_logf(JP_LOG_DEBUG, "%d\n",(rh.unique_ID[0]*256+rh.unique_ID[1])*256+rh.unique_ID[2]);
 #endif
       temp_mem_rh = (mem_rec_header *)malloc(sizeof(mem_rec_header));
       if (!temp_mem_rh) {
-	 jp_logf(LOG_WARN, "jp_read_DB_files(): Out of memory 1\n");
+	 jp_logf(JP_LOG_WARN, "jp_read_DB_files(): Out of memory 1\n");
 	 break;
       }
       temp_mem_rh->next = NULL;
@@ -802,9 +829,9 @@ int jp_read_DB_files(char *DB_name, GList **records)
 	 }
 	 rec_size = next_offset - fpos;
 #ifdef JPILOT_DEBUG
-	 jp_logf(LOG_DEBUG, "rec_size = %u\n",rec_size);
-	 jp_logf(LOG_DEBUG, "fpos,next_offset = %u %u\n",fpos,next_offset);
-	 jp_logf(LOG_DEBUG, "----------\n");
+	 jp_logf(JP_LOG_DEBUG, "rec_size = %u\n",rec_size);
+	 jp_logf(JP_LOG_DEBUG, "fpos,next_offset = %u %u\n",fpos,next_offset);
+	 jp_logf(JP_LOG_DEBUG, "----------\n");
 #endif
 	 buf = malloc(rec_size);
 	 if (!buf) break;
@@ -815,7 +842,7 @@ int jp_read_DB_files(char *DB_name, GList **records)
 	 }
 	 if ((num < 1)) {
 	    if (ferror(in)) {
-	       jp_logf(LOG_WARN, "Error reading %s 5\n", PDB_name);
+	       jp_logf(JP_LOG_WARN, "Error reading %s 5\n", PDB_name);
 	       free(buf);
 	       break;
 	    }
@@ -823,7 +850,7 @@ int jp_read_DB_files(char *DB_name, GList **records)
 
 	 temp_br = malloc(sizeof(buf_rec));
 	 if (!temp_br) {
-	    jp_logf(LOG_WARN, "jp_read_DB_files(): Out of memory 2\n");
+	    jp_logf(JP_LOG_WARN, "jp_read_DB_files(): Out of memory 2\n");
 	    break;
 	 }
 	 temp_br->rt = PALM_REC;
@@ -856,14 +883,15 @@ int jp_read_DB_files(char *DB_name, GList **records)
    /* */
    pc_in = jp_open_home_file(PC_name, "r");
    if (pc_in==NULL) {
-      jp_logf(LOG_DEBUG, "jp_open_home_file failed\n");
+      jp_logf(JP_LOG_DEBUG, "jp_open_home_file failed: %s\n", PC_name);
       return 0;
    }
 
    while(!feof(pc_in)) {
+      temp_br_used = 0;
       temp_br = malloc(sizeof(buf_rec));
       if (!temp_br) {
-	 jp_logf(LOG_WARN, "jp_read_DB_files(): Out of memory 3\n");
+	 jp_logf(JP_LOG_WARN, "jp_read_DB_files(): Out of memory 3\n");
 	 break;
       }
       r = pc_read_next_rec(pc_in, temp_br);
@@ -888,7 +916,7 @@ int jp_read_DB_files(char *DB_name, GList **records)
 	       end_of_list=end_of_list->next;
 	    }
 	 }
-
+         temp_br_used = 1;
 	 recs_returned++;
       }
       if ((temp_br->rt==DELETED_PALM_REC) || (temp_br->rt==MODIFIED_PALM_REC)) {
@@ -900,14 +928,21 @@ int jp_read_DB_files(char *DB_name, GList **records)
 	 }
 	 for (; temp_list; temp_list=temp_list->next) {
 	    if (((buf_rec *)temp_list->data)->unique_id == temp_br->unique_id) {
-	       ((buf_rec *)temp_list->data)->rt = temp_br->rt;
+	       if (((buf_rec *)temp_list->data)->rt == PALM_REC) {
+		  ((buf_rec *)temp_list->data)->rt = temp_br->rt;
+	       }
 	    }
 	 }
       }
+
+      if (!temp_br_used) {
+         free(temp_br->buf);
+         free(temp_br);
+      }
    }
    fclose(pc_in);
 
-   jp_logf(LOG_DEBUG, "Leaving jp_read_DB_files\n");
+   jp_logf(JP_LOG_DEBUG, "Leaving jp_read_DB_files\n");
 
    return recs_returned;
 }
