@@ -1,7 +1,7 @@
 /* libplugin.h
  * A module of J-Pilot http://jpilot.org
  *
- * Copyright (C) 1999-2001 by Judd Montgomery
+ * Copyright (C) 1999-2002 by Judd Montgomery
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,13 +19,27 @@
 #ifndef __LIBPLUGIN_H__
 #define __LIBPLUGIN_H__
 
+#include "config.h"
 #include <gtk/gtk.h>
 #include <time.h>
+#include <pi-appinfo.h>
 
 /*
  * PLUGIN API for J-Pilot
  */
 
+#ifdef ENABLE_PROMETHEON
+#define PN "CoPilot"
+#else
+#define PN "J-Pilot"
+#endif
+
+#ifdef ENABLE_PROMETHEON
+#define EPN "copilot"
+#else
+#define EPN "jpilot"
+#endif
+
 /*
  * For versioning of files
  */
@@ -101,18 +115,22 @@ int get_next_unique_pc_id(unsigned int *next_unique_id);
 #define NEW_FLAG    5
 #define COPY_FLAG   6
 
-#define CLIST_DEL_RED 0xCCCC;
-#define CLIST_DEL_GREEN 0xCCCC;
-#define CLIST_DEL_BLUE 0xCCCC;
-#define CLIST_NEW_RED 55000;
-#define CLIST_NEW_GREEN 55000;
-#define CLIST_NEW_BLUE 65535;
-#define CLIST_MOD_RED 55000;
-#define CLIST_MOD_GREEN 65535;
-#define CLIST_MOD_BLUE 65535;
-#define CLIST_PRIVATE_RED 60000;
-#define CLIST_PRIVATE_GREEN 55000;
-#define CLIST_PRIVATE_BLUE 55000;
+#define CLIST_DEL_RED 0xCCCC
+#define CLIST_DEL_GREEN 0xCCCC
+#define CLIST_DEL_BLUE 0xCCCC
+#define CLIST_NEW_RED 55000
+#define CLIST_NEW_GREEN 55000
+#define CLIST_NEW_BLUE 65535
+#define CLIST_MOD_RED 55000
+#define CLIST_MOD_GREEN 65535
+#define CLIST_MOD_BLUE 65535
+#define CLIST_PRIVATE_RED 60000
+#define CLIST_PRIVATE_GREEN 55000
+#define CLIST_PRIVATE_BLUE 55000
+#define CLIST_OVERDUE_RED 0xD900
+#define CLIST_OVERDUE_GREEN 0x0000
+#define CLIST_OVERDUE_BLUE 0x0000
+
 
 #define DIALOG_SAID_1        454
 #define DIALOG_SAID_PRINT    454
@@ -124,22 +142,19 @@ int get_next_unique_pc_id(unsigned int *next_unique_id);
 #define DIALOG_SAID_3        456
 #define DIALOG_SAID_CANCEL   456
 
-#define LOG_DEBUG  1    /*debugging info for programers, and bug reports */
-#define LOG_INFO   2    /*info, and misc messages */
-#define LOG_WARN   4    /*worse messages */
-#define LOG_FATAL  8    /*even worse messages */
-#define LOG_STDOUT 256  /*messages always go to stdout */
-#define LOG_FILE   512  /*messages always go to the log file */
-#define LOG_GUI    1024 /*messages always go to the gui window */
+#define JP_LOG_DEBUG  1    /*debugging info for programers, and bug reports */
+#define JP_LOG_INFO   2    /*info, and misc messages */
+#define JP_LOG_WARN   4    /*worse messages */
+#define JP_LOG_FATAL  8    /*even worse messages */
+#define JP_LOG_STDOUT 256  /*messages always go to stdout */
+#define JP_LOG_FILE   512  /*messages always go to the log file */
+#define JP_LOG_GUI    1024 /*messages always go to the gui window */
 
 #define JPILOT_EOF -7
 
-extern int jpilot_logf(int level, char *format, ...);
-/* FIXME: Need a policy.  Should all symbols avaliable to 
- * plugins start with jp or jpilot?
- */
-#define jp_logf jpilot_logf
+extern int jp_logf(int level, char *format, ...);
 
+/* This bit means that this record is of no importance anymore */
 #define SPENT_PC_RECORD_BIT 256
 
 typedef enum {
@@ -147,8 +162,9 @@ typedef enum {
    MODIFIED_PALM_REC = 101L,
    DELETED_PALM_REC = 102L,
    NEW_PC_REC = 103L,
-   DELETED_PC_REC =  SPENT_PC_RECORD_BIT + 104L,
-   DELETED_DELETED_PALM_REC =  SPENT_PC_RECORD_BIT + 105L
+   DELETED_PC_REC =  SPENT_PC_RECORD_BIT | 104L,
+   DELETED_DELETED_PALM_REC =  SPENT_PC_RECORD_BIT | 105L,
+   REPLACEMENT_PALM_REC = 106L
 } PCRecType;
 
 typedef struct
@@ -183,16 +199,42 @@ int plugin_startup(jp_startup_info *info);
 int plugin_gui(GtkWidget *vbox, GtkWidget *hbox, unsigned int unique_id);
 int plugin_help(char **text, int *width, int *height);
 int plugin_gui_cleanup(void);
+int plugin_pre_sync_pre_connect(void);
 int plugin_pre_sync(void);
 int plugin_sync(int sd);
-int plugin_search(char *search_string, int case_sense, struct search_result **sr);
+int plugin_search(const char *search_string, int case_sense, struct search_result **sr);
 int plugin_post_sync(void);
 int plugin_exit_cleanup(void);
+int plugin_unpack_cai_from_ai(struct CategoryAppInfo *cai,
+			      unsigned char *ai_raw, int len);
+int plugin_pack_cai_into_ai(struct CategoryAppInfo *cai,
+			    unsigned char *ai_raw, int len);
 /* callbacks are needed for print */
 
 void jp_init();
 extern FILE *jp_open_home_file(char *filename, char *mode);
 
+/* This takes the value of $JPILOT_HOME and appends /.jpilot/ and {file}
+ * onto it and puts it into full_name.  max_size is the size if the
+ * supplied buffer full_name
+ */
+int jp_get_home_file_name(char *file, char *full_name, int max_size);
+
+/*
+ * DB_name should be without filename ext, e.g. MemoDB
+ * bufp is the packed app info block
+ * size_in is the size of bufp
+ */
+int jp_pdb_file_write_app_block(char *DB_name, void *bufp, int size_in);
+
+/*
+ * widget is a widget inside the main window used to get main window handle
+ * db_name should be without filename ext, e.g. MemoDB
+ * cai is the category app info.  This should be unpacked by the user since
+ * category unpack functions are database specific.
+ */
+int jp_edit_cats(GtkWidget *widget, char *db_name, struct CategoryAppInfo *cai);
+
 /*************************************
  * convert char code
  *************************************/
