@@ -1,7 +1,7 @@
 /* memo.c
  * A module of J-Pilot http://jpilot.org
  * 
- * Copyright (C) 1999-2001 by Judd Montgomery
+ * Copyright (C) 1999-2002 by Judd Montgomery
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -89,7 +89,7 @@ int memo_sort(MemoList **memol, int sort_order)
    /* Allocate an array to be qsorted */
    sort_memol = calloc(count, sizeof(MemoList *));
    if (!sort_memol) {
-      jpilot_logf(LOG_WARN, "memo_sort(): Out of Memory\n");
+      jp_logf(JP_LOG_WARN, "memo_sort(): Out of Memory\n");
       return 0;
    }
 
@@ -125,30 +125,46 @@ int memo_sort(MemoList **memol, int sort_order)
    return 0;
 }
 
+/*
+ * This function writes to the MemoDB.pc3 file
+ * 
+ * memo - input - a memo to be written
+ * rt - input - type of record to be written
+ * attrib - input - attributes of record
+ * unique_id - input/output - If unique_id==0 then the palm assigns an ID,
+ *  else the unique_id passed in is used.
+ */
 int pc_memo_write(struct Memo *memo, PCRecType rt, unsigned char attrib,
 		  unsigned int *unique_id)
 {
-   char record[65536];
+   unsigned char record[65536];
    int rec_len;
    buf_rec br;
    long ivalue;
    long char_set;
 
    get_pref(PREF_CHAR_SET, &char_set, NULL);
-   if (char_set != CHAR_SET_ENGLISH) {
-      if (memo->text) charset_j2p(memo->text, strlen(memo->text)+1, char_set);
+   if (char_set != CHAR_SET_LATIN1) {
+      if (memo->text) {
+	 charset_j2p((unsigned char *)memo->text, strlen(memo->text)+1, char_set);
+      }
    }
-
    rec_len = pack_Memo(memo, record, 65535);
    if (!rec_len) {
       PRINT_FILE_LINE;
-      jpilot_logf(LOG_WARN, "pack_Memo %s\n", _("error"));
+      jp_logf(JP_LOG_WARN, "pack_Memo %s\n", _("error"));
       return -1;
    }
    br.rt=rt;
    br.attrib = attrib;
    br.buf = record;
    br.size = rec_len;
+   /* Keep unique ID intact */
+   if (unique_id) {
+      br.unique_id = *unique_id;
+   } else {
+      br.unique_id = 0;
+   }
 
    get_pref(PREF_MEMO32_MODE, &ivalue, NULL);
    if (ivalue) {
@@ -156,7 +172,9 @@ int pc_memo_write(struct Memo *memo, PCRecType rt, unsigned char attrib,
    } else {
       jp_pc_write("MemoDB", &br);
    }
-   *unique_id = br.unique_id;
+   if (unique_id) {
+      *unique_id = br.unique_id;
+   }
 
    return 0;
 }
@@ -175,34 +193,39 @@ void free_MemoList(MemoList **memo)
 
 int get_memo_app_info(struct MemoAppInfo *ai)
 {
-   int num,i;
+   int num;
    unsigned int rec_size;
    unsigned char *buf;
    long char_set;
+   int i;
    long ivalue;
+   char DBname[32];
 
-   bzero(ai, sizeof(*ai));
+   memset(ai, 0, sizeof(*ai));
+   /* Put at least one entry in there */
+   strcpy(ai->category.name[0], "Unfiled");
 
    get_pref(PREF_MEMO32_MODE, &ivalue, NULL);
    if (ivalue) {
-      jp_get_app_info("Memo32DB", &buf, &rec_size);
+      strcpy(DBname, "Memo32DB");
    } else {
-      jp_get_app_info("MemoDB", &buf, &rec_size);
+      strcpy(DBname, "MemoDB");
    }
+   jp_get_app_info(DBname, &buf, (int*)&rec_size);
    num = unpack_MemoAppInfo(ai, buf, rec_size);
    if (buf) {
       free(buf);
    }
-   if (num <= 0) {
-      jpilot_logf(LOG_WARN, _("Error reading"), "MemoDB.pdb");
+   if ((num<0) || (rec_size<=0)) {
+      jp_logf(JP_LOG_WARN, _("Error reading %s\n"), DBname);
       return -1;
    }
 
    get_pref(PREF_CHAR_SET, &char_set, NULL);
-   if (char_set != CHAR_SET_ENGLISH) {
+   if (char_set != CHAR_SET_LATIN1) {
       for (i = 0; i < 16; i++) {
 	 if (ai->category.name[i][0] != '\0') {
-	    charset_p2j(ai->category.name[i], 16, char_set);
+	    charset_p2j((unsigned char *)ai->category.name[i], 16, char_set);
 	 }
       }
    }
@@ -232,7 +255,7 @@ int get_memos2(MemoList **memo_list, int sort_order,
    long ivalue;
    buf_rec *br;
 
-   jpilot_logf(LOG_DEBUG, "get_memos2()\n");
+   jp_logf(JP_LOG_DEBUG, "get_memos2()\n");
    if (modified==2) {
       get_pref(PREF_SHOW_MODIFIED, &keep_modified, NULL);
    } else {
@@ -244,7 +267,7 @@ int get_memos2(MemoList **memo_list, int sort_order,
       keep_deleted = deleted;
    }
    if (privates==2) {
-      keep_priv = show_privates(GET_PRIVATES, NULL);
+      keep_priv = show_privates(GET_PRIVATES);
    } else {
       keep_priv = privates;
    }
@@ -281,6 +304,7 @@ int get_memos2(MemoList **memo_list, int sort_order,
 	 continue;
       }
 
+      /* JPA unpacking sets memo buffer */
       num = unpack_Memo(&memo, br->buf, br->size);
 
       if (num <= 0) {
@@ -291,11 +315,19 @@ int get_memos2(MemoList **memo_list, int sort_order,
 	 continue;
       }
       get_pref(PREF_CHAR_SET, &char_set, NULL);
-      if (memo.text) charset_p2j(memo.text, strlen(memo.text)+1, char_set);
+      if (memo.text) {
+	 /* JPA character set conversion may require a longer buffer */
+
+         unsigned char *newtext; /* new buffer */
+
+         newtext = charset_p2newj((const unsigned char *)memo.text, strlen(memo.text)+1, char_set);
+         free(memo.text);
+         memo.text = (char*)newtext;
+      }
 
       temp_memo_list = malloc(sizeof(MemoList));
       if (!temp_memo_list) {
-	 jpilot_logf(LOG_WARN, "get_memos2(): Out of memory\n");
+	 jp_logf(JP_LOG_WARN, "get_memos2(): Out of memory\n");
 	 break;
       }
       memcpy(&(temp_memo_list->mmemo.memo), &memo, sizeof(struct Memo));
@@ -312,7 +344,7 @@ int get_memos2(MemoList **memo_list, int sort_order,
 
    memo_sort(memo_list, sort_order);
 
-   jpilot_logf(LOG_DEBUG, "Leaving get_memos2()\n");
+   jp_logf(JP_LOG_DEBUG, "Leaving get_memos2()\n");
 
    return recs_returned;
 }
