@@ -1,7 +1,7 @@
 /* memo_gui.c
  * A module of J-Pilot http://jpilot.org
  * 
- * Copyright (C) 1999-2001 by Judd Montgomery
+ * Copyright (C) 1999-2002 by Judd Montgomery
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,6 +35,9 @@
 #include "export.h"
 
 
+#define MEMO_MAX_COLUMN_LEN 80
+#define MEMO_CLIST_CHAR_WIDTH 50
+
 #define NUM_MEMO_CAT_ITEMS 16
 
 #define CONNECT_SIGNALS 400
@@ -47,6 +50,9 @@ static int memo_category = CATEGORY_ALL;
 static int clist_row_selected;
 static GtkWidget *clist;
 static GtkWidget *memo_text;
+#ifdef ENABLE_GTK2
+static GObject   *memo_text_buffer;
+#endif
 static GtkWidget *private_checkbox;
 static GtkWidget *memo32_checkbox;
 /*Need one extra for the ALL category */
@@ -54,12 +60,13 @@ static GtkWidget *memo_cat_menu_item1[NUM_MEMO_CAT_ITEMS+1];
 static GtkWidget *memo_cat_menu_item2[NUM_MEMO_CAT_ITEMS];
 static GtkWidget *category_menu1;
 static GtkWidget *category_menu2;
-static GtkWidget *scrolled_window;
 static GtkWidget *pane;
 static struct sorted_cats sort_l[NUM_MEMO_CAT_ITEMS];
 static GtkWidget *new_record_button;
 static GtkWidget *apply_record_button;
 static GtkWidget *add_record_button;
+static GtkWidget *delete_record_button;
+static GtkWidget *copy_record_button;
 static int record_changed;
 static int clist_hack;
 
@@ -78,7 +85,7 @@ static void cb_add_new_record(GtkWidget *widget, gpointer data);
 static void
 set_new_button_to(int new_state)
 {
-   jpilot_logf(LOG_DEBUG, "set_new_button_to new %d old %d\n", new_state, record_changed);
+   jp_logf(JP_LOG_DEBUG, "set_new_button_to new %d old %d\n", new_state, record_changed);
    if (record_changed==new_state) {
       return;
    }
@@ -90,6 +97,7 @@ set_new_button_to(int new_state)
       /* The line selected on the clist becomes unhighlighted, so we do this */
       gtk_clist_select_row(GTK_CLIST(clist), clist_row_selected, 0);
       gtk_widget_show(apply_record_button);
+      gtk_widget_hide(delete_record_button);
       break;
     case NEW_FLAG:
       gtk_clist_set_selection_mode(GTK_CLIST(clist), GTK_SELECTION_SINGLE);
@@ -97,11 +105,15 @@ set_new_button_to(int new_state)
       /* The line selected on the clist becomes unhighlighted, so we do this */
       gtk_clist_select_row(GTK_CLIST(clist), clist_row_selected, 0);
       gtk_widget_show(add_record_button);
+      gtk_widget_hide(copy_record_button);
+      gtk_widget_hide(delete_record_button);
       break;
     case CLEAR_FLAG:
       gtk_clist_set_selection_mode(GTK_CLIST(clist), GTK_SELECTION_BROWSE);
       clist_hack=FALSE;
       gtk_widget_show(new_record_button);
+      gtk_widget_show(copy_record_button);
+      gtk_widget_show(delete_record_button);
       break;
     default:
       return;
@@ -109,12 +121,17 @@ set_new_button_to(int new_state)
    switch (record_changed) {
     case MODIFY_FLAG:
       gtk_widget_hide(apply_record_button);
+      gtk_widget_show(copy_record_button);
+      gtk_widget_show(delete_record_button);
       break;
     case NEW_FLAG:
       gtk_widget_hide(add_record_button);
+      gtk_widget_show(copy_record_button);
+      gtk_widget_show(delete_record_button);
       break;
     case CLEAR_FLAG:
       gtk_widget_hide(new_record_button);
+      gtk_widget_hide(delete_record_button);
       break;
    }
    record_changed=new_state;
@@ -124,7 +141,7 @@ static void
 cb_record_changed(GtkWidget *widget,
 		  gpointer   data)
 {
-   jpilot_logf(LOG_DEBUG, "cb_record_changed\n");
+   jp_logf(JP_LOG_DEBUG, "cb_record_changed\n");
    if (record_changed==CLEAR_FLAG) {
       connect_changed_signals(DISCONNECT_SIGNALS);
       if (((GtkCList *)clist)->rows > 0) {
@@ -149,8 +166,14 @@ static void connect_changed_signals(int con_or_dis)
 			       GTK_SIGNAL_FUNC(cb_record_changed), NULL);
 	 }
       }
+#ifdef ENABLE_GTK2
+      g_signal_connect(memo_text_buffer, "changed",
+		       GTK_SIGNAL_FUNC(cb_record_changed), NULL);
+#else
       gtk_signal_connect(GTK_OBJECT(memo_text), "changed",
 			 GTK_SIGNAL_FUNC(cb_record_changed), NULL);
+#endif
+
       gtk_signal_connect(GTK_OBJECT(private_checkbox), "toggled",
 			 GTK_SIGNAL_FUNC(cb_record_changed), NULL);
    }
@@ -165,8 +188,13 @@ static void connect_changed_signals(int con_or_dis)
 					  GTK_SIGNAL_FUNC(cb_record_changed), NULL);     
 	 }
       }
+#ifdef ENABLE_GTK2
+      g_signal_handlers_disconnect_by_func(memo_text_buffer,
+					   GTK_SIGNAL_FUNC(cb_record_changed), NULL);
+#else
       gtk_signal_disconnect_by_func(GTK_OBJECT(memo_text),
 				    GTK_SIGNAL_FUNC(cb_record_changed), NULL);
+#endif
       gtk_signal_disconnect_by_func(GTK_OBJECT(private_checkbox),
 				    GTK_SIGNAL_FUNC(cb_record_changed), NULL);
    }
@@ -210,14 +238,13 @@ int memo_print()
 /*
  * Start Import Code
  */
-int memo_import_callback(GtkWidget *parent_window, char *file_path, int type)
+int memo_import_callback(GtkWidget *parent_window, const char *file_path, int type)
 {
    FILE *in;
    char line[256];
    char text[65536];
    struct Memo new_memo;
    unsigned char attrib;
-   unsigned int unique_id;
    unsigned int len;
    unsigned int text_len;
    int i, ret, index;
@@ -237,13 +264,13 @@ int memo_import_callback(GtkWidget *parent_window, char *file_path, int type)
 
    in=fopen(file_path, "r");
    if (!in) {
-      jpilot_logf(LOG_WARN, _("Could not open file %s\n"), file_path);
+      jp_logf(JP_LOG_WARN, _("Could not open file %s\n"), file_path);
       return -1;
    }
 
    /* TEXT */
    if (type==IMPORT_TYPE_TEXT) {
-      jpilot_logf(LOG_DEBUG, "Memo import text [%s]\n", file_path);
+      jp_logf(JP_LOG_DEBUG, "Memo import text [%s]\n", file_path);
       text_len=0;
       text[0]='\0';
       while (!feof(in)) {
@@ -254,7 +281,7 @@ int memo_import_callback(GtkWidget *parent_window, char *file_path, int type)
 	 if (text_len+len>65534) {
 	    len=65534-text_len;
 	    line[len]='\0';
-	    jpilot_logf(LOG_WARN, "Memo text > 65535, truncating\n");
+	    jp_logf(JP_LOG_WARN, "Memo text > 65535, truncating\n");
 	    strcat(text, line);
 	    break;
 	 }
@@ -276,14 +303,14 @@ int memo_import_callback(GtkWidget *parent_window, char *file_path, int type)
 			    attrib & 0x0F,
 			    &new_cat_num);
       if ((ret==DIALOG_SAID_IMPORT_ALL) || (ret==DIALOG_SAID_IMPORT_YES)) {
-	 pc_memo_write(&new_memo, NEW_PC_REC, attrib, &unique_id);
-	 jpilot_logf(LOG_WARN, "Imported Memo %s\n", file_path);
+	 pc_memo_write(&new_memo, NEW_PC_REC, attrib, NULL);
+	 jp_logf(JP_LOG_WARN, "Imported Memo %s\n", file_path);
       }
    }
 
    /* CSV */
    if (type==IMPORT_TYPE_CSV) {
-      jpilot_logf(LOG_DEBUG, "Memo import CSV [%s]\n", file_path);
+      jp_logf(JP_LOG_DEBUG, "Memo import CSV [%s]\n", file_path);
       /* The first line is format, so we don't need it */
       fgets(text, 1000, in);
       import_all=FALSE;
@@ -298,7 +325,6 @@ int memo_import_callback(GtkWidget *parent_window, char *file_path, int type)
 	 old_cat_name[16]='\0';
 	 attrib=0;
 	 /* Figure out what the best category number is */
-	 index=temp_memolist->mmemo.unique_id-1;
 	 suggested_cat_num=0;
 	 for (i=0; i<NUM_MEMO_CAT_ITEMS; i++) {
 	    if (memo_app_info.category.name[i][0]=='\0') continue;
@@ -340,16 +366,16 @@ int memo_import_callback(GtkWidget *parent_window, char *file_path, int type)
 	 attrib = (new_cat_num & 0x0F) |
 	   (priv ? dlpRecAttrSecret : 0);
 	 if ((ret==DIALOG_SAID_IMPORT_YES) || (import_all)) {
-	    pc_memo_write(&new_memo, NEW_PC_REC, attrib, &unique_id);
+	    pc_memo_write(&new_memo, NEW_PC_REC, attrib, NULL);
 	 }
       }
    }
 
    /* Palm Desktop DAT format */
    if (type==IMPORT_TYPE_DAT) {
-      jpilot_logf(LOG_DEBUG, "Memo import DAT [%s]\n", file_path);
+      jp_logf(JP_LOG_DEBUG, "Memo import DAT [%s]\n", file_path);
       if (dat_check_if_dat_file(in)!=DAT_MEMO_FILE) {
-	 jpilot_logf(LOG_WARN, _("File doesn't appear to be memopad.dat format\n"));
+	 jp_logf(JP_LOG_WARN, _("File doesn't appear to be memopad.dat format\n"));
 	 fclose(in);
 	 return 1;
       }
@@ -408,7 +434,7 @@ int memo_import_callback(GtkWidget *parent_window, char *file_path, int type)
 	 attrib = (new_cat_num & 0x0F) |
 	   ((temp_memolist->mmemo.attrib & 0x10) ? dlpRecAttrSecret : 0);
 	 if ((ret==DIALOG_SAID_IMPORT_YES) || (import_all)) {
-	    pc_memo_write(&new_memo, NEW_PC_REC, attrib, &unique_id);
+	    pc_memo_write(&new_memo, NEW_PC_REC, attrib, NULL);
 	 }
       }
       free_MemoList(&memolist);
@@ -467,14 +493,14 @@ void cb_memo_export_ok(GtkWidget *export_window, GtkWidget *clist,
 
    if (!stat(filename, &statb)) {
       if (S_ISDIR(statb.st_mode)) {
-	 g_snprintf(text, 1024, _("%s is a directory"), filename);
-	 dialog_generic(GTK_WIDGET(export_window)->window,
+	 g_snprintf(text, sizeof(text), _("%s is a directory"), filename);
+	 dialog_generic(GTK_WINDOW(export_window),
 			0, 0, _("Error Opening File"),
 			"Directory", text, 1, button_text);
 	 return;
       }
-      g_snprintf(text, 1024, _("Do you want to overwrite file %s?"), filename);
-      r = dialog_generic(GTK_WIDGET(export_window)->window,
+      g_snprintf(text,sizeof(text), _("Do you want to overwrite file %s?"), filename);
+      r = dialog_generic(GTK_WINDOW(export_window),
 			 0, 0, _("Overwrite File?"),
 			 _("Overwrite File"), text, 2, button_overwrite_text);
       if (r!=DIALOG_SAID_1) {
@@ -484,8 +510,8 @@ void cb_memo_export_ok(GtkWidget *export_window, GtkWidget *clist,
 
    out = fopen(filename, "w");
    if (!out) {
-      g_snprintf(text, 1024, "Error Opening File: %s", filename);
-      dialog_generic(GTK_WIDGET(export_window)->window,
+      g_snprintf(text,sizeof(text), "Error Opening File: %s", filename);
+      dialog_generic(GTK_WINDOW(export_window),
 		     0, 0, _("Error Opening File"),
 		     "Filename", text, 1, button_text);
       return;
@@ -498,7 +524,7 @@ void cb_memo_export_ok(GtkWidget *export_window, GtkWidget *clist,
       mmemo = gtk_clist_get_row_data(GTK_CLIST(clist), (int) temp_list->data);
       if (!mmemo) {
 	 continue;
-	 jpilot_logf(LOG_WARN, "Can't export memo %d\n", (long) temp_list->data + 1);
+	 jp_logf(JP_LOG_WARN, "Can't export memo %d\n", (long) temp_list->data + 1);
       }
       switch (type) {
        case EXPORT_TYPE_CSV:
@@ -513,11 +539,11 @@ void cb_memo_export_ok(GtkWidget *export_window, GtkWidget *clist,
 	 csv_text=malloc(len);
 	 if (!csv_text) {
 	    continue;
-	    jpilot_logf(LOG_WARN, "Can't export memo %d\n", (long) temp_list->data + 1);
+	    jp_logf(JP_LOG_WARN, "Can't export memo %d\n", (long) temp_list->data + 1);
 	 }
 	 str_to_csv_str(csv_text, memo_app_info.category.name[mmemo->attrib & 0x0F]);
-	 fprintf(out, "\"%s\", ", csv_text);
-	 fprintf(out, "\"%s\", ", (mmemo->attrib & dlpRecAttrSecret) ? "1":"0");
+	 fprintf(out, "\"%s\",", csv_text);
+	 fprintf(out, "\"%s\",", (mmemo->attrib & dlpRecAttrSecret) ? "1":"0");
 	 str_to_csv_str(csv_text, mmemo->memo.text);
 	 fprintf(out, "\"%s\"\n", csv_text);
 	 free(csv_text);
@@ -527,10 +553,9 @@ void cb_memo_export_ok(GtkWidget *export_window, GtkWidget *clist,
 	 get_pref_time_no_secs(pref_time);
 	 time(&ltime);
 	 now = localtime(&ltime);
-	 strftime(str1, 50, short_date, now);
-	 strftime(str2, 50, pref_time, now);
-	 g_snprintf(text, 100, "%s %s", str1, str2);
-	 text[100]='\0';
+	 strftime(str1, sizeof(str1), short_date, now);
+	 strftime(str2, sizeof(str2), pref_time, now);
+	 g_snprintf(text, sizeof(text), "%s %s", str1, str2);
 
 	 /* Todo Should I translate these? */
 	 fprintf(out, "Memo: %ld, exported from %s on %s\n",
@@ -543,7 +568,7 @@ void cb_memo_export_ok(GtkWidget *export_window, GtkWidget *clist,
 	 fprintf(out, "\n----- End of Memo -----\n\n");
 	 break;
        default:
-	 jpilot_logf(LOG_WARN, "Unknown export type\n");
+	 jp_logf(JP_LOG_WARN, "Unknown export type\n");
       }
    }
 
@@ -563,21 +588,30 @@ static void cb_memo_export_done(GtkWidget *widget, const char *filename)
 {
    free_MemoList(&export_memo_list);
 
-   set_pref(PREF_MEMO_EXPORT_FILENAME, 0, filename);
+   set_pref(PREF_MEMO_EXPORT_FILENAME, 0, filename, TRUE);
 }
 
 int memo_export(GtkWidget *window)
 {
    int w, h, x, y;
+   char *type_text[]={"Text", "CSV", NULL};
+   int type_int[]={EXPORT_TYPE_TEXT, EXPORT_TYPE_CSV};
 
    gdk_window_get_size(window->window, &w, &h);
    gdk_window_get_root_origin(window->window, &x, &y);
 
+#ifdef ENABLE_GTK2
+   w = gtk_paned_get_position(GTK_PANED(pane));
+#else
    w = GTK_PANED(pane)->handle_xpos;
+#endif
    x+=40;
 
-   export_gui(w, h, x, y, 1, sort_l,
+   export_gui(window,
+              w, h, x, y, 1, sort_l,
 	      PREF_MEMO_EXPORT_FILENAME,
+	      type_text,
+	      type_int,
 	      cb_memo_update_clist,
 	      cb_memo_export_done,
 	      cb_memo_export_ok
@@ -608,20 +642,29 @@ void cb_delete_memo(GtkWidget *widget,
    MyMemo *mmemo;
    int flag;
    int show_priv;
+   long char_set; /* JPA */
 
    mmemo = gtk_clist_get_row_data(GTK_CLIST(clist), clist_row_selected);
    if (mmemo < (MyMemo *)CLIST_MIN_DATA) {
       return;
    }
+   /* JPA convert to Palm character set */
+   get_pref(PREF_CHAR_SET, &char_set, NULL);
+   if (char_set != CHAR_SET_LATIN1) {
+      if (mmemo->memo.text)
+	charset_j2p((unsigned char *)mmemo->memo.text,
+		    strlen(mmemo->memo.text)+1, char_set);
+   }
+
    /* Do masking like Palm OS 3.5 */
-   show_priv = show_privates(GET_PRIVATES, NULL);
+   show_priv = show_privates(GET_PRIVATES);
    if ((show_priv != SHOW_PRIVATES) &&
        (mmemo->attrib & dlpRecAttrSecret)) {
       return;
    }
    /* End Masking */
-   jpilot_logf(LOG_DEBUG, "mmemo->unique_id = %d\n",mmemo->unique_id);
-   jpilot_logf(LOG_DEBUG, "mmemo->rt = %d\n",mmemo->rt);
+   jp_logf(JP_LOG_DEBUG, "mmemo->unique_id = %d\n",mmemo->unique_id);
+   jp_logf(JP_LOG_DEBUG, "mmemo->rt = %d\n",mmemo->rt);
    flag = GPOINTER_TO_INT(data);
    if ((flag==MODIFY_FLAG) || (flag==DELETE_FLAG)) {
       delete_pc_record(MEMO, mmemo, flag);
@@ -646,11 +689,12 @@ static void cb_category(GtkWidget *item, int selection)
       if (b==DIALOG_SAID_1) {
 	 cb_add_new_record(NULL, GINT_TO_POINTER(record_changed));
       }
+
       memo_category = selection;
-      jpilot_logf(LOG_DEBUG, "cb_category() cat=%d\n", memo_category);
+      jp_logf(JP_LOG_DEBUG, "cb_category() cat=%d\n", memo_category);
       memo_clear_details();
       memo_update_clist(clist, category_menu1, glob_memo_list, memo_category, TRUE);
-      jpilot_logf(LOG_DEBUG, "Leaving cb_category()\n");
+      jp_logf(JP_LOG_DEBUG, "Leaving cb_category()\n");
    }
 }
 
@@ -659,11 +703,14 @@ static int memo_clear_details()
    int new_cat;
    int sorted_position;
 
-   jpilot_logf(LOG_DEBUG, "memo_clear_details()\n");
+   jp_logf(JP_LOG_DEBUG, "memo_clear_details()\n");
+
    /* Need to disconnect these signals first */
-   set_new_button_to(NEW_FLAG);
    connect_changed_signals(DISCONNECT_SIGNALS);
 
+#ifdef ENABLE_GTK2
+   gtk_text_buffer_set_text(GTK_TEXT_BUFFER(memo_text_buffer), "", -1);
+#else
    gtk_text_freeze(GTK_TEXT(memo_text));
 
    gtk_text_set_point(GTK_TEXT(memo_text), 0);
@@ -671,6 +718,7 @@ static int memo_clear_details()
 			   gtk_text_get_length(GTK_TEXT(memo_text)));
 
    gtk_text_thaw(GTK_TEXT(memo_text));
+#endif
 
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(private_checkbox), FALSE);
 
@@ -681,7 +729,7 @@ static int memo_clear_details()
    }
    sorted_position = find_sorted_cat(new_cat);
    if (sorted_position<0) {
-      jpilot_logf(LOG_WARN, "Category is not legal\n");
+      jp_logf(JP_LOG_WARN, "Category is not legal\n");
    } else {
       gtk_check_menu_item_set_active
 	(GTK_CHECK_MENU_ITEM(memo_cat_menu_item2[sorted_position]), TRUE);
@@ -689,16 +737,23 @@ static int memo_clear_details()
    }
 
    set_new_button_to(CLEAR_FLAG);
-   jpilot_logf(LOG_DEBUG, "Leaving memo_clear_details()\n");
+   jp_logf(JP_LOG_DEBUG, "Leaving memo_clear_details()\n");
    return 0;
 }
 
 int memo_get_details(struct Memo *new_memo, unsigned char *attrib)
 {
    int i;
+#ifdef ENABLE_GTK2
+   GtkTextIter start_iter;
+   GtkTextIter end_iter;
 
+   gtk_text_buffer_get_bounds(GTK_TEXT_BUFFER(memo_text_buffer),&start_iter,&end_iter);
+   new_memo->text = gtk_text_buffer_get_text(GTK_TEXT_BUFFER(memo_text_buffer),&start_iter,&end_iter,TRUE);
+#else
    new_memo->text = gtk_editable_get_chars
      (GTK_EDITABLE(memo_text), 0, -1);
+#endif
    if (new_memo->text[0]=='\0') {
       free(new_memo->text);
       new_memo->text=NULL;
@@ -722,7 +777,7 @@ int memo_get_details(struct Memo *new_memo, unsigned char *attrib)
 static void cb_use_memo32(GtkWidget *widget,
 			  gpointer   data)
 {
-   set_pref(PREF_MEMO32_MODE, GTK_TOGGLE_BUTTON(memo32_checkbox)->active, NULL);
+   set_pref(PREF_MEMO32_MODE, GTK_TOGGLE_BUTTON(memo32_checkbox)->active, NULL, TRUE);
    cb_app_button(NULL, GINT_TO_POINTER(REDRAW));
 }
 
@@ -737,10 +792,13 @@ static void cb_add_new_record(GtkWidget *widget, gpointer data)
 
    flag=GPOINTER_TO_INT(data);
 
+   mmemo=NULL;
+   unique_id=0;
+
    /* Do masking like Palm OS 3.5 */
    if ((GPOINTER_TO_INT(data)==COPY_FLAG) || 
        (GPOINTER_TO_INT(data)==MODIFY_FLAG)) {
-      show_priv = show_privates(GET_PRIVATES, NULL);
+      show_priv = show_privates(GET_PRIVATES);
       mmemo = gtk_clist_get_row_data(GTK_CLIST(clist), clist_row_selected);
       if (mmemo < (MyMemo *)CLIST_MIN_DATA) {
 	 return;
@@ -764,11 +822,12 @@ static void cb_add_new_record(GtkWidget *widget, gpointer data)
    }
    if (flag==MODIFY_FLAG) {
       mmemo = gtk_clist_get_row_data(GTK_CLIST(clist), clist_row_selected);
+      unique_id = mmemo->unique_id;
       if (mmemo < (MyMemo *)CLIST_MIN_DATA) {
 	 return;
       }
       if ((mmemo->rt==DELETED_PALM_REC) || (mmemo->rt==MODIFIED_PALM_REC)) {
-	 jpilot_logf(LOG_INFO, "You can't modify a record that is deleted\n");
+	 jp_logf(JP_LOG_INFO, "You can't modify a record that is deleted\n");
 	 return;
       }
    }
@@ -776,13 +835,23 @@ static void cb_add_new_record(GtkWidget *widget, gpointer data)
 
    set_new_button_to(CLEAR_FLAG);
 
-   pc_memo_write(&new_memo, NEW_PC_REC, attrib, &unique_id);
-   free_Memo(&new_memo);
+   /* Keep unique ID intact */
    if (flag==MODIFY_FLAG) {
       cb_delete_memo(NULL, data);
+      if ((mmemo->rt==PALM_REC) || (mmemo->rt==REPLACEMENT_PALM_REC)) {
+	 pc_memo_write(&new_memo, REPLACEMENT_PALM_REC, attrib, &unique_id);
+      } else {
+	 unique_id=0;
+	 pc_memo_write(&new_memo, NEW_PC_REC, attrib, &unique_id);
+      }
    } else {
-      memo_clist_redraw();
+      unique_id=0;
+      pc_memo_write(&new_memo, NEW_PC_REC, attrib, &unique_id);
    }
+
+   free_Memo(&new_memo);
+   memo_clist_redraw();
+
    glob_find_id = unique_id;
    memo_find();
 
@@ -804,6 +873,55 @@ static void clear_mymemo(MyMemo *mmemo)
 /* End Masking */
 
 
+static void cb_edit_cats(GtkWidget *widget, gpointer data)
+{
+   struct MemoAppInfo ai;
+   char db_name[FILENAME_MAX];
+   char pdb_name[FILENAME_MAX];
+   char full_name[FILENAME_MAX];
+   unsigned char buffer[65536];
+   int num, r;
+   int size;
+   void *buf;
+   long ivalue;
+   struct pi_file *pf;
+
+   jp_logf(JP_LOG_DEBUG, "cb_edit_cats\n");
+
+   get_pref(PREF_MEMO32_MODE, &ivalue, NULL);
+   if (ivalue) {
+      strcpy(pdb_name, "Memo32DB.pdb");
+      strcpy(db_name, "Memo32DB");
+   } else {
+      strcpy(pdb_name, "MemoDB.pdb");
+      strcpy(db_name, "MemoDB");
+   }
+   get_home_file_name(pdb_name, full_name, sizeof(full_name));
+
+   buf=NULL;
+   memset(&ai, 0, sizeof(ai));
+
+   pf = pi_file_open(full_name);
+   r = pi_file_get_app_info(pf, &buf, &size);
+
+   num = unpack_MemoAppInfo(&ai, buf, size);
+   if (num <= 0) {
+      jp_logf(JP_LOG_WARN, _("Error reading %s\n"), pdb_name);
+      return;
+   }
+
+   pi_file_close(pf);
+
+   edit_cats(widget, db_name, &(ai.category));
+
+   size = pack_MemoAppInfo(&ai, buffer, sizeof(buffer));
+
+   pdb_file_write_app_block("MemoDB", buffer, size);
+
+   cb_app_button(NULL, GINT_TO_POINTER(REDRAW));
+}
+
+
 static void cb_clist_selection(GtkWidget      *clist,
 			       gint           row,
 			       gint           column,
@@ -818,7 +936,7 @@ static void cb_clist_selection(GtkWidget      *clist,
 
    if ((!event) && (clist_hack)) return;
 
-   /* HACK */
+   /* HACK, see clist hack explanation in memo_gui.c */
    if (clist_hack) {
       keep=record_changed;
       gtk_clist_select_row(GTK_CLIST(clist), clist_row_selected, column);
@@ -851,7 +969,7 @@ static void cb_clist_selection(GtkWidget      *clist,
    sorted_position = find_sorted_cat(index);
    if (memo_cat_menu_item2[sorted_position]==NULL) {
       /* Illegal category */
-      jpilot_logf(LOG_DEBUG, "Category is not legal\n");
+      jp_logf(JP_LOG_DEBUG, "Category is not legal\n");
       index = sorted_position = 0;
    }
    /* We need to count how many items down in the list this is */
@@ -863,13 +981,16 @@ static void cb_clist_selection(GtkWidget      *clist,
    count--;
 
    if (sorted_position<0) {
-      jpilot_logf(LOG_WARN, "Category is not legal\n");
+      jp_logf(JP_LOG_WARN, "Category is not legal\n");
    } else {
       gtk_check_menu_item_set_active
 	(GTK_CHECK_MENU_ITEM(memo_cat_menu_item2[sorted_position]), TRUE);
    }
    gtk_option_menu_set_history(GTK_OPTION_MENU(category_menu2), count);
 
+#ifdef ENABLE_GTK2
+   gtk_text_buffer_set_text(GTK_TEXT_BUFFER(memo_text_buffer), memo->text, -1);
+#else
    gtk_text_freeze(GTK_TEXT(memo_text));
 
    gtk_text_set_point(GTK_TEXT(memo_text), 0);
@@ -879,6 +1000,7 @@ static void cb_clist_selection(GtkWidget      *clist,
    gtk_text_insert(GTK_TEXT(memo_text), NULL,NULL,NULL, memo->text, -1);
 
    gtk_text_thaw(GTK_TEXT(memo_text));
+#endif
 
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(private_checkbox),
 				mmemo->attrib & dlpRecAttrSecret);
@@ -889,29 +1011,39 @@ static void cb_clist_selection(GtkWidget      *clist,
 static void memo_update_clist(GtkWidget *clist, GtkWidget *tooltip_widget,
 			      MemoList *memo_list, int category, int main)
 {
-#define MEMO_CLIST_CHAR_WIDTH 50
    int num_entries, entries_shown, i;
    int row_count;
    size_t copy_max_length;
    char *last;
    gchar *empty_line[] = { "" };
-   GdkColor color;
-   GdkColormap *colormap;
+   char str2[MEMO_MAX_COLUMN_LEN];
    MemoList *temp_memo;
    char str[MEMO_CLIST_CHAR_WIDTH+10];
    int len, len1;
    int show_priv;
 
-   jpilot_logf(LOG_DEBUG, "memo_update_clist()\n");
+   jp_logf(JP_LOG_DEBUG, "memo_update_clist()\n");
 
-   row_count=((GtkCList *)clist)->rows;
+   row_count=(GTK_CLIST(clist))->rows;
+
+   /* Need to remove pointers to data we are about to delete */
+   for (i=0; i<row_count; i++) {
+      gtk_clist_set_row_data(GTK_CLIST(clist), i, NULL);
+   }
 
    free_MemoList(&memo_list);
 
-   /* Need to get the private ones back for the hints calculation */
+   /* Need to get all records including private ones for the tooltips calculation */
    num_entries = get_memos2(&memo_list, SORT_ASCENDING, 2, 2, 1, CATEGORY_ALL);
 
+   /* Start by clearing existing entry */
+   memo_clear_details();
+
    if (memo_list==NULL) {
+      /* Remove row 1 if the last item has just been deleted */
+      if (row_count == 1) {
+         gtk_clist_remove(GTK_CLIST(clist),0);
+      }
       if (tooltip_widget) {
 	 gtk_tooltips_set_tip(glob_tooltips, tooltip_widget, _("0 records"), NULL);
       }
@@ -920,7 +1052,7 @@ static void memo_update_clist(GtkWidget *clist, GtkWidget *tooltip_widget,
 
    gtk_clist_freeze(GTK_CLIST(clist));
 
-   show_priv = show_privates(GET_PRIVATES, NULL);
+   show_priv = show_privates(GET_PRIVATES);
 
    for (temp_memo = memo_list; temp_memo; temp_memo=temp_memo->next) {
       if ( ((temp_memo->mmemo.attrib & 0x0F) != category) &&
@@ -950,7 +1082,7 @@ static void memo_update_clist(GtkWidget *clist, GtkWidget *tooltip_widget,
 	 gtk_clist_set_text(GTK_CLIST(clist), entries_shown, 0, "----------------------------------------");
 	 clear_mymemo(&temp_memo->mmemo);
 	 gtk_clist_set_row_data(GTK_CLIST(clist), entries_shown, &(temp_memo->mmemo));
-	 gtk_clist_set_background(GTK_CLIST(clist), entries_shown, NULL);
+	 gtk_clist_set_row_style(GTK_CLIST(clist), entries_shown, NULL);
 	 entries_shown++;
 	 continue;
       }
@@ -975,52 +1107,38 @@ static void memo_update_clist(GtkWidget *clist, GtkWidget *tooltip_widget,
       if (last) {
 	 *(last-1)='\0';
       } else {
-	 str[copy_max_length]='\0';
+	 str[copy_max_length + len1]='\0';
       }
-      gtk_clist_set_text(GTK_CLIST(clist), entries_shown, 0, str);
+      lstrncpy_remove_cr_lfs(str2, str, MEMO_MAX_COLUMN_LEN);
+      gtk_clist_set_text(GTK_CLIST(clist), entries_shown, 0, str2);
       gtk_clist_set_row_data(GTK_CLIST(clist), entries_shown, &(temp_memo->mmemo));
 
       switch (temp_memo->mmemo.rt) {
        case NEW_PC_REC:
-	 colormap = gtk_widget_get_colormap(clist);
-	 color.red=CLIST_NEW_RED;
-	 color.green=CLIST_NEW_GREEN;
-	 color.blue=CLIST_NEW_BLUE;
-	 gdk_color_alloc(colormap, &color);
-	 gtk_clist_set_background(GTK_CLIST(clist), entries_shown, &color);
+       case REPLACEMENT_PALM_REC:
+	 set_bg_rgb_clist_row(clist, entries_shown,
+			  CLIST_NEW_RED, CLIST_NEW_GREEN, CLIST_NEW_BLUE);
 	 break;
        case DELETED_PALM_REC:
-	 colormap = gtk_widget_get_colormap(clist);
-	 color.red=CLIST_DEL_RED;
-	 color.green=CLIST_DEL_GREEN;
-	 color.blue=CLIST_DEL_BLUE;
-	 gdk_color_alloc(colormap, &color);
-	 gtk_clist_set_background(GTK_CLIST(clist), entries_shown, &color);
+	 set_bg_rgb_clist_row(clist, entries_shown,
+			  CLIST_DEL_RED, CLIST_DEL_GREEN, CLIST_DEL_BLUE);
 	 break;
        case MODIFIED_PALM_REC:
-	 colormap = gtk_widget_get_colormap(clist);
-	 color.red=CLIST_MOD_RED;
-	 color.green=CLIST_MOD_GREEN;
-	 color.blue=CLIST_MOD_BLUE;
-	 gdk_color_alloc(colormap, &color);
-	 gtk_clist_set_background(GTK_CLIST(clist), entries_shown, &color);
+	 set_bg_rgb_clist_row(clist, entries_shown,
+			  CLIST_MOD_RED, CLIST_MOD_GREEN, CLIST_MOD_BLUE);
 	 break;
        default:
 	 if (temp_memo->mmemo.attrib & dlpRecAttrSecret) {
-	    colormap = gtk_widget_get_colormap(clist);
-	    color.red=CLIST_PRIVATE_RED;
-	    color.green=CLIST_PRIVATE_GREEN;
-	    color.blue=CLIST_PRIVATE_BLUE;
-	    gdk_color_alloc(colormap, &color);
-	    gtk_clist_set_background(GTK_CLIST(clist), entries_shown, &color);
+	    set_bg_rgb_clist_row(clist, entries_shown, 
+			     CLIST_PRIVATE_RED, CLIST_PRIVATE_GREEN, CLIST_PRIVATE_BLUE);
 	 } else {
-	    gtk_clist_set_background(GTK_CLIST(clist), entries_shown, NULL);
+	    gtk_clist_set_row_style(GTK_CLIST(clist), entries_shown, NULL);
 	 }
       }
       entries_shown++;
    }
 
-   jpilot_logf(LOG_DEBUG, "entries_shown=%d\n", entries_shown);
+   jp_logf(JP_LOG_DEBUG, "entries_shown=%d\n", entries_shown);
 
    for (i=row_count-1; i>=entries_shown; i--) {
       gtk_clist_set_row_data(GTK_CLIST(clist), i, NULL);
@@ -1030,13 +1148,13 @@ static void memo_update_clist(GtkWidget *clist, GtkWidget *tooltip_widget,
    /*If there is an item in the list, select the first one */
    if ((main) && (entries_shown>0)) {
       gtk_clist_select_row(GTK_CLIST(clist), 0, 0);
-      cb_clist_selection(clist, 0, 0, (gpointer)455, NULL);
+      cb_clist_selection(clist, 0, 0, (GdkEventButton *)455, NULL);
    }
 
    gtk_clist_thaw(GTK_CLIST(clist));
 
-   sprintf(str, _("%d of %d records"), entries_shown, num_entries);
    if (tooltip_widget) {
+      sprintf(str, _("%d of %d records"), entries_shown, num_entries);
       gtk_tooltips_set_tip(glob_tooltips, tooltip_widget, str, NULL);
    }
 
@@ -1044,7 +1162,7 @@ static void memo_update_clist(GtkWidget *clist, GtkWidget *tooltip_widget,
       connect_changed_signals(CONNECT_SIGNALS);
    }
 
-   jpilot_logf(LOG_DEBUG, "Leaving memo_update_clist()\n");
+   jp_logf(JP_LOG_DEBUG, "Leaving memo_update_clist()\n");
 }
 
 static int memo_find()
@@ -1063,8 +1181,7 @@ static int memo_find()
 	 gtk_clist_select_row(GTK_CLIST(clist), found_at, 0);
 	 cb_clist_selection(clist, found_at, 0, (gpointer)455, NULL);
 	 if (!gtk_clist_row_is_visible(GTK_CLIST(clist), found_at)) {
-	    move_scrolled_window_hack(scrolled_window,
-				      (float)found_at/(float)total_count);
+	    gtk_clist_moveto(GTK_CLIST(clist), found_at, 0, 0.5, 0.0);
 	 }
       }
       glob_find_id = 0;
@@ -1087,6 +1204,35 @@ int memo_clist_redraw()
    return 0;
 }
 
+int memo_cycle_cat()
+{
+   int b;
+   int i, new_cat;
+
+   b=dialog_save_changed_record(pane, record_changed);
+   if (b==DIALOG_SAID_1) {
+      cb_add_new_record(NULL, GINT_TO_POINTER(record_changed));
+   }
+
+   if (memo_category == CATEGORY_ALL) {
+      new_cat = -1;
+   } else {
+      new_cat = find_sorted_cat(memo_category);
+   }
+   for (i=0; i<NUM_MEMO_CAT_ITEMS; i++) {
+      new_cat++;
+      if (new_cat >= NUM_MEMO_CAT_ITEMS) {
+	 memo_category = CATEGORY_ALL;
+	 break;
+      }
+      if ((sort_l[new_cat].Pcat) && (sort_l[new_cat].Pcat[0])) {
+	 memo_category = sort_l[new_cat].cat_num;
+	 break;
+      }
+   }
+   return 0;
+}
+
 int memo_refresh()
 {
    int index;
@@ -1101,7 +1247,7 @@ int memo_refresh()
    }
    memo_update_clist(clist, category_menu1, glob_memo_list, memo_category, TRUE);
    if (index<0) {
-      jpilot_logf(LOG_WARN, "Category not legal\n");
+      jp_logf(JP_LOG_WARN, "Category not legal\n");
    } else {	 
       gtk_option_menu_set_history(GTK_OPTION_MENU(category_menu1), index);
       gtk_check_menu_item_set_active
@@ -1121,7 +1267,13 @@ int memo_gui_cleanup()
    }
    free_MemoList(&glob_memo_list);
    connect_changed_signals(DISCONNECT_SIGNALS);
-   set_pref(PREF_MEMO_PANE, GTK_PANED(pane)->handle_xpos, NULL);
+#ifdef ENABLE_GTK2
+   set_pref(PREF_MEMO_PANE, gtk_paned_get_position(GTK_PANED(pane)), NULL, TRUE);
+#else
+   set_pref(PREF_MEMO_PANE, GTK_PANED(pane)->handle_xpos, NULL, TRUE);
+#endif
+   set_pref(PREF_LAST_MEMO_CATEGORY, memo_category, NULL, TRUE);
+
    return 0;
 }
 
@@ -1133,10 +1285,13 @@ int memo_gui(GtkWidget *vbox, GtkWidget *hbox)
    extern GtkWidget *glob_date_label;
    extern int glob_date_timer_tag;
    int i;
+   GtkWidget *scrolled_window;
    GtkWidget *vbox1, *vbox2, *hbox_temp;
    GtkWidget *separator;
    GtkWidget *button;
+#ifndef ENABLE_GTK2
    GtkWidget *vscrollbar;
+#endif
    long ivalue;
    const char *svalue;
 
@@ -1162,6 +1317,9 @@ int memo_gui(GtkWidget *vbox, GtkWidget *hbox)
    }
 #endif
 
+   get_pref(PREF_LAST_MEMO_CATEGORY, &ivalue, NULL);
+   memo_category = ivalue;
+
    if ((memo_category != CATEGORY_ALL) && (memo_app_info.category.name[memo_category][0]=='\0')) {
       memo_category=CATEGORY_ALL;
    }
@@ -1197,10 +1355,30 @@ int memo_gui(GtkWidget *vbox, GtkWidget *hbox)
    gtk_box_pack_start(GTK_BOX(vbox1), separator, FALSE, FALSE, 5);
 
 
+   /* Category Box */
+   hbox_temp = gtk_hbox_new(FALSE, 0);
+   gtk_box_pack_start(GTK_BOX(vbox1), hbox_temp, FALSE, FALSE, 0);
+
    /* Put the left-hand category menu up */
    make_category_menu(&category_menu1, memo_cat_menu_item1,
 		      sort_l, cb_category, TRUE);
-   gtk_box_pack_start(GTK_BOX(vbox1), category_menu1, FALSE, FALSE, 0);
+   gtk_box_pack_start(GTK_BOX(hbox_temp), category_menu1, TRUE, TRUE, 0);
+
+
+   /* Edit category button */
+   button = gtk_button_new_with_label(_("Edit Categories"));
+   gtk_signal_connect(GTK_OBJECT(button), "clicked",
+		      GTK_SIGNAL_FUNC(cb_edit_cats), NULL);
+   gtk_box_pack_start(GTK_BOX(hbox_temp), button, FALSE, FALSE, 0);
+
+
+   /* Memo32 check box */
+   memo32_checkbox = gtk_check_button_new_with_label(_("Use Memo32 (pedit32)"));
+   gtk_box_pack_start(GTK_BOX(vbox1), memo32_checkbox, FALSE, FALSE, 0);
+   get_pref(PREF_MEMO32_MODE, &ivalue, NULL);
+   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memo32_checkbox), ivalue);
+   gtk_signal_connect(GTK_OBJECT(memo32_checkbox), "clicked",
+		      GTK_SIGNAL_FUNC(cb_use_memo32), NULL);
 
 
    /* Put the memo list window up */
@@ -1218,6 +1396,7 @@ int memo_gui(GtkWidget *vbox, GtkWidget *hbox)
 		      GTK_SIGNAL_FUNC(cb_clist_selection), NULL);
    gtk_clist_set_shadow_type(GTK_CLIST(clist), SHADOW);
    /*
+    * clist hack explanation:
     * There is a strange hack here.
     * If a clist is in browse mode and a callback is called on it and then
     * a dialog window is opened (which we do if a record has changed, and we
@@ -1231,6 +1410,9 @@ int memo_gui(GtkWidget *vbox, GtkWidget *hbox)
 
    gtk_container_add(GTK_CONTAINER(scrolled_window), GTK_WIDGET(clist));
 
+   /* The focus doesn't do any good on the application button */
+   gtk_widget_grab_focus(GTK_WIDGET(clist));
+
    /*
     * The right hand part of the main window follows:
     */
@@ -1239,17 +1421,17 @@ int memo_gui(GtkWidget *vbox, GtkWidget *hbox)
 
 
    /* Add record modification buttons on right side */
-   button = gtk_button_new_with_label(_("Delete"));
-   gtk_signal_connect(GTK_OBJECT(button), "clicked",
+   delete_record_button = gtk_button_new_with_label(_("Delete"));
+   gtk_signal_connect(GTK_OBJECT(delete_record_button), "clicked",
 		      GTK_SIGNAL_FUNC(cb_delete_memo),
 		      GINT_TO_POINTER(DELETE_FLAG));
-   gtk_box_pack_start(GTK_BOX(hbox_temp), button, TRUE, TRUE, 0);
+   gtk_box_pack_start(GTK_BOX(hbox_temp), delete_record_button, TRUE, TRUE, 0);
 
-   button = gtk_button_new_with_label(_("Copy"));
-   gtk_signal_connect(GTK_OBJECT(button), "clicked",
+   copy_record_button = gtk_button_new_with_label(_("Copy"));
+   gtk_signal_connect(GTK_OBJECT(copy_record_button), "clicked",
 		      GTK_SIGNAL_FUNC(cb_add_new_record), 
 		      GINT_TO_POINTER(COPY_FLAG));
-   gtk_box_pack_start(GTK_BOX(hbox_temp), button, TRUE, TRUE, 0);
+   gtk_box_pack_start(GTK_BOX(hbox_temp), copy_record_button, TRUE, TRUE, 0);
 
    new_record_button = gtk_button_new_with_label(_("New Record"));
    gtk_signal_connect(GTK_OBJECT(new_record_button), "clicked",
@@ -1279,15 +1461,6 @@ int memo_gui(GtkWidget *vbox, GtkWidget *hbox)
    gtk_box_pack_start(GTK_BOX(vbox2), separator, FALSE, FALSE, 5);
 
 
-   /* Memo32 check box */
-   memo32_checkbox = gtk_check_button_new_with_label(_("Use Memo32 (pedit32)"));
-   gtk_box_pack_start(GTK_BOX(vbox2), memo32_checkbox, FALSE, FALSE, 0);
-   get_pref(PREF_MEMO32_MODE, &ivalue, NULL);
-   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(memo32_checkbox), ivalue);
-   gtk_signal_connect(GTK_OBJECT(memo32_checkbox), "clicked",
-		      GTK_SIGNAL_FUNC(cb_use_memo32), NULL);
-
-
    /*Private check box */
    hbox_temp = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(vbox2), hbox_temp, FALSE, FALSE, 0);
@@ -1309,12 +1482,26 @@ int memo_gui(GtkWidget *vbox, GtkWidget *hbox)
    hbox_temp = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start(GTK_BOX(vbox2), hbox_temp, TRUE, TRUE, 0);
 
+#ifdef ENABLE_GTK2
+   memo_text = gtk_text_view_new();
+   memo_text_buffer = G_OBJECT(gtk_text_view_get_buffer(GTK_TEXT_VIEW(memo_text)));
+   gtk_text_view_set_editable(GTK_TEXT_VIEW(memo_text), TRUE);
+   gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(memo_text), GTK_WRAP_WORD);
+
+   scrolled_window = gtk_scrolled_window_new(NULL, NULL);
+   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
+				  GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
+   gtk_container_set_border_width(GTK_CONTAINER(scrolled_window), 1);
+   gtk_container_add(GTK_CONTAINER(scrolled_window), memo_text);
+   gtk_box_pack_start_defaults(GTK_BOX(hbox_temp), scrolled_window);
+#else
    memo_text = gtk_text_new(NULL, NULL);
    gtk_text_set_editable(GTK_TEXT(memo_text), TRUE);
    gtk_text_set_word_wrap(GTK_TEXT(memo_text), TRUE);
    vscrollbar = gtk_vscrollbar_new(GTK_TEXT(memo_text)->vadj);
    gtk_box_pack_start(GTK_BOX(hbox_temp), memo_text, TRUE, TRUE, 0);
    gtk_box_pack_start(GTK_BOX(hbox_temp), vscrollbar, FALSE, FALSE, 0);
+#endif
 
    gtk_widget_show_all(vbox);
    gtk_widget_show_all(hbox);
@@ -1323,7 +1510,6 @@ int memo_gui(GtkWidget *vbox, GtkWidget *hbox)
    gtk_widget_hide(apply_record_button);
 
    memo_refresh();
-   memo_find();
 
    return 0;
 }
