@@ -1,7 +1,7 @@
 /* password.c
  * A module of J-Pilot http://jpilot.org
  *
- * Copyright (C) 1999-2001 by Judd Montgomery
+ * Copyright (C) 1999-2002 by Judd Montgomery
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,6 +26,27 @@
 #include "prefs.h"
 #include "i18n.h"
 
+#include <pi-version.h>
+
+/* Try to determine if version of pilot-link > 0.9.x */
+#ifdef USB_PILOT_LINK
+# undef USB_PILOT_LINK
+#endif
+
+#if PILOT_LINK_VERSION > 0
+# define USB_PILOT_LINK
+#else
+# if PILOT_LINK_MAJOR > 9
+#  define USB_PILOT_LINK
+# endif
+#endif
+
+#ifdef USB_PILOT_LINK
+#include <pi-md5.h>
+#endif
+
+#ifdef ENABLE_PRIVATE
+
 struct dialog_data {
    GtkWidget *entry;
    int button_hit;
@@ -35,7 +56,6 @@ struct dialog_data {
 #define DIALOG_SAID_1  454
 #define DIALOG_SAID_2  455
 
-#ifdef ENABLE_PRIVATE
 unsigned char short_salt[]=
 {
    0x09, 0x02, 0x13, 0x45, 0x07, 0x04, 0x13, 0x44, 
@@ -67,8 +87,11 @@ void bin_to_hex_str(unsigned char *bin, char *hex_str, int len)
 
 /*
  * encoded is a pre-allocated buffer at least 34 bytes long
+ * 
+ * This is the hashing algorithm used in palm OS < 4.0.
+ * It is not very secure and is reversible.
  */
-void palm_encode(unsigned char *ascii, unsigned char *encoded)
+void palm_encode_hash(unsigned char *ascii, unsigned char *encoded)
 {
    unsigned char index, shift;
    unsigned short temp;
@@ -80,17 +103,10 @@ void palm_encode(unsigned char *ascii, unsigned char *encoded)
    int mi;
    int m;
 
-   /* It seems that Palm OS lower cases the password first */
-   /* Yes, I have found this documented on Palms site */
-   len = strlen(encoded);
-   for (ai=0; ai < PASSWD_LEN; ai++) {
-      ascii[ai] = tolower(ascii[ai]);
-   }
-
    encoded[0]='\0';
    end=0;
-   if (strlen(ascii)<5) {
-      si = (ascii[0] + strlen(ascii)) % PASSWD_LEN;
+   if (strlen((char *)ascii)<5) {
+      si = (ascii[0] + strlen((char*)ascii)) % PASSWD_LEN;
       for (ai=ei=0; ei<32; ai++, ei++, si++) {
 	 if (ascii[ai]=='\0') {
 	    end=1;
@@ -106,7 +122,7 @@ void palm_encode(unsigned char *ascii, unsigned char *encoded)
 
    strncpy(encoded, ascii, PASSWD_LEN);
    encoded[PASSWD_LEN-1]='\0';
-   len = strlen(encoded);
+   len = strlen((char *)encoded);
    for (ai=len; ai < PASSWD_LEN; ai++) {
       encoded[ai] = encoded[ai-len] + len;
    }
@@ -127,6 +143,79 @@ void palm_encode(unsigned char *ascii, unsigned char *encoded)
    }  
 }
 
+#ifdef USB_PILOT_LINK
+/*
+ * encoded is a pre-allocated buffer at least 16 bytes long
+ * 
+ * This is the hashing algorithm used in palm OS >= 4.0.
+ * Its just a plain ole' MD5.
+ */
+void palm_encode_md5(unsigned char *ascii, unsigned char *encoded)
+{
+   struct MD5Context m;
+   unsigned char digest[20];
+   
+   MD5Init(&m);
+   MD5Update(&m, ascii, strlen((char *)ascii));
+   MD5Final(digest, &m);
+   
+   memcpy(encoded, digest, 16);   
+}
+#endif /* USB_PILOT_LINK */
+
+#endif
+
+int verify_password(char *password)
+{
+   int i;
+#ifdef ENABLE_PRIVATE
+   unsigned char encoded[34];
+   unsigned char password_lower[PASSWD_LEN+2];
+   const char *pref_password;
+   char hex_str[68];
+   long ivalue;
+#endif
+
+#ifdef ENABLE_PRIVATE
+   if (!password) {
+      return FALSE;
+   }
+
+   /* It seems that Palm OS lower cases the password first */
+   /* Yes, I have found this documented on Palms site */
+   for (i=0; i < PASSWD_LEN; i++) {
+      password_lower[i] = tolower(password[i]);
+   }
+
+   /* Check to see that the password matches */
+   palm_encode_hash(password_lower, encoded);
+   bin_to_hex_str(encoded, hex_str, 32);
+   get_pref(PREF_PASSWORD, &ivalue, &pref_password);
+   if (!strcmp(hex_str, pref_password)) {
+      return TRUE;
+   }
+
+# ifdef USB_PILOT_LINK
+   /* We need a new pilot-link > 0.11 for this */
+   /* The Password didn't match.
+    * It could also be an MD5 password.
+    * Try that now.
+    */
+   palm_encode_md5((unsigned char *)password, encoded);
+   bin_to_hex_str(encoded, hex_str, 32);
+   hex_str[32]='\0';
+   get_pref(PREF_PASSWORD, &ivalue, &pref_password);
+   if (!strcmp(hex_str, pref_password)) {
+      return TRUE;
+   }
+# endif /* USB_PILOT_LINK */
+   return FALSE;
+#else
+   /* Return TRUE without checking the password */
+   return TRUE;
+#endif
+}
+
 /*
  * hide passed HIDE_PRIVATES will set the hide flag.
  * hide passed SHOW_PRIVATES will unset the hide flag and also need a
@@ -136,14 +225,14 @@ void palm_encode(unsigned char *ascii, unsigned char *encoded)
  * hide passed GET_PRIVATES will return the current hide flag.
  * hide flag is always returned, it is boolean.
  */
-int show_privates(int hide, char *password)
+int show_privates(int hide)
 {
+#ifdef ENABLE_PRIVATE
    static int hidden=HIDE_PRIVATES;
-   unsigned char encoded[34];
-   unsigned char hex_str[68];
-   unsigned long ivalue;
-   const char *pref_password;
-
+#else
+   static int hidden=SHOW_PRIVATES;
+#endif
+   
    if (hide==GET_PRIVATES) {
       return hidden;
    }
@@ -157,22 +246,14 @@ int show_privates(int hide, char *password)
    }
    if ( (hide==SHOW_PRIVATES) ||
        ((hide==MASK_PRIVATES) && (hidden!=SHOW_PRIVATES)) ) {
-      /* Need to have the proper password */
-      if (!password) {
-	 return HIDE_PRIVATES;
-      }
-      palm_encode(password, encoded);
-      bin_to_hex_str(encoded, hex_str, 32);
-      get_pref(PREF_PASSWORD, &ivalue, &pref_password);
-      if (!strcmp(hex_str, pref_password)) {
-	 hidden=SHOW_PRIVATES;
-      }
+
+      hidden=SHOW_PRIVATES;
    }
 
    return hidden;
 }
-#endif
 
+#ifdef ENABLE_PRIVATE
 /*
  * PASSWORD GUI
  */
@@ -184,23 +265,20 @@ static void cb_dialog_button(GtkWidget *widget,
 {
    struct dialog_data *Pdata;
    GtkWidget *w;
-   int i;
 
-   for (w=widget, i=10; w && (i>0); w=w->parent, i--) {
-      if (GTK_IS_WINDOW(w)) {
-	 Pdata = gtk_object_get_data(GTK_OBJECT(w), "dialog_data");
-	 if (Pdata) {
-	    Pdata->button_hit = GPOINTER_TO_INT(data);
-	 }
-	 gtk_widget_destroy(GTK_WIDGET(w));
-      }
+   w = gtk_widget_get_toplevel(widget);
+
+   Pdata = gtk_object_get_data(GTK_OBJECT(w), "dialog_data");
+   if (Pdata) {
+      Pdata->button_hit = GPOINTER_TO_INT(data);
    }
+   gtk_widget_destroy(GTK_WIDGET(w));
 }
 
 static gboolean cb_destroy_dialog(GtkWidget *widget)
 {
    struct dialog_data *Pdata;
-   char *entry;
+   const char *entry;
 
    Pdata = gtk_object_get_data(GTK_OBJECT(widget), "dialog_data");
    if (!Pdata) {
@@ -221,7 +299,7 @@ static gboolean cb_destroy_dialog(GtkWidget *widget)
 /*
  * returns 1 if OK was pressed, 2 if cancel was hit
  */
-int dialog_password(char *ascii_password)
+int dialog_password(GtkWindow *main_window, char *ascii_password, int retry)
 {
    GtkWidget *button, *label;
    GtkWidget *hbox1, *vbox1;
@@ -249,6 +327,8 @@ int dialog_password(char *ascii_password)
 
    gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);
 
+   gtk_window_set_transient_for(GTK_WINDOW(dialog), GTK_WINDOW(main_window));
+
    vbox1 = gtk_vbox_new(FALSE, 2);
 
    gtk_container_set_border_width(GTK_CONTAINER(vbox1), 5);
@@ -260,7 +340,11 @@ int dialog_password(char *ascii_password)
    gtk_box_pack_start(GTK_BOX(vbox1), hbox1, FALSE, FALSE, 2);
 
    /* Label */
-   label = gtk_label_new(_("Enter PalmOS Password"));
+   if (retry) {
+      label = gtk_label_new(_("Incorrect, Reenter PalmOS Password"));
+   } else {
+      label = gtk_label_new(_("Enter PalmOS Password"));
+   }
    gtk_box_pack_start(GTK_BOX(hbox1), label, FALSE, FALSE, 2);
 
    entry = gtk_entry_new_with_max_length(32);
@@ -317,3 +401,4 @@ int dialog_password(char *ascii_password)
 
    return ret;
 }
+#endif
