@@ -1,7 +1,7 @@
 /* plugins.c
  * A module of J-Pilot http://jpilot.org
  *
- * Copyright (C) 1999-2001 by Judd Montgomery
+ * Copyright (C) 1999-2002 by Judd Montgomery
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,10 +30,10 @@
 #include <dlfcn.h>
 
 GList *plugins = NULL;
-GList *plugins_eol = NULL;
 
 static int get_plugin_info(struct plugin_s *p, char *path);
 static int get_plugin_sync_bits();
+gint plugin_sort(gconstpointer a, gconstpointer b);
 
 /*
  * Write out the jpilot.plugins file that tells which plugins to sync
@@ -44,7 +44,7 @@ void write_plugin_sync_file()
    GList *temp_list;
    struct plugin_s *Pplugin;
 
-   out=jp_open_home_file("jpilot.plugins", "w");
+   out=jp_open_home_file(EPN".plugins", "w");
    if (!out) {
       return;
    }
@@ -75,13 +75,13 @@ int load_plugins_sub1(DIR *dir, char *path, int *number, unsigned char user_only
    int i, r;
    int count;
    struct dirent *dirent;
-   char full_name[256];
+   char full_name[FILENAME_MAX];
    struct plugin_s temp_plugin, *new_plugin;
 
    count = 0;
    for (i=0; (dirent = readdir(dir)); i++) {
       if (i>1000) {
-	 jpilot_logf(LOG_WARN, "load_plugins_sub1(): infinite loop\n");
+	 jp_logf(JP_LOG_WARN, "load_plugins_sub1(): infinite loop\n");
 	 return 0;
       }
       /* If the filename has either of these extensions then plug it in */
@@ -89,43 +89,56 @@ int load_plugins_sub1(DIR *dir, char *path, int *number, unsigned char user_only
 	  (strcmp(&(dirent->d_name[strlen(dirent->d_name)-3]), ".sl"))) {
 	 continue;
       } else {
-	 jpilot_logf(LOG_DEBUG, "found plugin %s\n", dirent->d_name);
+	 jp_logf(JP_LOG_DEBUG, "found plugin %s\n", dirent->d_name);
 	 /* We know path has a trailing slash after it */
-	 g_snprintf(full_name, 250, "%s%s", path, dirent->d_name);
+	 g_snprintf(full_name, sizeof(full_name), "%s%s", path, dirent->d_name);
 	 r = get_plugin_info(&temp_plugin, full_name);
 	 temp_plugin.number = *number;
 	 temp_plugin.user_only = user_only;
 	 if (r==0) {
 	    if (temp_plugin.name) {
-	       jpilot_logf(LOG_DEBUG, "plugin name is [%s]\n", temp_plugin.name);
+	       jp_logf(JP_LOG_DEBUG, "plugin name is [%s]\n", temp_plugin.name);
 	    }
 	    new_plugin = malloc(sizeof(struct plugin_s));
 	    if (!new_plugin) {
-	       jpilot_logf(LOG_DEBUG, "load plugins(): Out of memory\n");
+	       jp_logf(JP_LOG_DEBUG, "load plugins(): Out of memory\n");
 	       return count;
 	    }
 	    memcpy(new_plugin, &temp_plugin, sizeof(struct plugin_s));
-	    if (plugins==NULL) {
-	       plugins = g_list_append(plugins, new_plugin);
-	       plugins_eol=plugins;
-	    } else {
-	       plugins = g_list_append(plugins_eol, new_plugin);
-	       if (plugins_eol->next) {
-		  plugins_eol=plugins_eol->next;
-	       }
-	    }
+	    /* g_list_append searches the list until it reaches the end
+	     * we want to avoid that slowness, prepend works for that
+	     * in this case since we have the head */
+	    plugins = g_list_prepend(plugins, new_plugin);
 	    count++;
 	    (*number)++;
 	 }
       }
    }
+
+   plugins = g_list_sort(plugins, plugin_sort);
+
    return count;
 }
 
+gint plugin_sort(gconstpointer a, gconstpointer b)
+{
+   const char *ca = ((struct plugin_s *)a)->menu_name;
+   const char *cb = ((struct plugin_s *)b)->menu_name;
+
+   /* menu_name is NULL for plugin without menu entry */
+   if (ca == NULL)
+     ca = ((struct plugin_s *)a)->name;
+
+   if (cb == NULL)
+     cb = ((struct plugin_s *)b)->name;
+
+   return strcasecmp(ca, cb);
+}
+
 int load_plugins()
 {
    DIR *dir;
-   char path[256];
+   char path[FILENAME_MAX];
    int count, number;
    GList *temp_list;
 
@@ -134,8 +147,8 @@ int load_plugins()
    plugins = NULL;
 
    /* ABILIB is for Irix, should normally be "lib" */
-   g_snprintf(path, 250, "%s/%s/%s/%s/", BASE_DIR, ABILIB, EPN, "plugins");
-   jpilot_logf(LOG_DEBUG, "opening dir %s\n", path);
+   g_snprintf(path, sizeof(path), "%s/%s/%s/%s/", BASE_DIR, ABILIB, EPN, "plugins");
+   jp_logf(JP_LOG_DEBUG, "opening dir %s\n", path);
    cleanup_path(path);
    dir = opendir(path);
    if (dir) {
@@ -143,9 +156,9 @@ int load_plugins()
       closedir(dir);
    }
 
-   get_home_file_name("plugins/", path, 240);
+   get_home_file_name("plugins/", path, sizeof(path));
    cleanup_path(path);
-   jpilot_logf(LOG_DEBUG, "opening dir %s\n", path);
+   jp_logf(JP_LOG_DEBUG, "opening dir %s\n", path);
    dir = opendir(path);
    if (dir) {
       count += load_plugins_sub1(dir, path, &number, 1);
@@ -174,13 +187,13 @@ static int get_plugin_sync_bits()
    char *Pc;
    FILE *in;
 
-   in=jp_open_home_file("jpilot.plugins", "r");
+   in=jp_open_home_file(EPN".plugins", "r");
    if (!in) {
       return 0;
    }
    for (i=0; (!feof(in)); i++) {   
       if (i>1000) {
-	 jpilot_logf(LOG_WARN, "load_plugins(): infinite loop\n");
+	 jp_logf(JP_LOG_WARN, "load_plugins(): infinite loop\n");
 	 fclose(in);
 	 return 0;
       }
@@ -193,9 +206,9 @@ static int get_plugin_sync_bits()
 	 line[strlen(line)-1]='\0';
       }
       if ((!strncmp(line, "Version", 7)) && (strcmp(line, "Version 1"))) {
-	 jpilot_logf(LOG_WARN, "While reading jpilot.plugins line 1:[%s]\n", line);
-	 jpilot_logf(LOG_WARN, "Wrong Version\n");
-	 jpilot_logf(LOG_WARN, "Check preferences->conduits\n");
+	 jp_logf(JP_LOG_WARN, "While reading "EPN".plugins line 1:[%s]\n", line);
+	 jp_logf(JP_LOG_WARN, "Wrong Version\n");
+	 jp_logf(JP_LOG_WARN, "Check preferences->conduits\n");
 	 fclose(in);
 	 return 0;
       }
@@ -241,18 +254,21 @@ static int get_plugin_info(struct plugin_s *p, char *path)
    p->plugin_import = NULL;
    p->plugin_export = NULL;
    p->plugin_gui_cleanup = NULL;
+   p->plugin_pre_sync_pre_connect = NULL;
    p->plugin_pre_sync = NULL;
    p->plugin_sync = NULL;
    p->plugin_post_sync = NULL;
    p->plugin_exit_cleanup = NULL;
+   p->plugin_unpack_cai_from_ai = NULL;
+   p->plugin_pack_cai_into_ai = NULL;
 
    h = dlopen(path, RTLD_LAZY);
    if (!h) {
-      jpilot_logf(LOG_WARN, "open failed on plugin [%s]\n error [%s]\n", path,
+      jp_logf(JP_LOG_WARN, "open failed on plugin [%s]\n error [%s]\n", path,
 		  dlerror());
       return -1;
    }
-   jpilot_logf(LOG_DEBUG, "opened plugin [%s]\n", path);
+   jp_logf(JP_LOG_DEBUG, "opened plugin [%s]\n", path);
    p->handle=h;
 
    p->full_path = strdup(path);
@@ -264,8 +280,8 @@ static int get_plugin_info(struct plugin_s *p, char *path)
    plugin_versionM = dlsym(h, "plugin_version");
    if (plugin_versionM==NULL)  {
       err = dlerror();
-      jpilot_logf(LOG_WARN, "plugin_version: [%s]\n", err);
-      jpilot_logf(LOG_WARN, " plugin is invalid: [%s]\n", path);
+      jp_logf(JP_LOG_WARN, "plugin_version: [%s]\n", err);
+      jp_logf(JP_LOG_WARN, " plugin is invalid: [%s]\n", path);
       dlclose(h);
       p->handle=NULL;
       return -1;
@@ -273,25 +289,25 @@ static int get_plugin_info(struct plugin_s *p, char *path)
    plugin_versionM(&major_version, &minor_version);
    version=major_version*1000+minor_version;
    if ((major_version <= 0) && (minor_version < 99)) {
-      jpilot_logf(LOG_WARN, "Plugin:[%s]\n", path);
-      jpilot_logf(LOG_WARN, "This plugin is version (%d.%d).\n",
+      jp_logf(JP_LOG_WARN, "Plugin:[%s]\n", path);
+      jp_logf(JP_LOG_WARN, "This plugin is version (%d.%d).\n",
 		  major_version, minor_version);
-      jpilot_logf(LOG_WARN, "It is too old to work with this version of J-Pilot.\n");
+      jp_logf(JP_LOG_WARN, "It is too old to work with this version of J-Pilot.\n");
       dlclose(h);
       p->handle=NULL;
       return -1;
    }
-   jpilot_logf(LOG_DEBUG, "This plugin is version (%d.%d).\n",
+   jp_logf(JP_LOG_DEBUG, "This plugin is version (%d.%d).\n",
 	       major_version, minor_version);
 
 
    /* plugin_get_name */
-   jpilot_logf(LOG_DEBUG, "getting plugin_get_name\n");
+   jp_logf(JP_LOG_DEBUG, "getting plugin_get_name\n");
    p->plugin_get_name = dlsym(h, "plugin_get_name");
    if (p->plugin_get_name==NULL)  {
       err = dlerror();
-      jpilot_logf(LOG_WARN, "plugin_get_name: [%s]\n", err);
-      jpilot_logf(LOG_WARN, " plugin is invalid: [%s]\n", path);
+      jp_logf(JP_LOG_WARN, "plugin_get_name: [%s]\n", err);
+      jp_logf(JP_LOG_WARN, " plugin is invalid: [%s]\n", path);
       dlclose(h);
       p->handle=NULL;
       return -1;
@@ -307,7 +323,7 @@ static int get_plugin_info(struct plugin_s *p, char *path)
 
 
    /* plugin_get_menu_name */
-   jpilot_logf(LOG_DEBUG, "getting plugin_get_menu_name\n");
+   jp_logf(JP_LOG_DEBUG, "getting plugin_get_menu_name\n");
    p->plugin_get_menu_name = dlsym(h, "plugin_get_menu_name");
    if (p->plugin_get_menu_name) {
       p->plugin_get_menu_name(name, 50);
@@ -319,7 +335,7 @@ static int get_plugin_info(struct plugin_s *p, char *path)
 
 
    /* plugin_get_help_name */
-   jpilot_logf(LOG_DEBUG, "getting plugin_get_help_name\n");
+   jp_logf(JP_LOG_DEBUG, "getting plugin_get_help_name\n");
    p->plugin_get_help_name = dlsym(h, "plugin_get_help_name");
    if (p->plugin_get_help_name) {
       p->plugin_get_help_name(name, 50);
@@ -330,7 +346,7 @@ static int get_plugin_info(struct plugin_s *p, char *path)
    }
 
    /* plugin_get_db_name */
-   jpilot_logf(LOG_DEBUG, "getting plugin_get_db_name\n");
+   jp_logf(JP_LOG_DEBUG, "getting plugin_get_db_name\n");
    p->plugin_get_db_name = dlsym(h, "plugin_get_db_name");
 
    if (p->plugin_get_db_name) {
@@ -367,6 +383,9 @@ static int get_plugin_info(struct plugin_s *p, char *path)
    /* plugin_pre_sync */
    p->plugin_pre_sync = dlsym(h, "plugin_pre_sync");
 
+   /* plugin_pre_sync_pre_connect */
+   p->plugin_pre_sync_pre_connect = dlsym(h, "plugin_pre_sync_pre_connect");
+
    /* plugin_sync */
    p->plugin_sync = dlsym(h, "plugin_sync");
 
@@ -379,6 +398,9 @@ static int get_plugin_info(struct plugin_s *p, char *path)
    /* plugin_exit_cleanup */
    p->plugin_exit_cleanup = dlsym(h, "plugin_exit_cleanup");
 
+   p->plugin_unpack_cai_from_ai = dlsym(h, "plugin_unpack_cai_from_ai");
+   p->plugin_pack_cai_into_ai = dlsym(h, "plugin_pack_cai_into_ai");
+
    return 0;
 }
 
