@@ -1,7 +1,7 @@
 /* prefs.c
  * A module of J-Pilot http://jpilot.org
  *
- * Copyright (C) 1999-2001 by Judd Montgomery
+ * Copyright (C) 1999-2002 by Judd Montgomery
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,7 +31,7 @@
 /*These are the default settings */
 /*name, usertype, filetype, ivalue, char *svalue, svalue_size; */
 static prefType glob_prefs[NUM_PREFS] = {
-     {"jpilotrc", CHARTYPE, CHARTYPE, 0, NULL, 0},
+     {EPN"rc", CHARTYPE, CHARTYPE, 0, NULL, 0},
      {"time", CHARTYPE, INTTYPE, 0, NULL, 0},
      {"sdate", CHARTYPE, INTTYPE, 0, NULL, 0},
      {"ldate", CHARTYPE, INTTYPE, 0, NULL, 0},
@@ -46,8 +46,8 @@ static prefType glob_prefs[NUM_PREFS] = {
      {"user_id", INTTYPE, INTTYPE, 0, NULL, 0},
      {"pc_id", INTTYPE, INTTYPE, 0, NULL, 0},
      {"num_backups", INTTYPE, INTTYPE, 2, NULL, 0},
-     {"window_width", INTTYPE, INTTYPE, 770, NULL, 0},
-     {"window_height", INTTYPE, INTTYPE, 500, NULL, 0},
+     {"window_width", INTTYPE, INTTYPE, 640, NULL, 0},
+     {"window_height", INTTYPE, INTTYPE, 480, NULL, 0},
      {"datebook_pane", INTTYPE, INTTYPE, 370, NULL, 0},
      {"address_pane", INTTYPE, INTTYPE, 340, NULL, 0},
      {"todo_pane", INTTYPE, INTTYPE, 370, NULL, 0},
@@ -58,7 +58,7 @@ static prefType glob_prefs[NUM_PREFS] = {
      {"print_one_per_page", INTTYPE, INTTYPE, 0, NULL ,0},
      {"print_blank_lines", INTTYPE, INTTYPE, 1, NULL, 0},
      {"print_command", CHARTYPE, CHARTYPE, 0, NULL, 0},
-     {"char_set", INTTYPE, INTTYPE, CHAR_SET_ENGLISH, NULL, 0},
+     {"char_set", INTTYPE, INTTYPE, CHAR_SET_LATIN1, NULL, 0},
      {"sync_datebook", INTTYPE, INTTYPE, 1, NULL, 0},
      {"sync_address", INTTYPE, INTTYPE, 1, NULL, 0},
      {"sync_todo", INTTYPE, INTTYPE, 1, NULL, 0},
@@ -82,7 +82,32 @@ static prefType glob_prefs[NUM_PREFS] = {
      {"todo_export_filename", CHARTYPE, CHARTYPE, 0, NULL, 0},
      {"todo_import_path", CHARTYPE, CHARTYPE, 0, NULL, 0},
      {"memo_export_filename", CHARTYPE, CHARTYPE, 0, NULL, 0},
-     {"memo_import_path", CHARTYPE, CHARTYPE, 0, NULL, 0}
+     {"memo_import_path", CHARTYPE, CHARTYPE, 0, NULL, 0},
+     {"manana_mode", INTTYPE, INTTYPE, 0, NULL, 0},
+     {"sync_manana", INTTYPE, INTTYPE, 0, NULL, 0},
+     {"use_jos", INTTYPE, INTTYPE, 0, ""},
+     {"phone_prefix1", CHARTYPE, CHARTYPE, 0, NULL, 0},
+     {"phone_prefix1_on", INTTYPE, INTTYPE, 0, NULL, 0},
+     {"phone_prefix2", CHARTYPE, CHARTYPE, 0, NULL, 0},
+     {"phone_prefix2_on", INTTYPE, INTTYPE, 0, NULL, 0},
+     {"phone_prefix3", CHARTYPE, CHARTYPE, 0, NULL, 0},
+     {"phone_prefix3_on", INTTYPE, INTTYPE, 0, NULL, 0},
+     {"dial_command", CHARTYPE, CHARTYPE, 0, NULL, 0},
+     {"datebook_todo_pane", INTTYPE, INTTYPE, 350, NULL, 0},
+     {"datebook_todo_show", INTTYPE, INTTYPE, 0, NULL, 0},
+     {"todo_hide_not_due", INTTYPE, INTTYPE, 0, NULL, 0},
+     {"todo_completion_date", INTTYPE, INTTYPE, 0, NULL, 0},
+     {"install_path", CHARTYPE, CHARTYPE, 0, NULL, 0},
+     {"memo_show_prefs", INTTYPE, INTTYPE, 1, NULL, 0},
+     {"monthview_width", INTTYPE, INTTYPE, 640, NULL, 0},
+     {"monthview_height", INTTYPE, INTTYPE, 480, NULL, 0},
+     {"weekview_width", INTTYPE, INTTYPE, 640, NULL, 0},
+     {"weekview_height", INTTYPE, INTTYPE, 480, NULL, 0},
+     {"last_date_category", INTTYPE, INTTYPE, CATEGORY_ALL, NULL, 0},
+     {"last_addr_category", INTTYPE, INTTYPE, CATEGORY_ALL, NULL, 0},
+     {"last_todo_category", INTTYPE, INTTYPE, CATEGORY_ALL, NULL, 0},
+     {"last_memo_category", INTTYPE, INTTYPE, CATEGORY_ALL, NULL, 0},
+     {"version", CHARTYPE, CHARTYPE, 0, NULL, 0},
 };
 
 struct name_list {
@@ -100,7 +125,7 @@ void pref_init()
    for (i=0; i<NUM_PREFS; i++) {
       switch (i) {
        case PREF_RCFILE:
-	 glob_prefs[i].svalue=strdup("jpilotrc.default");
+	 glob_prefs[i].svalue=strdup(EPN"rc.default");
 	 glob_prefs[i].svalue_size=strlen(glob_prefs[i].svalue)+1;
 	 break;
        case PREF_PRINT_COMMAND:
@@ -119,6 +144,10 @@ void pref_init()
 	 glob_prefs[i].svalue=strdup("09021345070413440c08135a3215135dd217ead3b5df556322e9a14a994b0f88");
 	 glob_prefs[i].svalue_size=strlen(glob_prefs[i].svalue)+1;
 	 break;
+       case PREF_DIAL_COMMAND:
+	 glob_prefs[i].svalue=strdup("jpilot-dial --lv 0 --rv 50 %n");
+	 glob_prefs[i].svalue_size=strlen(glob_prefs[i].svalue)+1;
+	 break;
        default:
 	 glob_prefs[i].svalue=strdup("");
 	 glob_prefs[i].svalue_size=1;
@@ -152,6 +181,21 @@ void jp_free_prefs(prefType prefs[], int count)
    }
 }
 
+/* Get just the formatting for just the hour and am/pm if its set */
+/* datef needs to be preallocated */
+void get_pref_hour_ampm(char *datef)
+{
+   const char *svalue;
+
+   get_pref(PREF_TIME, NULL, &svalue);
+   strncpy(datef, svalue, 2);
+   datef[2]='\0';
+   if (!strncasecmp(&(svalue[strlen(svalue)-2]), "%p", 2)) {
+      strncpy(&(datef[2]), &(svalue[strlen(svalue)-2]), 2);
+      datef[4]='\0';
+   }
+}
+
 int get_pref_time_no_secs(char *datef)
 {
    /* "%I:%M:%S %p" */
@@ -240,9 +284,9 @@ static int get_rcfile_name(int n, char *rc_copy)
 {
    DIR *dir;
    struct dirent *dirent;
-   char full_name[256];
+   char full_name[FILENAME_MAX];
    int i;
-   char filename[256];
+   char filename[FILENAME_MAX];
    int found, count;
    struct name_list *temp_list, *new_entry;
 
@@ -250,7 +294,7 @@ static int get_rcfile_name(int n, char *rc_copy)
    if (dir_list == NULL) {
       i = found = count = 0;
       sprintf(filename, "%s/%s/%s/", BASE_DIR, "share", EPN);
-      jpilot_logf(LOG_DEBUG, "opening dir %s\n", filename);
+      jp_logf(JP_LOG_DEBUG, "opening dir %s\n", filename);
       dir = opendir(filename);
       if (dir) {
 	 for(i=0; (dirent = readdir(dir)); i++) {
@@ -258,10 +302,10 @@ static int get_rcfile_name(int n, char *rc_copy)
 	    if (strncmp(filename, dirent->d_name, strlen(filename))) {
 	       continue;
 	    } else {
-	       jpilot_logf(LOG_DEBUG, "found %s\n", dirent->d_name);
+	       jp_logf(JP_LOG_DEBUG, "found %s\n", dirent->d_name);
 	       new_entry = malloc(sizeof(struct name_list));
 	       if (!new_entry) {
-		  jpilot_logf(LOG_FATAL, "get_rcfile_name(): Out of memory\n");
+		  jp_logf(JP_LOG_FATAL, "get_rcfile_name(): Out of memory\n");
 		  return -1;
 	       }  
 	       new_entry->name = strdup(dirent->d_name);
@@ -274,8 +318,8 @@ static int get_rcfile_name(int n, char *rc_copy)
 	 closedir(dir);
       }
 
-      get_home_file_name("", full_name, 255);
-      jpilot_logf(LOG_DEBUG, "opening dir %s\n", full_name);
+      get_home_file_name("", full_name, sizeof(full_name));
+      jp_logf(JP_LOG_DEBUG, "opening dir %s\n", full_name);
       dir = opendir(full_name);
       if (dir) {
 	 for(; (dirent = readdir(dir)); i++) {
@@ -283,10 +327,10 @@ static int get_rcfile_name(int n, char *rc_copy)
 	    if (strncmp(filename, dirent->d_name, strlen(filename))) {
 	       continue;
 	    } else {
-	       jpilot_logf(LOG_DEBUG, "found %s\n", dirent->d_name);
+	       jp_logf(JP_LOG_DEBUG, "found %s\n", dirent->d_name);
 	       new_entry = malloc(sizeof(struct name_list));
 	       if (!new_entry) {
-		  jpilot_logf(LOG_FATAL, "get_rcfile_name(): Out of memory 2\n");
+		  jp_logf(JP_LOG_FATAL, "get_rcfile_name(): Out of memory 2\n");
 		  return -1;
 	       }  
 	       new_entry->name = strdup(dirent->d_name);
@@ -376,13 +420,16 @@ int get_pref_possibility(int which, int n, char *pref_str)
    };
 
    static const char *char_sets[] = {
-      "English",
+      "Latin1 / No conversion",
       "Japanese",
-      "Windows1250 (EE)", 
+      "Host ISO-8859-2 <-> Palm Windows1250 (EE)", 
       "Host Windows1251 <-> Palm KOI8-R",
       "Host KOI8-R <-> Palm Windows-1251",
       "Chinese(Big5)",
-      "Korean"
+      "Korean",
+      "Host UTF-8 <-> Palm Windows1250 (EE)",
+      "Host UTF-8 <-> Palm Windows1252", /* JPA */
+      "Host UTF-8 <-> Palm Windows1253 (Greek)"
    };
 
    static const char *paper_sizes[] = {
@@ -457,7 +504,7 @@ int get_pref_possibility(int which, int n, char *pref_str)
 
     default:
       pref_str[0]='\0';
-      jpilot_logf(LOG_DEBUG, "Unknown preference type\n");
+      jp_logf(JP_LOG_DEBUG, "Unknown preference type\n");
       return -1;
    }
 
@@ -569,17 +616,13 @@ int jp_set_pref(prefType prefs[], int which, long n, const char *string)
       pref_lstrncpy_realloc(&(prefs[which].svalue), Pstr,
 			    &(prefs[which].svalue_size), MAX_PREF_VALUE);
    }
-   /* #ifdef PROMETHEON */
-   /* Some people like to just kill the window manager */
-   /* Prometheon kills us, always be prepared for death */
-   pref_write_rc_file();
-   /* #endif */
    return 0;
 }
 
-int set_pref(int which, long n, const char *string)
+int set_pref(int which, long n, const char *string, int save)
 {
    const char *str;
+   int r;
 
    if (which > NUM_PREFS) {
       return -1;
@@ -590,16 +633,25 @@ int set_pref(int which, long n, const char *string)
        (which==PREF_LONGDATE) ||
        (which==PREF_TIME) ||
        (which==PREF_PAPER_SIZE)) {
-      set_pref_possibility(which, n);
+      set_pref_possibility(which, n, FALSE);
       str=glob_prefs[which].svalue;
    }
-   return jp_set_pref(glob_prefs, which, n, str);
+   r = jp_set_pref(glob_prefs, which, n, str);
+   /* #ifdef ENABLE_PROMETHEON */
+   /* Some people like to just kill the window manager */
+   /* Prometheon kills us, always be prepared for death */
+   if (save) {
+      pref_write_rc_file();
+   }
+   /* #endif */
+   return r;
 }
 
-int set_pref_possibility(int which, long n)
+int set_pref_possibility(int which, long n, int save)
 {
    char svalue[MAX_PREF_VALUE];
    char *str=NULL;
+   int r;
 
    if (which > NUM_PREFS) {
       return -1;
@@ -608,7 +660,15 @@ int set_pref_possibility(int which, long n)
       get_pref_possibility(which, n, svalue);
       str=svalue;
    }
-   return jp_set_pref(glob_prefs, which, n, str);
+   r = jp_set_pref(glob_prefs, which, n, str);
+   /* #ifdef ENABLE_PROMETHEON */
+   /* Some people like to just kill the window manager */
+   /* Prometheon kills us, always be prepared for death */
+   if (save) {
+      pref_write_rc_file();
+   }
+   /* #endif */
+   return r;
 }
 
 static int validate_glob_prefs()
@@ -771,7 +831,7 @@ int jp_pref_read_rc_file(char *filename, prefType prefs[], int num_prefs)
 	       if (pref_lstrncpy_realloc(&(prefs[i].svalue), field2,
 					&(prefs[i].svalue_size),
 					MAX_PREF_VALUE)==NULL) {
-		  jpilot_logf(LOG_WARN, "Out of memory: read_rc_file()\n");
+		  jp_logf(JP_LOG_WARN, "Out of memory: read_rc_file()\n");
 		  continue;
 	       }
 	    }
@@ -787,7 +847,7 @@ int pref_read_rc_file()
 {
    int r;
 
-   r=jp_pref_read_rc_file("jpilot.rc", glob_prefs, NUM_PREFS);
+   r=jp_pref_read_rc_file(EPN".rc", glob_prefs, NUM_PREFS);
 
    validate_glob_prefs();
 
@@ -799,7 +859,7 @@ int jp_pref_write_rc_file(char *filename, prefType prefs[], int num_prefs)
    int i;
    FILE *out;
 
-   jpilot_logf(LOG_DEBUG, "jp_pref_write_rc_file()\n");
+   jp_logf(JP_LOG_DEBUG, "jp_pref_write_rc_file()\n");
 
    out=jp_open_home_file(filename,"w" );
    if (!out) {
@@ -823,5 +883,5 @@ int jp_pref_write_rc_file(char *filename, prefType prefs[], int num_prefs)
 
 int pref_write_rc_file()
 {
-   return jp_pref_write_rc_file("jpilot.rc", glob_prefs, NUM_PREFS);
+   return jp_pref_write_rc_file(EPN".rc", glob_prefs, NUM_PREFS);
 }
