@@ -1,7 +1,7 @@
 /* prefs.h
  * A module of J-Pilot http://jpilot.org
  *
- * Copyright (C) 1999-2001 by Judd Montgomery
+ * Copyright (C) 1999-2002 by Judd Montgomery
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -72,8 +72,33 @@
 #define PREF_TODO_IMPORT_PATH 48
 #define PREF_MEMO_EXPORT_FILENAME 49
 #define PREF_MEMO_IMPORT_PATH 50
+#define PREF_MANANA_MODE 51
+#define PREF_SYNC_MANANA 52
+#define PREF_USE_JOS 53
+#define PREF_PHONE_PREFIX1 54
+#define PREF_CHECK_PREFIX1 55
+#define PREF_PHONE_PREFIX2 56
+#define PREF_CHECK_PREFIX2 57
+#define PREF_PHONE_PREFIX3 58
+#define PREF_CHECK_PREFIX3 59
+#define PREF_DIAL_COMMAND 60
+#define PREF_DATEBOOK_TODO_PANE 61
+#define PREF_DATEBOOK_TODO_SHOW 62
+#define PREF_HIDE_NOT_DUE 63
+#define PREF_TODO_COMPLETION_DATE 64
+#define PREF_INSTALL_PATH 65
+#define PREF_SHOW_MEMO_PREFS 66
+#define PREF_MONTHVIEW_WIDTH 67
+#define PREF_MONTHVIEW_HEIGHT 68
+#define PREF_WEEKVIEW_WIDTH 69
+#define PREF_WEEKVIEW_HEIGHT 70
+#define PREF_LAST_DATE_CATEGORY 71
+#define PREF_LAST_ADDR_CATEGORY 72
+#define PREF_LAST_TODO_CATEGORY 73
+#define PREF_LAST_MEMO_CATEGORY 74
+#define PREF_VERSION 75
 
-#define NUM_PREFS 51
+#define NUM_PREFS 76
 
 #define MAX_PREF_NUM_BACKUPS 99
 
@@ -81,14 +106,17 @@
 #define PREF_DMY 1
 #define PREF_YMD 2
 
-#define CHAR_SET_ENGLISH  0
+#define CHAR_SET_LATIN1   0 /* English, European, Latin based languages */
 #define CHAR_SET_JAPANESE 1
-#define CHAR_SET_1250     2 /* Czech */
+#define CHAR_SET_1250     2 /* Czech, Polish (Unix: ISO-8859-2) */
 #define CHAR_SET_1251     3 /* Russian; palm koi8-r, host win1251 */
 #define CHAR_SET_1251_B   4 /* Russian; palm win1251, host koi8-r */
 #define CHAR_SET_TRADITIONAL_CHINESE  5 /* Taiwan Chinese */
 #define CHAR_SET_KOREAN   6 /* Korean Hangul */
-#define NUM_CHAR_SETS     7
+#define CHAR_SET_1250UTF  7 /* Czech, Polish (Unix: UTF-8) */
+#define CHAR_SET_LATINUTF 8 /* JPA : Latin European (Unix: UTF-8) */
+#define CHAR_SET_1253UTF  9 /* Modern Greek (Palm: Windows 1253, Host: UTF-8) */
+#define NUM_CHAR_SETS    10
 
 #define MAX_PREF_VALUE 200
 
@@ -96,12 +124,13 @@ void pref_init();
 int pref_read_rc_file();
 int pref_write_rc_file();
 int get_pref(int which, long *n, const char **ret);
-int set_pref(int which, long n, const char *string);
+int set_pref(int which, long n, const char *string, int save);
 
 /* Specialized functions */
-int set_pref_possibility(int which, long n);
+int set_pref_possibility(int which, long n, int save);
 int get_pref_possibility(int which, int n, char *ret);
 int get_pref_dmy_order();
+void get_pref_hour_ampm(char *datef);
 int get_pref_time_no_secs(char *datef);
 int get_pref_time_no_secs_no_ampm(char *datef);
 
