@@ -1,7 +1,7 @@
 /* prefs_gui.c
  * A module of J-Pilot http://jpilot.org
  *
- * Copyright (C) 1999-2001 by Judd Montgomery
+ * Copyright (C) 1999-2002 by Judd Montgomery
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -39,13 +39,31 @@ extern int glob_app;
 #ifdef COLORS
 
 /* This doesn't work quite right.  There is supposedly no way to do it in GTK. */
+void r(GtkWidget *w, gpointer data)
+{
+   GtkStyle *style;
+   
+   style = gtk_rc_get_style(GTK_WIDGET(w));
+   if (style) gtk_rc_style_unref(style);
+   if (GTK_IS_CONTAINER(w)) {
+      gtk_container_foreach(GTK_CONTAINER(w), r, w);
+   }
+}
 
 void set_colors()
 {
+   GtkStyle* style;
+   int i;
+   
+   r(main_window, NULL);
+   r(window, NULL);
+   
+   gtk_rc_reparse_all();
    read_gtkrc_file();
    gtk_rc_reparse_all();
    gtk_widget_reset_rc_styles(window);
    gtk_widget_reset_rc_styles(main_window);
+   gtk_rc_reparse_all();
    gtk_widget_queue_draw(window);
    gtk_widget_queue_draw(main_window);
 }
@@ -66,8 +84,8 @@ static void cb_pref_menu(GtkWidget *widget,
    pref = GPOINTER_TO_INT(data);
    value = pref & 0xFF;
    pref = pref >> 8;
-   set_pref(pref, value, NULL);
-   jpilot_logf(LOG_DEBUG, "pref %d, value %d\n", pref, value);
+   set_pref_possibility(pref, value, TRUE);
+   jp_logf(JP_LOG_DEBUG, "pref %d, value %d\n", pref, value);
 #ifdef COLORS
    if (pref==PREF_RCFILE) {
       set_colors();
@@ -124,7 +142,7 @@ int make_pref_menu(GtkWidget **pref_menu, int pref_num)
 	 gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menu_item), ivalue);
       }
 
-      gtk_signal_connect(GTK_OBJECT(menu_item), "activate", cb_pref_menu,
+      gtk_signal_connect(GTK_OBJECT(menu_item), "activate", GTK_SIGNAL_FUNC(cb_pref_menu),
 			 GINT_TO_POINTER(((pref_num*0x100) + (i & 0xFF))));
 
       gtk_widget_show(menu_item);
@@ -134,50 +152,52 @@ int make_pref_menu(GtkWidget **pref_menu, int pref_num)
    return 0;
 }
 
-void cb_show_deleted(GtkWidget *widget,
-		     gpointer data)
+void cb_show_deleted(GtkWidget *widget, gpointer data)
 {
-   set_pref(PREF_SHOW_DELETED, GTK_TOGGLE_BUTTON(widget)->active, NULL);
+   set_pref(PREF_SHOW_DELETED, GTK_TOGGLE_BUTTON(widget)->active, NULL, TRUE);
 }
-void cb_show_modified(GtkWidget *widget,
-		      gpointer data)
+void cb_show_modified(GtkWidget *widget, gpointer data)
 {
-   set_pref(PREF_SHOW_MODIFIED, GTK_TOGGLE_BUTTON(widget)->active, NULL);
+   set_pref(PREF_SHOW_MODIFIED, GTK_TOGGLE_BUTTON(widget)->active, NULL, TRUE);
 }
-void cb_highlight(GtkWidget *widget,
-		  gpointer data)
+void cb_highlight(GtkWidget *widget, gpointer data)
 {
-   set_pref(PREF_HIGHLIGHT, GTK_TOGGLE_BUTTON(widget)->active, NULL);
+   set_pref(PREF_HIGHLIGHT, GTK_TOGGLE_BUTTON(widget)->active, NULL, TRUE);
 }
-void cb_use_db3(GtkWidget *widget,
+void cb_use_db3(GtkWidget *widget, gpointer data)
+{
+   set_pref(PREF_USE_DB3, GTK_TOGGLE_BUTTON(widget)->active, NULL, TRUE);
+}
+void cb_sync_datebook(GtkWidget *widget, gpointer data)
+{
+   set_pref(PREF_SYNC_DATEBOOK, GTK_TOGGLE_BUTTON(widget)->active, NULL, TRUE);
+}
+void cb_sync_address(GtkWidget *widget, gpointer data)
+{
+   set_pref(PREF_SYNC_ADDRESS, GTK_TOGGLE_BUTTON(widget)->active, NULL, TRUE);
+}
+void cb_sync_todo(GtkWidget *widget, gpointer data)
+{
+   set_pref(PREF_SYNC_TODO, GTK_TOGGLE_BUTTON(widget)->active, NULL, TRUE);
+}
+void cb_sync_memo(GtkWidget *widget, gpointer data)
+{
+   set_pref(PREF_SYNC_MEMO, GTK_TOGGLE_BUTTON(widget)->active, NULL, TRUE);
+}
+void cb_sync_memo32(GtkWidget *widget, gpointer data)
+{
+   set_pref(PREF_SYNC_MEMO32, GTK_TOGGLE_BUTTON(widget)->active, NULL, TRUE);
+}
+#ifdef ENABLE_MANANA
+void cb_sync_manana(GtkWidget *widget, gpointer data)
+{
+   set_pref(PREF_SYNC_MANANA, GTK_TOGGLE_BUTTON(widget)->active, NULL, TRUE);
+}
+#endif
+void cb_use_jos(GtkWidget *widget,
 		gpointer data)
 {
-   set_pref(PREF_USE_DB3, GTK_TOGGLE_BUTTON(widget)->active, NULL);
-}
-void cb_sync_datebook(GtkWidget *widget,
-		      gpointer data)
-{
-   set_pref(PREF_SYNC_DATEBOOK, GTK_TOGGLE_BUTTON(widget)->active, NULL);
-}
-void cb_sync_address(GtkWidget *widget,
-		     gpointer data)
-{
-   set_pref(PREF_SYNC_ADDRESS, GTK_TOGGLE_BUTTON(widget)->active, NULL);
-}
-void cb_sync_todo(GtkWidget *widget,
-		  gpointer data)
-{
-   set_pref(PREF_SYNC_TODO, GTK_TOGGLE_BUTTON(widget)->active, NULL);
-}
-void cb_sync_memo(GtkWidget *widget,
-		  gpointer data)
-{
-   set_pref(PREF_SYNC_MEMO, GTK_TOGGLE_BUTTON(widget)->active, NULL);
-}
-void cb_sync_memo32(GtkWidget *widget,
-		    gpointer data)
-{
-   set_pref(PREF_SYNC_MEMO32, GTK_TOGGLE_BUTTON(widget)->active, NULL);
+   set_pref(PREF_USE_JOS, GTK_TOGGLE_BUTTON(widget)->active, NULL, TRUE);
 }
 #ifdef ENABLE_PLUGINS
 void cb_sync_plugin(GtkWidget *widget,
@@ -211,34 +231,34 @@ void cb_sync_plugin(GtkWidget *widget,
 void cb_open_alarm(GtkWidget *widget,
 		   gpointer data)
 {
-   set_pref(PREF_OPEN_ALARM_WINDOWS, GTK_TOGGLE_BUTTON(widget)->active, NULL);
+   set_pref(PREF_OPEN_ALARM_WINDOWS, GTK_TOGGLE_BUTTON(widget)->active, NULL, TRUE);
 }
 void cb_do_command(GtkWidget *widget,
 		   gpointer data)
 {
-   set_pref(PREF_DO_ALARM_COMMAND, GTK_TOGGLE_BUTTON(widget)->active, NULL);
+   set_pref(PREF_DO_ALARM_COMMAND, GTK_TOGGLE_BUTTON(widget)->active, NULL, TRUE);
 }
 
 
 
 static gboolean cb_destroy(GtkWidget *widget)
 {
-   char *entry_text;
-   char *backups_text;
+   const char *entry_text;
+   const char *backups_text;
    int num_backups;
 
-   jpilot_logf(LOG_DEBUG, "Cleanup\n");
+   jp_logf(JP_LOG_DEBUG, "Cleanup\n");
 
    entry_text = gtk_entry_get_text(GTK_ENTRY(port_entry));
-   jpilot_logf(LOG_DEBUG, "port_entry = [%s]\n", entry_text);
-   set_pref(PREF_PORT, 0, entry_text);
+   jp_logf(JP_LOG_DEBUG, "port_entry = [%s]\n", entry_text);
+   set_pref(PREF_PORT, 0, entry_text, FALSE);
 
    entry_text = gtk_entry_get_text(GTK_ENTRY(alarm_command_entry));
-   jpilot_logf(LOG_DEBUG, "alarm_command_entry = [%s]\n", entry_text);
-   set_pref(PREF_ALARM_COMMAND, 0, entry_text);
+   jp_logf(JP_LOG_DEBUG, "alarm_command_entry = [%s]\n", entry_text);
+   set_pref(PREF_ALARM_COMMAND, 0, entry_text, FALSE);
 
    backups_text = gtk_entry_get_text(GTK_ENTRY(backups_entry));
-   jpilot_logf(LOG_DEBUG, "backups_entry = [%s]\n", backups_text);
+   jp_logf(JP_LOG_DEBUG, "backups_entry = [%s]\n", backups_text);
    num_backups = atoi(backups_text);
    if (num_backups < 1) {
       num_backups = 1;
@@ -246,7 +266,7 @@ static gboolean cb_destroy(GtkWidget *widget)
    if (num_backups > 99) {
       num_backups = 99;
    }
-   set_pref(PREF_NUM_BACKUPS, num_backups, NULL);
+   set_pref(PREF_NUM_BACKUPS, num_backups, NULL, TRUE);
 
    pref_write_rc_file();
 
@@ -261,7 +281,7 @@ static void
   cb_quit(GtkWidget *widget,
 	   gpointer   data)
 {
-   jpilot_logf(LOG_DEBUG, "cb_quit\n");
+   jp_logf(JP_LOG_DEBUG, "cb_quit\n");
    if (GTK_IS_WIDGET(data)) {
       gtk_widget_destroy(data);
    }
@@ -291,9 +311,14 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    extern unsigned char skip_plugins;
 #endif
 
-   jpilot_logf(LOG_DEBUG, "cb_prefs_gui\n");
+   jp_logf(JP_LOG_DEBUG, "cb_prefs_gui\n");
    if (window) {
-      jpilot_logf(LOG_DEBUG, "pref_window is already up\n");
+      jp_logf(JP_LOG_DEBUG, "pref_window is already up\n");
+#ifdef ENABLE_GTK2
+      /* Shift focus to existing window if called again 
+         and window is still alive. */
+      gtk_window_present(GTK_WINDOW(window));
+#endif
       return;
    }
 
@@ -303,8 +328,7 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    /*gtk_window_set_default_size(GTK_WINDOW(window), 500, 300); */
 
    gtk_container_set_border_width(GTK_CONTAINER(window), 10);
-   g_snprintf(temp, 255, "%s %s", PN, _("Preferences"));
-   temp[255]='\0';
+   g_snprintf(temp, sizeof(temp), "%s %s", PN, _("Preferences"));
    gtk_window_set_title(GTK_WINDOW(window), temp);
 
    gtk_signal_connect(GTK_OBJECT(window), "destroy",
@@ -446,7 +470,7 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
 
 
    /* Rate */
-   label = gtk_label_new(_("Serial Rate "));
+   label = gtk_label_new(_("Serial Rate (Does not affect USB)"));
    gtk_table_attach_defaults(GTK_TABLE(table), GTK_WIDGET(label),
 			     0, 1, 2, 3);
    gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
@@ -473,7 +497,7 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    gtk_entry_set_text(GTK_ENTRY(backups_entry), temp_str);
 
 
-   /*Show deleted files check box */
+   /* Show deleted files check box */
    checkbutton = gtk_check_button_new_with_label
      (_("Show deleted records (default NO)"));
    gtk_box_pack_start(GTK_BOX(vbox_settings), checkbutton, FALSE, FALSE, 0);
@@ -486,7 +510,7 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
 		      "clicked", GTK_SIGNAL_FUNC(cb_show_deleted),
 		      GINT_TO_POINTER(PREF_SHOW_DELETED));
 
-   /*Show modified files check box */
+   /* Show modified files check box */
    checkbutton = gtk_check_button_new_with_label
      (_("Show modified deleted records (default NO)"));
    gtk_box_pack_start(GTK_BOX(vbox_settings), checkbutton, FALSE, FALSE, 0);
@@ -500,7 +524,7 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
 		      GINT_TO_POINTER(PREF_SHOW_MODIFIED));
 
 
-   /*Show highlight days check box */
+   /* Show highlight days check box */
    checkbutton = gtk_check_button_new_with_label
      (_("Highlight calendar days with appointments"));
    gtk_box_pack_start(GTK_BOX(vbox_settings), checkbutton, FALSE, FALSE, 0);
@@ -513,9 +537,9 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
 		      "clicked", GTK_SIGNAL_FUNC(cb_highlight), NULL);
 
 
-   /*Show use DateBk3/4 check box */
-   checkbutton = gtk_check_button_new_with_label
-     (_("Use DateBk3/4 note tags"));
+#ifdef ENABLE_DATEBK
+   /* Show use DateBk check box */
+   checkbutton = gtk_check_button_new_with_label(_("Use DateBk note tags"));
    gtk_box_pack_start(GTK_BOX(vbox_settings), checkbutton, FALSE, FALSE, 0);
    get_pref(PREF_USE_DB3, &ivalue, &cstr);
    gtk_widget_show(checkbutton);
@@ -524,12 +548,15 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    }
    gtk_signal_connect(GTK_OBJECT(checkbutton), 
 		      "clicked", GTK_SIGNAL_FUNC(cb_use_db3), NULL);
+#else
+   label = gtk_label_new(_("DateBk support disabled in this build"));
+   gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
+   gtk_box_pack_start(GTK_BOX(vbox_settings), label, FALSE, FALSE, 0);
+#endif
 
 
-
-   /*Show sync datebook check box */
-   checkbutton = gtk_check_button_new_with_label
-     (_("Sync datebook"));
+   /* Show sync datebook check box */
+   checkbutton = gtk_check_button_new_with_label(_("Sync datebook"));
    gtk_box_pack_start(GTK_BOX(vbox_conduits), checkbutton, FALSE, FALSE, 0);
    get_pref(PREF_SYNC_DATEBOOK, &ivalue, &cstr);
    gtk_widget_show(checkbutton);
@@ -539,9 +566,8 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    gtk_signal_connect(GTK_OBJECT(checkbutton), 
 		      "clicked", GTK_SIGNAL_FUNC(cb_sync_datebook), NULL);
 
-   /*Show sync address check box */
-   checkbutton = gtk_check_button_new_with_label
-     (_("Sync address"));
+   /* Show sync address check box */
+   checkbutton = gtk_check_button_new_with_label(_("Sync address"));
    gtk_box_pack_start(GTK_BOX(vbox_conduits), checkbutton, FALSE, FALSE, 0);
    get_pref(PREF_SYNC_ADDRESS, &ivalue, &cstr);
    gtk_widget_show(checkbutton);
@@ -551,9 +577,8 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    gtk_signal_connect(GTK_OBJECT(checkbutton), 
 		      "clicked", GTK_SIGNAL_FUNC(cb_sync_address), NULL);
 
-   /*Show sync todo check box */
-   checkbutton = gtk_check_button_new_with_label
-     (_("Sync todo"));
+   /* Show sync todo check box */
+   checkbutton = gtk_check_button_new_with_label(_("Sync todo"));
    gtk_box_pack_start(GTK_BOX(vbox_conduits), checkbutton, FALSE, FALSE, 0);
    get_pref(PREF_SYNC_TODO, &ivalue, &cstr);
    gtk_widget_show(checkbutton);
@@ -563,9 +588,8 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    gtk_signal_connect(GTK_OBJECT(checkbutton), 
 		      "clicked", GTK_SIGNAL_FUNC(cb_sync_todo), NULL);
 
-   /*Show sync memo check box */
-   checkbutton = gtk_check_button_new_with_label
-     (_("Sync memo"));
+   /* Show sync memo check box */
+   checkbutton = gtk_check_button_new_with_label(_("Sync memo"));
    gtk_box_pack_start(GTK_BOX(vbox_conduits), checkbutton, FALSE, FALSE, 0);
    get_pref(PREF_SYNC_MEMO, &ivalue, &cstr);
    gtk_widget_show(checkbutton);
@@ -575,7 +599,7 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    gtk_signal_connect(GTK_OBJECT(checkbutton), 
 		      "clicked", GTK_SIGNAL_FUNC(cb_sync_memo), NULL);
 
-   /*Show sync Memo32 check box */
+   /* Show sync Memo32 check box */
    checkbutton = gtk_check_button_new_with_label
      (_("Sync memo32 (pedit32)"));
    gtk_box_pack_start(GTK_BOX(vbox_conduits), checkbutton, FALSE, FALSE, 0);
@@ -587,6 +611,32 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    gtk_signal_connect(GTK_OBJECT(checkbutton), 
 		      "clicked", GTK_SIGNAL_FUNC(cb_sync_memo32), NULL);
 
+#ifdef ENABLE_MANANA
+   /* Show sync Ma~nana check box */
+   checkbutton = gtk_check_button_new_with_label(_("Sync Manana"));
+   gtk_box_pack_start(GTK_BOX(vbox_conduits), checkbutton, FALSE, FALSE, 0);
+   get_pref(PREF_SYNC_MANANA, &ivalue, &cstr);
+   gtk_widget_show(checkbutton);
+   if (ivalue) {
+      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton), TRUE);
+   }
+   gtk_signal_connect(GTK_OBJECT(checkbutton), 
+		      "clicked", GTK_SIGNAL_FUNC(cb_sync_manana), NULL);
+#endif
+   get_pref(PREF_CHAR_SET, &ivalue, &cstr);
+   if (ivalue == CHAR_SET_JAPANESE) {
+      /*Show use Japanese Kana extention check box */
+      checkbutton = gtk_check_button_new_with_label(_("Use J-OS (Not Japanese PalmOS:WorkPad/CLIE)"));
+      gtk_box_pack_start(GTK_BOX(vbox_settings), checkbutton, FALSE, FALSE, 0);
+      get_pref(PREF_USE_JOS, &ivalue, &cstr);
+      gtk_widget_show(checkbutton);
+      if (ivalue) {
+	 gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton), TRUE);
+      }
+      gtk_signal_connect(GTK_OBJECT(checkbutton), 
+			 "clicked", GTK_SIGNAL_FUNC(cb_use_jos), NULL);	   
+   }
+   
 #ifdef  ENABLE_PLUGINS
    if (!skip_plugins) {
 
@@ -598,8 +648,7 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
 	 Pplugin = (struct plugin_s *)temp_list->data;
 	 if (Pplugin) {
 	    /* Make a checkbox for each plugin */
-	    g_snprintf(temp, 250, "Sync %s (%s)", Pplugin->name, Pplugin->full_path);
-	    temp[250]='\0';
+	    g_snprintf(temp, sizeof(temp), "Sync %s (%s)", Pplugin->name, Pplugin->full_path);
 	    checkbutton = gtk_check_button_new_with_label(temp);
 	    gtk_box_pack_start(GTK_BOX(vbox_conduits), checkbutton, FALSE, FALSE, 0);
 	    gtk_widget_show(checkbutton);
@@ -668,7 +717,7 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.0);
    gtk_box_pack_start(GTK_BOX(vbox_alarms), label, FALSE, FALSE, 0);
 
-#ifdef ALARM_SHELL_DESC_NOTE
+#ifdef ENABLE_ALARM_SHELL_DANGER
    label = gtk_label_new(_("%D is replaced with the alarm description"));
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.0);
    gtk_box_pack_start(GTK_BOX(vbox_alarms), label, FALSE, FALSE, 0);
@@ -676,6 +725,14 @@ void cb_prefs_gui(GtkWidget *widget, gpointer data)
    label = gtk_label_new(_("%N is replaced with the alarm note"));
    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.0);
    gtk_box_pack_start(GTK_BOX(vbox_alarms), label, FALSE, FALSE, 0);
+#else
+   label = gtk_label_new(_("%D (description substitution) is disabled in this build"));
+   gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
+   gtk_box_pack_start(GTK_BOX(vbox_alarms), label, FALSE, FALSE, 0);
+
+   label = gtk_label_new(_("%N (note substitution) is disabled in this build"));
+   gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
+   gtk_box_pack_start(GTK_BOX(vbox_alarms), label, FALSE, FALSE, 0);
 #endif
 
    /* Create a "Done" button */
