@@ -1,7 +1,7 @@
 /* print.c
  * A module of J-Pilot http://jpilot.org
  *
- * Copyright (C) 2000-2001 by Judd Montgomery
+ * Copyright (C) 2000-2002 by Judd Montgomery
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -106,7 +106,7 @@ int puttext(float x, float y, char *text)
 
    len = strlen(text);
    buf = (char *) malloc(2 * len + 1);
-   bzero(buf, 2 * len + 1);
+   memset(buf, 0, 2 * len + 1);
    ps_strncat(buf, text, 2 * len);
    fprintf(out, "%g inch %g inch moveto (%s) show\n", x, y, buf);
    free(buf);
@@ -172,7 +172,7 @@ int print_dayview(struct tm *date, AppointmentList *a_list)
    long ivalue;
 
 #ifdef HAVE_LOCALE_H
-   setlocale(LC_ALL,"C");
+   setlocale(LC_NUMERIC,"C");
 #endif
 
    header();
@@ -183,13 +183,13 @@ int print_dayview(struct tm *date, AppointmentList *a_list)
    fprintf(out, "/Times-Bold-ISOLatin1 findfont 20 scalefont setfont\n"
            "newpath 0 setgray\n");
    get_pref(PREF_LONGDATE, &ivalue, &svalue);
-   /* strftime(str, 80, "%B %d, %Y", date); */
-   strftime(str, 80, svalue, date);
+   /* strftime(str, sizeof(str), "%B %d, %Y", date); */
+   strftime(str, sizeof(str), svalue, date);
    puttext(0.5, 10.25, str);
 
    /* Put the weekday name up */
    fprintf(out, "/Times-Roman-ISOLatin1 findfont 15 scalefont setfont\n");
-   strftime(str, 80, "%A", date);
+   strftime(str, sizeof(str), "%A", date);
    puttext(0.5, 10, str);
 
    /* Put the time of printing up */
@@ -199,8 +199,8 @@ int print_dayview(struct tm *date, AppointmentList *a_list)
    time(&ltime);
    now = localtime(&ltime);
    get_pref(PREF_SHORTDATE, &ivalue, &svalue);
-   g_snprintf(datef, 80, "%s %s", "Printed on: ", svalue);
-   strftime(str, 80, datef, now);
+   g_snprintf(datef, sizeof(date), "%s %s", "Printed on: ", svalue);
+   strftime(str, sizeof(str), datef, now);
    puttext(0.5, 0.9, str);
    puttext(7.5, 0.9, "J-Pilot");
    fprintf(out, "stroke\n");
@@ -216,7 +216,7 @@ int print_dayview(struct tm *date, AppointmentList *a_list)
 	   "%%%%EOF\n");
 
 #ifdef HAVE_LOCALE_H
-   setlocale(LC_ALL,"C");
+   setlocale(LC_ALL,"");
 #endif
 
    return 0;
@@ -276,7 +276,7 @@ int fill_in(struct tm *date, AppointmentList *a_list)
 	    }
 
 	    get_pref_time_no_secs(datef);
-	    strftime(str, 32, datef, &temp_al->ma.a.begin);
+	    strftime(str, sizeof(str), datef, &temp_al->ma.a.begin);
 
 	    if (hour24 > 11) {
 	       x=indent2;
@@ -297,14 +297,15 @@ int fill_in(struct tm *date, AppointmentList *a_list)
 	    }
 	 }
 	 if (temp_al->ma.a.description) {
-	    strncat(str, temp_al->ma.a.description, 150);
+	    strcat(str, " ");
+	    strncat(str, temp_al->ma.a.description, sizeof(str)-strlen(str)-2);
 	    str[128]='\0';
 	 }
 	 if (y > 1.0) {
 	    puttext(x, y, str);
 	    /* printf("[%s]\n", str);*/
 	 } else {
-	    jpilot_logf(LOG_WARN, "Too many appointments, dropping one\n");
+	    jp_logf(JP_LOG_WARN, "Too many appointments, dropping one\n");
 	 }
       }
       fprintf(out, "grestore\n");
@@ -324,7 +325,7 @@ int print_days_appts(struct tm *date)
 
    a_list = NULL;
 
-   get_days_appointments2(&a_list, date, 2, 2, 2);
+   get_days_appointments2(&a_list, date, 2, 2, 2, NULL);
 
    print_dayview(date, a_list);
 
@@ -377,7 +378,7 @@ void ps_strncat(char *dest, char *src, int n)
 	 break;
       }
       if (strchr("()", src[i]) != NULL) {
-	 dest2[j] = '\\';
+	 if(j<n-1) dest2[j] = '\\'; else dest2[j]=' ';
 	 j++;
       }
       dest2[j] = src[i];
@@ -425,13 +426,14 @@ int print_months_appts(struct tm *date_in, PaperSize paper_size)
    const char *svalue;
    long fdow;
    int mask;
-#ifdef USE_DB3
+#ifdef ENABLE_DATEBK
    int ret;
-   int category;
    int cat_bit;
+   int db3_type;
    long use_db3_tags;
-   get_pref(PREF_USE_DB3, &use_db3_tags, NULL);
+   struct db4_struct db4;
 #endif
+
    /*------------------------------------------------------------------
     * Set up the PostScript output file, and print the header to it.
     *------------------------------------------------------------------*/
@@ -439,6 +441,13 @@ int print_months_appts(struct tm *date_in, PaperSize paper_size)
 
    time(&ltime);
 
+#ifdef ENABLE_DATEBK
+   get_pref(PREF_USE_DB3, &use_db3_tags, NULL);
+#endif
+
+#ifdef HAVE_LOCALE_H
+   setlocale(LC_NUMERIC, "C");
+#endif
    if (! (out = print_open())) return(-1);
    fprintf(out, "%%!PS-Adobe-2.0\n\n"
 	   "/PageSize (%s) def\n\n", PaperSizes[paper_size]);
@@ -451,9 +460,9 @@ int print_months_appts(struct tm *date_in, PaperSize paper_size)
    memcpy(&date, date_in, sizeof(struct tm));
    /* Get all of the appointments */
 
-   get_days_appointments2(&a_list, NULL, 2, 2, 2);
+   get_days_appointments2(&a_list, NULL, 2, 2, 2, NULL);
    get_month_info(date.tm_mon, 1, date.tm_year, &dow, &ndim);
-   weed_datebook_list(&a_list, date.tm_mon, date.tm_year, &mask);
+   weed_datebook_list(&a_list, date.tm_mon, date.tm_year, 0, &mask);
 
    /*------------------------------------------------------------------
     * Loop through the days in the month, printing appointments
@@ -488,14 +497,13 @@ int print_months_appts(struct tm *date_in, PaperSize paper_size)
       fprintf(out, "NextDay\n");
 
       for (temp_al = a_list; temp_al; temp_al=temp_al->next) {
-#ifdef USE_DB3
+#ifdef ENABLE_DATEBK
 	 if (use_db3_tags) {
-	    ret = db3_is_float(&(temp_al->ma.a), &category);
-	    /* jpilot_logf(LOG_DEBUG, "category = 0x%x\n", category); */
-	    cat_bit=1<<category;
+	    ret = db3_parse_tag(temp_al->ma.a.note, &db3_type, &db4);
+	    /* jp_logf(JP_LOG_DEBUG, "category = 0x%x\n", db4.category); */
+	    cat_bit=1<<db4.category;
 	    if (!(cat_bit & datebook_category)) {
-	       jpilot_logf(LOG_DEBUG, "skipping rec not in this "
-			   "category\n");
+	       jp_logf(JP_LOG_DEBUG, "skipping rec not in this category\n");
 	       continue;
 	    }
 	 }
@@ -507,9 +515,8 @@ int print_months_appts(struct tm *date_in, PaperSize paper_size)
 	    tmp[0]='\0';
 	    if ( ! temp_al->ma.a.event) {
 	       get_pref_time_no_secs(datef1);
-	       g_snprintf(datef2, 19, "(%s )", datef1);
-	       datef2[19]='\0';
-	       strftime(tmp, 19, datef2, &(temp_al->ma.a.begin));
+	       g_snprintf(datef2, sizeof(datef2), "(%s )", datef1);
+	       strftime(tmp, sizeof(tmp), datef2, &(temp_al->ma.a.begin));
 	       tmp[19]='\0';
 	    }
 	    desc[0]='\0';
@@ -529,7 +536,7 @@ int print_months_appts(struct tm *date_in, PaperSize paper_size)
    sub_months_from_date(&date, 1);
    date.tm_mday = 1;	/* Go to the first of the month */
    mktime(&date);
-   strftime(desc, 50, "(%B %Y) %w ", &date);
+   strftime(desc, sizeof(desc), "(%B %Y) %w ", &date);
    fprintf(out, "\n\n%%----------------------------------------\n"
            "%% Now generate the small months\n\n"
            "%s %d ", desc, days_in_mon(&date));
@@ -538,7 +545,7 @@ int print_months_appts(struct tm *date_in, PaperSize paper_size)
    add_months_to_date(&date, 1);
    date.tm_mday = 1;	/* Go to the first of the month */
    mktime(&date);
-   strftime(desc, 50, "(%B %Y) %w ", &date);
+   strftime(desc, sizeof(desc), "(%B %Y) %w ", &date);
    fprintf(out, "%s %d SmallMonths\n", desc, days_in_mon(&date));
 
    /*------------------------------------------------------------------*/
@@ -551,7 +558,10 @@ int print_months_appts(struct tm *date_in, PaperSize paper_size)
 
    print_close(out);
 
-   return(0);
+#ifdef HAVE_LOCALE_H
+   setlocale(LC_ALL, "");
+#endif
+     return(0);
 }
 
 /*----------------------------------------------------------------------
@@ -605,11 +615,12 @@ int print_weeks_appts(struct tm *date_in, PaperSize paper_size)
    char desc[256], short_date[32];
    int n;
    time_t ltime;
-#ifdef USE_DB3
-   int ret, category, cat_bit;
+#ifdef ENABLE_DATEBK
+   int ret;
+   int cat_bit;
+   int db3_type;
    long use_db3_tags;
-
-   get_pref(PREF_USE_DB3, &use_db3_tags, NULL);
+   struct db4_struct db4;
 #endif
 
    /*------------------------------------------------------------------
@@ -618,6 +629,13 @@ int print_weeks_appts(struct tm *date_in, PaperSize paper_size)
    time(&ltime);
    today_date = localtime(&ltime);
 
+#ifdef ENABLE_DATEBK
+   get_pref(PREF_USE_DB3, &use_db3_tags, NULL);
+#endif
+#ifdef HAVE_LOCALE_H
+   setlocale(LC_NUMERIC, "C");
+#endif
+     
    if (! (out = print_open())) return(-1);
    /*------------------------------------------------------------------
     * These are preferences for page size (passed in), first and last
@@ -636,16 +654,16 @@ int print_weeks_appts(struct tm *date_in, PaperSize paper_size)
     * Run through the appointments, looking for earliest and latest
     *------------------------------------------------------------------*/
    a_list = NULL;
-   get_days_appointments2(&a_list, NULL, 2, 2, 2);
+   get_days_appointments2(&a_list, NULL, 2, 2, 2, NULL);
    reset_first_last();
 
    memcpy(&date, date_in, sizeof(struct tm));
    for (n = 0; n < 7; n++, add_days_to_date(&date, 1)) {
       for (temp_al = a_list; temp_al; temp_al=temp_al->next) {
-#ifdef USE_DB3
+#ifdef ENABLE_DATEBK
 	 if (use_db3_tags) {
-	    ret = db3_is_float(&(temp_al->ma.a), &category);
-	    cat_bit=1<<category;
+	    ret = db3_parse_tag(temp_al->ma.a.note, &db3_type, &db4);
+	    cat_bit=1<<db4.category;
 	    if (!(cat_bit & datebook_category)) continue;
 	 }
 #endif
@@ -678,38 +696,46 @@ int print_weeks_appts(struct tm *date_in, PaperSize paper_size)
    a_list = NULL;
 
    /* Get all of the appointments */
-   get_days_appointments2(&a_list, NULL, 2, 2, 2);
+   get_days_appointments2(&a_list, NULL, 2, 2, 2, NULL);
 
    /* iterate through seven days */
    memcpy(&date, date_in, sizeof(struct tm));
 
    for (n = 0; n < 7; n++, add_days_to_date(&date, 1)) {
-      strftime(short_date, 30, "%a, %d %b, %Y", &date);
+      strftime(short_date, sizeof(short_date), "%a, %d %b, %Y", &date);
       fprintf(out, "%d startday\n(%s) dateline\n", n, short_date);
 
       for (temp_al = a_list; temp_al; temp_al=temp_al->next) {
-#ifdef USE_DB3
+#ifdef ENABLE_DATEBK
 	 if (use_db3_tags) {
-	    ret = db3_is_float(&(temp_al->ma.a), &category);
-	    jpilot_logf(LOG_DEBUG, "category = 0x%x\n", category);
-	    cat_bit=1<<category;
+	    ret = db3_parse_tag(temp_al->ma.a.note, &db3_type, &db4);
+	    jp_logf(JP_LOG_DEBUG, "category = 0x%x\n", db4.category);
+	    cat_bit=1<<db4.category;
 	    if (!(cat_bit & datebook_category)) {
-	       jpilot_logf(LOG_DEBUG, "skip rec not in this category\n");
+	       jp_logf(JP_LOG_DEBUG, "skip rec not in this category\n");
 	       continue;
 	    }
 	 }
 #endif
 	 if (isApptOnDate(&(temp_al->ma.a), &date)) {
-	    bzero(desc, 256);
-	    bzero(short_date, 32);
+	    memset(desc, 0, sizeof(desc));
+	    memset(short_date, 0, sizeof(short_date));
 
 	    if ( ! temp_al->ma.a.event)
 	      {
-		 char t1[21], t2[21];
-		 int j;
-		 strftime(t1, 10, "%I:%M%p", &(temp_al->ma.a.begin));
-		 strftime(t2, 10, "%I:%M%p", &(temp_al->ma.a.end));
-		 sprintf(short_date, "(%s to %s) ", t1, t2);
+		 char t1[6], t2[6], ht[3], mt[3];
+		 int j, m;
+
+		 strftime(ht, sizeof(ht), "%H", &(temp_al->ma.a.begin));
+		 strftime(mt, sizeof(mt), "%M", &(temp_al->ma.a.begin));
+		 m = atoi(mt);
+		 snprintf(t1, sizeof(t1), "%s.%02d", ht, (int)((m * 100.)/60));
+
+		 strftime(ht, sizeof(ht), "%H", &(temp_al->ma.a.end));
+		 strftime(mt, sizeof(mt), "%M", &(temp_al->ma.a.end));
+		 m = atoi(mt);
+		 snprintf(t2, sizeof(t2), "%s.%02d", ht, (int)((m * 100.)/60));
+		 sprintf(short_date, "%s %s ", t1, t2);
 		 for (j=0; j<30;j++) short_date[j] =tolower(short_date[j]);
 	      }
 	    if (temp_al->ma.a.description) {
@@ -724,7 +750,10 @@ int print_weeks_appts(struct tm *date_in, PaperSize paper_size)
    fprintf(out, "\nfinishprinting\n");
    print_close(out);
 
-   return(0);
+#ifdef HAVE_LOCALE_H
+   setlocale(LC_ALL, "");
+#endif
+     return(0);
 }
 
 /*----------------------------------------------------------------------
@@ -745,7 +774,7 @@ int print_address_header()
    date = localtime(&ltime);
 
    get_pref(PREF_SHORTDATE, &ivalue, &svalue);
-   strftime(str, 250, svalue, date);
+   strftime(str, sizeof(str), svalue, date);
 
    fprintf(out,
 	   "%%!PS-Adobe-2.0\n"
@@ -891,6 +920,9 @@ int print_address_header()
 	   "hline\n"
 	   "pop pop pop ( )\n"
 	   "} if\n"
+	   "("Q_FLAG_CHAR"END) search { %%if\n"
+	   "   stop\n"
+	   "} if\n"
 	   "("Q_FLAG_CHAR"C12) search {\n"
 	   "/Courier-ISOLatin1 12 selectfont\n"
 	   "currentpoint 14 add moveto\n"
@@ -971,7 +1003,7 @@ int print_addresses(AddressList *address_list)
    }
 
 #ifdef HAVE_LOCALE_H
-   setlocale(LC_ALL,"C");
+   setlocale(LC_NUMERIC,"C");
 #endif
 
    get_address_app_info(&address_app_info);
@@ -999,7 +1031,7 @@ int print_addresses(AddressList *address_list)
       str[0]='\0';
       if (temp_al->ma.a.entry[show1] || temp_al->ma.a.entry[show2]) {
 	 if (temp_al->ma.a.entry[show1] && temp_al->ma.a.entry[show2]) {
-	    g_snprintf(str, 48, "%s, %s", temp_al->ma.a.entry[show1], temp_al->ma.a.entry[show2]);
+	    g_snprintf(str, sizeof(str), "%s, %s", temp_al->ma.a.entry[show1], temp_al->ma.a.entry[show2]);
 	 }
 	 if (temp_al->ma.a.entry[show1] && ! temp_al->ma.a.entry[show2]) {
 	    strncpy(str, temp_al->ma.a.entry[show1], 48);
@@ -1077,7 +1109,7 @@ int print_todos(ToDoList *todo_list, char *category_name)
    }
 
 #ifdef HAVE_LOCALE_H
-   setlocale(LC_ALL,"C");
+   setlocale(LC_NUMERIC,"C");
 #endif
 
    fprintf(out, "%%!PS-Adobe-2.0\n\n"
@@ -1091,7 +1123,7 @@ int print_todos(ToDoList *todo_list, char *category_name)
 
    get_pref(PREF_SHORTDATE, &ivalue, &datef);
    now = localtime(&ltime);
-   strftime(str, 80, datef, now);
+   strftime(str, sizeof(str), datef, now);
    indent=strlen(str) + 8;
 
    for (temp_l = todo_list; temp_l; temp_l=temp_l->next) {
@@ -1105,7 +1137,7 @@ int print_todos(ToDoList *todo_list, char *category_name)
 	 sprintf(str, "%s           ", "No Due");
 	 str[indent-8]='\0';
       } else {
-	 strftime(str, 20, datef, &(todo->due));
+	 strftime(str, sizeof(str), datef, &(todo->due));
       }
       fprintf(out, "(%s) ", str);
 
@@ -1155,7 +1187,7 @@ int print_memos(MemoList *memo_list)
    }
 
 #ifdef HAVE_LOCALE_H
-   setlocale(LC_ALL,"C");
+   setlocale(LC_NUMERIC,"C");
 #endif
 
    print_address_header();
@@ -1183,6 +1215,8 @@ int print_memos(MemoList *memo_list)
       }
    }
 
+   fprintf(out, "%cEND\n", FLAG_CHAR);
+
 #ifdef HAVE_LOCALE_H
    setlocale(LC_ALL,"");
 #endif
