@@ -23,7 +23,23 @@
 
 void print_common_header(FILE *f)
 {
-    char *common_header =
+         char *common_header =
+        "/Recode {\n"
+        "  exch\n"
+        "  findfont\n"
+        "  dup length dict\n"
+        "  begin\n"
+        "    { def } forall\n"
+        "    /Encoding ISOLatin1Encoding def\n"
+        "    currentdict\n"
+        "  end\n"
+        "  definefont pop\n"
+        "} bind def\n\n"
+        "/Times-Roman      /Times-Roman-ISOLatin1 Recode\n"
+        "/Times-Bold       /Times-Bold-ISOLatin1 Recode\n"
+        "/Courier          /Courier-ISOLatin1 Recode\n"
+        "/Courier-Bold     /Courier-Bold-ISOLatin1 Recode\n"
+        "/Helvetica        /Helvetica-ISOLatin1 Recode\n"
         "/edef   { exch def } def\n"
         "/esub   { exch sub } def\n"
         "/l      { lineto } def\n"
@@ -143,7 +159,7 @@ void print_week_header(FILE *f)
         "\n"
         "%------------------------------------------------------------------\n"
         "\n"
-        "/itemfont       /Helvetica              def\n"
+        "/itemfont       /Helvetica-ISOLatin1    def\n"
         "/datefont       /Helvetica-Bold         def\n"
         "/FirstHour       9 def  % Default beginning and end of the plan\n"
         "/LastHour       22 def\n"
@@ -399,9 +415,8 @@ void print_week_header(FILE *f)
         "            /BotItemY BotItemY linespacing sub def\n"
         "            TodayLM 3 add BotItemY m show\n"
         "        } {\n"
-        "            { count 2 gt { concat } { exit } ifelse } loop\n"
+        "            { count 3 gt { concat } { exit } ifelse } loop\n"
         "            /Details        edef\n"
-        "            StringToTimes\n"
         "            /HourTo         edef\n"
         "            /HourFrom       edef\n"
         "            %----------------------------------------\n"
@@ -565,14 +580,14 @@ void print_month_header(FILE *f)
         "        Y Ytop2 gt { /Y Ytop2 def } if\n"
         "        /Helvetica-Oblique findfont fsize scalefont setfont\n"
         "        X Y m show\n"
-        "        /Helvetica findfont fsize scalefont setfont\n"
+        "        /Helvetica-ISOLatin1 findfont fsize scalefont setfont\n"
         "        X 4.2 fsize mul add Y m ApptText show\n"
         "        /Y Y fsize sub def\n"
         "} def\n"
         "\n"
         "/UntimedItem {\n"
         "        /ApptText edef\n"
-        "        /Helvetica findfont fsize scalefont setfont\n"
+        "        /Helvetica-ISOLatin1 findfont fsize scalefont setfont\n"
         "        X 3 fsize mul add Y m ApptText show\n"
         "        /Y Y fsize sub def\n"
         "} def\n"
