@@ -1,7 +1,7 @@
 /* restore.h
  * A module of J-Pilot http://jpilot.org
  * 
- * Copyright (C) 2001 by Judd Montgomery
+ * Copyright (C) 2001-2002 by Judd Montgomery
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,6 +19,6 @@
 #ifndef _RESTORE_GUI_H__
 #define _RESTORE_GUI_H__
 
-int restore_gui(int w, int h, int x, int y);
+int restore_gui(GtkWidget *main_window, int w, int h, int x, int y);
 
 #endif
