@@ -1,7 +1,7 @@
 /* todo.c
  * A module of J-Pilot http://jpilot.org
  * 
- * Copyright (C) 1999-2001 by Judd Montgomery
+ * Copyright (C) 1999-2002 by Judd Montgomery
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -200,7 +200,7 @@ int todo_sort(ToDoList **todol, int sort_order)
    /* Allocate an array to be qsorted */
    sort_todol = calloc(count, sizeof(ToDoList *));
    if (!sort_todol) {
-      jpilot_logf(LOG_WARN, "todo_sort(): Out of Memory\n");
+      jp_logf(JP_LOG_WARN, "todo_sort(): Out of Memory\n");
       return 0;
    }
 
@@ -233,33 +233,80 @@ int todo_sort(ToDoList **todol, int sort_order)
    return 0;
 }
 
+/*
+ * This function just checks some todo fields to make sure they are valid.
+ * It truncates the description and note fields if necessary.
+ */
+void pc_todo_validate_correct(struct ToDo *todo)
+{
+   if (todo->description) {
+      if ((strlen(todo->description)+1 > MAX_TODO_DESC_LEN)) {
+	 jp_logf(JP_LOG_WARN, "%s\n", todo->description);
+	 jp_logf(JP_LOG_WARN, _("Warning ToDo description too long, truncating to %d\n"), MAX_TODO_DESC_LEN-1);
+	 todo->description[MAX_TODO_DESC_LEN-1]='\0';
+      }
+   }
+   if (todo->note) {
+      if ((strlen(todo->note)+1 > MAX_TODO_NOTE_LEN)) {
+	 jp_logf(JP_LOG_WARN, "%s\n", todo->note);
+	 jp_logf(JP_LOG_WARN, _("Warning ToDo note too long, truncating to %d\n"), MAX_TODO_NOTE_LEN-1);
+	 todo->note[MAX_TODO_NOTE_LEN-1]='\0';
+      }
+   }
+}
+
 int pc_todo_write(struct ToDo *todo, PCRecType rt, unsigned char attrib,
 		  unsigned int *unique_id)
 {
-   char record[65536];
+   unsigned char record[65536];
    int rec_len;
    buf_rec br;
    long char_set;
+#ifdef ENABLE_MANANA
+   long ivalue;
+#endif
 
    get_pref(PREF_CHAR_SET, &char_set, NULL);
-   if (char_set != CHAR_SET_ENGLISH) {
-      if (todo->description) charset_j2p(todo->description, strlen(todo->description)+1, char_set);
-      if (todo->note) charset_j2p(todo->note, strlen(todo->note)+1, char_set);
+   if (char_set != CHAR_SET_LATIN1) {
+      if (todo->description) {
+	 charset_j2p((unsigned char *)todo->description, strlen(todo->description)+1, char_set);
+      }
+      if (todo->note) {
+	 charset_j2p((unsigned char *)todo->note, strlen(todo->note)+1, char_set);
+      }
    }
 
+   pc_todo_validate_correct(todo);
    rec_len = pack_ToDo(todo, record, 65535);
    if (!rec_len) {
       PRINT_FILE_LINE;
-      jpilot_logf(LOG_WARN, "pack_ToDo %s\n", _("error"));
+      jp_logf(JP_LOG_WARN, "pack_ToDo %s\n", _("error"));
       return -1;
    }
    br.rt=rt;
    br.attrib = attrib;
    br.buf = record;
    br.size = rec_len;
+   /* Keep unique ID intact */
+   if (unique_id) {
+      br.unique_id = *unique_id;
+   } else {
+      br.unique_id = 0;
+   }
 
+#ifdef ENABLE_MANANA
+   get_pref(PREF_MANANA_MODE, &ivalue, NULL);
+   if (ivalue) {
+      jp_pc_write("MaanaDB", &br);
+   } else {
+      jp_pc_write("ToDoDB", &br);
+   }
+#else
    jp_pc_write("ToDoDB", &br);
-   *unique_id = br.unique_id;
+#endif
+   if (unique_id) {
+      *unique_id = br.unique_id;
+   }
 
    return 0;
 }
@@ -278,28 +325,53 @@ void free_ToDoList(ToDoList **todo)
 
 int get_todo_app_info(struct ToDoAppInfo *ai)
 {
-   int num,i;
-   unsigned int rec_size;
+   int num, i, r;
+   int rec_size;
    unsigned char *buf;
    long char_set;
+#ifdef ENABLE_MANANA
+   long ivalue;
+#endif
+   char DBname[32];
 
-   bzero(ai, sizeof(*ai));
+   memset(ai, 0, sizeof(*ai));
+   buf=NULL;
+   /* Put at least one entry in there */
+   strcpy(ai->category.name[0], "Unfiled");
 
-   jp_get_app_info("ToDoDB", &buf, &rec_size);
+#ifdef ENABLE_MANANA
+   get_pref(PREF_MANANA_MODE, &ivalue, NULL);                                  
+   if (ivalue) {
+      strcpy(DBname, "MaanaDB");
+   } else {
+      strcpy(DBname, "ToDoDB");
+   }
+#else
+   strcpy(DBname, "ToDoDB");
+#endif
+   r = jp_get_app_info(DBname, &buf, &rec_size);
+
+   if ((r<0) || (rec_size<=0)) {
+      jp_logf(JP_LOG_WARN, _("%s:%d Error reading category info %s\n"), __FILE__, __LINE__, DBname);
+      if (buf) {
+	 free(buf);
+      }
+      return -1;
+   }
    num = unpack_ToDoAppInfo(ai, buf, rec_size);
    if (buf) {
       free(buf);
    }
    if (num <= 0) {
-      jpilot_logf(LOG_WARN, _("Error reading"), "ToDoDB.pdb");
+      jp_logf(JP_LOG_WARN, _("%s:%d Error reading %s\n"), __FILE__, __LINE__, DBname);
       return -1;
    }
 
    get_pref(PREF_CHAR_SET, &char_set, NULL);
-   if (char_set != CHAR_SET_ENGLISH) {
+   if (char_set != CHAR_SET_LATIN1) {
       for (i = 0; i < 16; i++) {
 	 if (ai->category.name[i][0] != '\0') {
-	    charset_p2j(ai->category.name[i], 16, char_set);
+	    charset_p2j((unsigned char *)ai->category.name[i], 16, char_set);
 	 }
       }
    }
@@ -329,8 +401,12 @@ int get_todos2(ToDoList **todo_list, int sort_order,
    int keep_priv;
    buf_rec *br;
    long char_set;
+   char *buf;
+#ifdef ENABLE_MANANA
+   long ivalue;
+#endif
 
-   jpilot_logf(LOG_DEBUG, "get_todos2()\n");
+   jp_logf(JP_LOG_DEBUG, "get_todos2()\n");
    if (modified==2) {
       get_pref(PREF_SHOW_MODIFIED, &keep_modified, NULL);
    } else {
@@ -342,7 +418,7 @@ int get_todos2(ToDoList **todo_list, int sort_order,
       keep_deleted = deleted;
    }
    if (privates==2) {
-      keep_priv = show_privates(GET_PRIVATES, NULL);
+      keep_priv = show_privates(GET_PRIVATES);
    } else {
       keep_priv = privates;
    }
@@ -355,7 +431,16 @@ int get_todos2(ToDoList **todo_list, int sort_order,
    *todo_list=NULL;
    recs_returned = 0;
 
+#ifdef ENABLE_MANANA
+   get_pref(PREF_MANANA_MODE, &ivalue, NULL);
+   if (ivalue) {
+      num = jp_read_DB_files("MaanaDB", &records);
+   } else {
+      num = jp_read_DB_files("ToDoDB", &records);
+   }
+#else
    num = jp_read_DB_files("ToDoDB", &records);
+#endif
    /* Go to first entry in the list */
    for (temp_list = records; temp_list; temp_list = temp_list->prev) {
       records = temp_list;
@@ -394,12 +479,33 @@ int get_todos2(ToDoList **todo_list, int sort_order,
       }
 
       get_pref(PREF_CHAR_SET, &char_set, NULL);
-      if (todo.description) charset_p2j(todo.description, strlen(todo.description)+1, char_set);
-      if (todo.note) charset_p2j(todo.note, strlen(todo.note)+1, char_set);
-
+      if (todo.description) {
+         buf = charset_p2newj((unsigned char *)todo.description, strlen(todo.description)+1, char_set);
+         if (buf) {
+	    /* JPA use new conversion routines
+	     if ((buf = (char *)malloc(strlen(todo.description)*2+1)) != NULL) {
+	     strcpy(buf, todo.description);
+	     charset_p2j((unsigned char *)buf, strlen(todo.description)*2+1, char_set);
+	     */
+	    free(todo.description);
+	    todo.description = buf;
+	 }
+      }
+      if (todo.note) {
+	 buf = charset_p2newj((unsigned char *)todo.note, strlen(todo.note)+1, char_set);
+         if (buf) {
+	    /* JPA use new conversion routines
+	     if ((buf = (char *)malloc(strlen(todo.note)*2+1)) != NULL) {
+	     strcpy(buf, todo.note);
+	     charset_p2j((unsigned char *)buf, strlen(todo.note)*2+1, char_set);
+	     */
+	    free(todo.note);
+	    todo.note = buf;
+	 }
+      }
       temp_todo_list = malloc(sizeof(ToDoList));
       if (!temp_todo_list) {
-	 jpilot_logf(LOG_WARN, "get_todos2(): Out of memory\n");
+	 jp_logf(JP_LOG_WARN, "get_todos2(): Out of memory\n");
 	 break;
       }
       memcpy(&(temp_todo_list->mtodo.todo), &todo, sizeof(struct ToDo));
@@ -416,7 +522,7 @@ int get_todos2(ToDoList **todo_list, int sort_order,
 
    todo_sort(todo_list, sort_order);
 
-   jpilot_logf(LOG_DEBUG, "Leaving get_todos2()\n");
+   jp_logf(JP_LOG_DEBUG, "Leaving get_todos2()\n");
 
    return recs_returned;
 }
