@@ -1,7 +1,7 @@
 /* todo.h
  * A module of J-Pilot http://jpilot.org
  * 
- * Copyright (C) 1999-2001 by Judd Montgomery
+ * Copyright (C) 1999-2002 by Judd Montgomery
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,6 +22,15 @@
 #include <pi-todo.h>
 #include "utils.h"
 
+#define MAX_TODO_DESC_LEN	256
+#define MAX_TODO_NOTE_LEN	4000
+
+#define TODO_CHECK_COLUMN     0
+#define TODO_PRIORITY_COLUMN  1
+#define TODO_NOTE_COLUMN      2
+#define TODO_DATE_COLUMN      3
+#define TODO_TEXT_COLUMN      4
+
 void free_ToDoList(ToDoList **todo);
 int get_todos(ToDoList **todo_list, int sort_order);
 int get_todos2(ToDoList **todo_list, int sort_order,
@@ -34,4 +43,8 @@ int todo_print();
 int todo_import(GtkWidget *window);
 int todo_export(GtkWidget *window);
 
+/* I only export this for datebook, don't use this */
+void todo_update_clist(GtkWidget *clist, GtkWidget *tooltip_widget,
+		       ToDoList *todo_list, int category, int main);
+
 #endif
