@@ -1,7 +1,7 @@
 /* utils.h
  * A module of J-Pilot http://jpilot.org
  * 
- * Copyright (C) 1999-2001 by Judd Montgomery
+ * Copyright (C) 1999-2002 by Judd Montgomery
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,24 +27,30 @@
 #include <pi-address.h>
 #include <pi-todo.h>
 #include <pi-memo.h>
+#include <pi-file.h>
 #include <gtk/gtk.h>
 
 #include "libplugin.h"
 #include "japanese.h"
 #include "cp1250.h"
+#include "cp1253.h"
 #include "russian.h"
 
 #define PRINT_FILE_LINE printf("%s line %d\n", __FILE__, __LINE__)
 
-#ifdef WITH_PROMETHEON
+#ifdef ENABLE_PROMETHEON
 #define PN "CoPilot"
 #else
 #define PN "J-Pilot"
 #endif
 
+#ifdef ENABLE_PROMETHEON
+#define EPN "copilot"
+#else
 #define EPN "jpilot"
-#define VERSION_STRING "\n"PN" version "VERSION"\n"\
-" Copyright (C) 1999-2001 by Judd Montgomery\n"
+#endif
+
+#define	FPI_STRING	"-//Judd Montgomery//NONSGML "PN" "VERSION"//EN"
 
 /*This is how often the clock updates in milliseconds */
 #define CLOCK_TICK 1000
@@ -53,6 +59,20 @@
 
 #define SHADOW GTK_SHADOW_ETCHED_OUT
 
+/*
+ *          JPA define the maximum length of a category name
+ *             when expressed in Pilot character set (assuming 15
+ *             character plus a delimiter) or in host character
+ *             set (might be 50 or more if UTF-8).
+ *         Note : host length is temporarily kept as 16 until all
+ *             consequences are identified. In the meantime category
+ *             names may be (hopefully safely) truncated.
+ */
+
+#define HOSTCATLTH 16
+#define PILOTCATLTH 16
+#define CATCOUNT 16 /* number of categories */
+
 /*Used to mark the entry in the clist to add a record */
 /* FIXME: Move to libplugin */
 #define CLIST_NEW_ENTRY_DATA 100
@@ -96,11 +116,14 @@
 #define IMPORT_TYPE_XML     103
 
 /* Export defines */
-#define EXPORT_TYPE_UNKNOWN  99
-#define EXPORT_TYPE_TEXT    100
-#define EXPORT_TYPE_DAT     101
-#define EXPORT_TYPE_CSV     102
-#define EXPORT_TYPE_XML     103
+#define EXPORT_TYPE_UNKNOWN    99
+#define EXPORT_TYPE_TEXT      100
+#define EXPORT_TYPE_DAT       101
+#define EXPORT_TYPE_CSV       102
+#define EXPORT_TYPE_XML       103
+#define EXPORT_TYPE_VCARD     104
+#define EXPORT_TYPE_ICALENDAR 105
+#define EXPORT_TYPE_LDIF      106
 
 /* Dat file types */
 #define DAT_DATEBOOK_FILE    10
@@ -240,13 +263,28 @@ int setup_sync(unsigned int flags);
 
 /* Found in jpilot.c */
 void cb_sync(GtkWidget *widget, unsigned int flags);
+void get_compile_options(char *string, int len);
+
+int dialog_generic_with_text(GtkWindow *main_window,
+			     int w, int h,
+			     char *title, char *frame_text,
+			     char *text, int nob, char *button_text[],
+			     int with_text);
 
 /* Returns the number of the button that was pressed */
-int dialog_generic(GdkWindow *main_window,
+int dialog_generic(GtkWindow *main_window,
 		   int w, int h,
 		   char *title, char *frame_text,
 		   char *text, int nob, char *button_text[]);
 
+/*
+ * Widget must be some widget used to get the main window from.
+ * The main window passed in would be fastest.
+ * This just calls dialog_generic with an OK button.
+ */
+int dialog_generic_ok(GtkWidget *widget,
+		      char *title, char *frame_text, char *text);
+
 /*
  * Widget must be some widget used to get the main window from.
  * The main window passed in would be fastest.
@@ -260,35 +298,36 @@ int dialog_save_changed_record(GtkWidget *widget, int changed);
  * This function will bring up the cal at mon, day, year
  * After a new date is selected it will return mon, day, year
  */
-int cal_dialog(const char *title, int monday_is_fdow,
+int cal_dialog(GtkWindow *main_window,
+	       const char *title, int monday_is_fdow,
 	       int *mon, int *day, int *year);
 
+void set_bg_rgb_clist_row(GtkWidget *clist, int row, int r, int g, int b);
+
+void set_fg_rgb_clist_cell(GtkWidget *clist, int row, int col, int r, int g, int b);
+
 int clist_find_id(GtkWidget *clist,
 		  unsigned int unique_id,
 		  int *found_at,
 		  int *total_count);
 
-int move_scrolled_window(GtkWidget *sw, float percentage);
-
-void move_scrolled_window_hack(GtkWidget *sw, float percentage);
-
 int check_copy_DBs_to_home();
 
-int jpilot_copy_file(char *src, char *dest);
+int jp_copy_file(char *src, char *dest);
 
 /*search_gui.c */
 void cb_search_gui(GtkWidget *widget, gpointer data);
 
 /*install_gui.c */
-int install_gui(int w, int h, int x, int y);
+int install_gui(GtkWidget *main_window, int w, int h, int x, int y);
 /*install_gui.c */
-int install_append_line(char *line);
+int install_append_line(const char *line);
 
 /*import_gui.c */
 void import_gui(GtkWidget *main_window, GtkWidget *main_pane,
 		char *type_desc[], int type_int[],
 		int (*import_callback)(GtkWidget *parent_window,
-				       char *file_path, int type));
+				       const char *file_path, int type));
 int import_record_ask(GtkWidget *main_window, GtkWidget *pane,
 		      char *text, struct CategoryAppInfo *cai,
 		      char *old_cat_name,
@@ -299,6 +338,7 @@ int import_record_ask(GtkWidget *main_window, GtkWidget *pane,
 #define DAT_TYPE_CSTRING 5
 #define DAT_TYPE_DATE    3
 #define DAT_TYPE_BOOLEAN 6
+#define DAT_TYPE_BITFLAG 7
 #define DAT_TYPE_REPEAT  8
 
 /* Returns a dat type, or 0 */
@@ -308,12 +348,6 @@ int dat_get_addresses(FILE *in, AddressList **addrlist, struct CategoryAppInfo *
 int dat_get_todos(FILE *in, ToDoList **todolist, struct CategoryAppInfo *ai);
 int dat_get_memos(FILE *in, MemoList **memolist, struct CategoryAppInfo *ai);
 
-/*weekview_gui.c */
-void cb_weekview_gui(GtkWidget *widget, gpointer data);
-
-/*monthview_gui.c */
-void cb_monthview_gui(GtkWidget *widget, gpointer data);
-
 void free_search_record_list(struct search_record **sr);
 
 
@@ -340,11 +374,42 @@ int str_to_csv_str(char *dest, char *src);
  */
 int csv_str_to_str(char *dest, char *src);
 
+/*
+ * Copy src string into dest while escaping carraige returns,
+ * backslashes, commas and semicolons.  Also do line wrapping.
+ * dest could be as long as strlen(src) * 2 + strlen(src) / 30.
+ * Return value is the number of chars written to dest.
+ */
+int str_to_ical_str(char *, int, char *);
+
+/*
+ * Same as str_to_ical_str() except doesn't escape semicolons.
+ */
+int str_to_vcard_str(char *, int, char *);
+
 /*
  * Parse the string and replace CR and LFs with spaces
  */
 void remove_cr_lfs(char *str);
 
+/*
+ * Parse the string and replace CR and LFs with spaces
+ * a null is written if len is reached
+ */
+void lstrncpy_remove_cr_lfs(char *dest, char *src, int len);
+
+/*
+ * Output LDIF format (RFC 2849) to file.
+ * Name is name of item (e.g. "cn")
+ * fmt ... is like printf.
+ */
+void ldif_out(FILE *f, char *name, char *fmt, ...);
+
+/*
+ * Output base64-encoded string to file
+ */
+void base64_out(FILE *, char *);
+
 void cleanup_path(char *path);
 
 int add_days_to_date(struct tm *date, int n);
@@ -369,12 +434,15 @@ int datebook_refresh(int first);
 
 /*address_gui */
 int address_refresh();
-
+int address_cycle_cat();
+ 
 /*todo_gui */
 int todo_refresh();
+int todo_cycle_cat();
 
 /*memo_gui */
 int memo_refresh();
+int memo_cycle_cat();
 
 /* monthview_gui */
 void monthview_gui(struct tm *date);
@@ -382,6 +450,73 @@ void monthview_gui(struct tm *date);
 /* weekview_gui */
 void weekview_gui(struct tm *date_in);
 
+/* dialer.c */
+int dialog_dial(GtkWindow *main_window, char *string, char *ext);
+
+/* These are in utils.c for now */
+/*
+ * DB_name should be without filename ext, e.g. MemoDB
+ * num is the number of records counted returned.
+ */
+int pdb_file_count_recs(char *DB_name, int *num);
+/*
+ * DB_name should be without filename ext, e.g. MemoDB
+ * uid_in the the unique ID to remove from the pdb file
+ */
+int pdb_file_delete_record_by_id(char *DB_name, pi_uid_t uid_in);
+/*
+ * DB_name should be without filename ext, e.g. MemoDB
+ * uid_in the the unique ID to modify from the pdb file
+ * the other parameters are set in the pdb file for this record.
+ */
+int pdb_file_modify_record(char *DB_name, void *record_in, int size_in,
+			   int attr_in, int cat_in, pi_uid_t uid_in);
+/*
+ * DB_name should be without filename ext, e.g. MemoDB
+ * uid is unique id in
+ * bufp is a copy of the raw record (unpacked) and should be freed by caller
+ * sizep is size of bufp returned
+ * idxp is the index in the file rec was found
+ * attrp is the attributes
+ * catp is the category (index)
+ */
+int pdb_file_read_record_by_id(char *DB_name, 
+			       pi_uid_t uid,
+			       void **bufp, int *sizep, int *idxp,
+			       int *attrp, int * catp);
+/*
+ * DB_name should be without filename ext, e.g. MemoDB
+ * bufp is the packed app info block
+ * size_in is the size of bufp
+ */
+int pdb_file_write_app_block(char *DB_name, void *bufp, int size_in);
+
+/*
+ * This copies the datebase (pdb, or prc) and writes the DBInfo privided
+ * since there is no other way to set it in a file.
+ */
+int pdb_file_write_dbinfo(char *DB_name, struct DBInfo *Pinfo_in);
+
+/* category.c */
+/*
+ * widget is a widget inside the main window used to get main window handle
+ * db_name should be without filename ext, e.g. MemoDB
+ * cai is the category app info.  This should be unpacked by the user since
+ * category unpack functions are database specific.
+ */
+int edit_cats(GtkWidget *widget, char *db_name, struct CategoryAppInfo *cai);
+/*
+ * This changes every record with index old_index and changes it to new_index
+ * returns the number of record's categories changed.
+ */
+int pdb_file_change_indexes(char *DB_name, int old_index, int new_index);
+int edit_cats_change_cats_pc3(char *DB_name, int old_cat,
+			      int new_cat);
+int edit_cats_swap_cats_pc3(char *DB_name, int old_cat,
+			    int new_cat);
+int edit_cats_change_cats_pdb(char *DB_name, int old_cat, int new_cat);
+
+  
 int make_category_menu(GtkWidget **category_menu,
 		       GtkWidget **cat_menu_item,
 		       struct sorted_cats *sort_l,
@@ -389,22 +524,18 @@ int make_category_menu(GtkWidget **category_menu,
 		       (GtkWidget *item, int selection),
 		       int add_an_all_item);
 
-void multibyte_safe_strncpy(char *dst, char *src, size_t max_len);
+void multibyte_safe_strncpy(char *dst, char *src, size_t len);
 char *multibyte_safe_memccpy(char *dst, const char *src, int c, size_t len);
 
 /*************************************
  * convert char code 
  *************************************/
-#define charset_j2p(buf, max_len, char_set)  {\
-	if (char_set == CHAR_SET_JAPANESE) Euc2Sjis(buf, max_len);\
-	if (char_set == CHAR_SET_1250) Lat2Win(buf,max_len);\
-	if (char_set == CHAR_SET_1251) koi8_to_win1251(buf, max_len);\
-	if (char_set == CHAR_SET_1251_B) win1251_to_koi8(buf, max_len);}
-#define charset_p2j(buf, max_len, char_set) {\
-        if (char_set == CHAR_SET_JAPANESE) Sjis2Euc(buf, max_len);\
-        if (char_set == CHAR_SET_1250) Win2Lat(buf,max_len);\
-        if (char_set == CHAR_SET_1251) win1251_to_koi8(buf, max_len);\
-        if (char_set == CHAR_SET_1251_B) koi8_to_win1251(buf, max_len);}
+/* host character set to Palm character set */
+void charset_j2p(unsigned char *buf, int max_len, long char_set);
+
+/* Palm character set to host character set */
+void charset_p2j(unsigned char *buf, int max_len, int char_set);
+unsigned char *charset_p2newj(const unsigned char *buf, int max_len, int char_set);
 
 void jp_charset_p2j(unsigned char *buf, int max_len);
 void jp_charset_j2p(unsigned char *buf, int max_len);
