@@ -1,7 +1,7 @@
 /* weekview_gui.c
  * A module of J-Pilot http://jpilot.org
  *
- * Copyright (C) 1999-2001 by Judd Montgomery
+ * Copyright (C) 1999-2002 by Judd Montgomery
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,7 +34,11 @@ extern int datebook_category;
 
 
 static GtkWidget *window=NULL;
+static GtkWidget *glob_dow_labels[8];
 static GtkWidget *glob_week_texts[8];
+#ifdef ENABLE_GTK2
+static GObject   *glob_week_text_buffers[8];
+#endif
 static struct tm glob_week_date;
 
 /* Function prototypes */
@@ -52,7 +56,12 @@ static void
   cb_quit(GtkWidget *widget,
 	   gpointer   data)
 {
-   window = NULL;
+   int w, h;
+
+   gdk_window_get_size(window->window, &w, &h);
+   set_pref(PREF_WEEKVIEW_WIDTH, w, NULL, FALSE);
+   set_pref(PREF_WEEKVIEW_HEIGHT, h, NULL, FALSE);
+
    gtk_widget_destroy(data);
 }
 
@@ -64,7 +73,7 @@ static void cb_week_print(GtkWidget *widget, gpointer data)
 {
    long paper_size;
 
-   jpilot_logf(LOG_DEBUG, "cb_week_print called\n");
+   jp_logf(JP_LOG_DEBUG, "cb_week_print called\n");
    if (print_gui(window, DATEBOOK, 2, 0x02) == DIALOG_SAID_PRINT) {
       get_pref(PREF_PAPER_SIZE, &paper_size, NULL);
       if (paper_size==1) {
@@ -82,7 +91,11 @@ void freeze_weeks_appts()
    int i;
 
    for (i=0; i<8; i++) {
+#ifdef ENABLE_GTK2
+      gtk_widget_freeze_child_notify(glob_week_texts[i]);
+#else
       gtk_text_freeze(GTK_TEXT(glob_week_texts[i]));
+#endif
    }
 }
 
@@ -91,7 +104,11 @@ void thaw_weeks_appts()
    int i;
 
    for (i=0; i<8; i++) {
+#ifdef ENABLE_GTK2
+      gtk_widget_thaw_child_notify(glob_week_texts[i]);
+#else
       gtk_text_thaw(GTK_TEXT(glob_week_texts[i]));
+#endif
    }
 }
 
@@ -105,20 +122,31 @@ cb_week_move(GtkWidget *widget,
    if (GPOINTER_TO_INT(data)==1) {
       add_days_to_date(&glob_week_date, 7);
    }
+
    freeze_weeks_appts();
+
    clear_weeks_appts(glob_week_texts);
    display_weeks_appts(&glob_week_date, glob_week_texts);
+
    thaw_weeks_appts();
 }
 
 int clear_weeks_appts(GtkWidget **day_texts)
 {
    int i;
+#ifdef ENABLE_GTK2
+   GObject   *text_buffer;
+#endif
 
    for (i=0; i<8; i++) {
+#ifdef ENABLE_GTK2
+      text_buffer = G_OBJECT(gtk_text_view_get_buffer(GTK_TEXT_VIEW(day_texts[i])));
+      gtk_text_buffer_set_text(GTK_TEXT_BUFFER(text_buffer), "", -1);
+#else
       gtk_text_set_point(GTK_TEXT(day_texts[i]), 0);
       gtk_text_forward_delete(GTK_TEXT(day_texts[i]),
 			      gtk_text_get_length(GTK_TEXT(day_texts[i])));
+#endif
    }
    return 0;
 }
@@ -131,14 +159,6 @@ int clear_weeks_appts(GtkWidget **day_texts)
  */
 int display_weeks_appts(struct tm *date_in, GtkWidget **day_texts)
 {
-   char *days[]={
-      gettext_noop("Sunday"),
-      gettext_noop("Monday"),
-      gettext_noop("Tuesday"),
-      gettext_noop("Wednesday"),
-      gettext_noop("Thursday"),
-      gettext_noop("Friday"),
-      gettext_noop("Saturday")};
    AppointmentList *a_list;
    AppointmentList *temp_al;
    struct tm date;
@@ -149,38 +169,24 @@ int display_weeks_appts(struct tm *date_in, GtkWidget **day_texts)
    long ivalue;
    const char *svalue;
    char str[82];
+   char str_dow[32];
    long fdow;
    char short_date[32];
    char default_date[]="%x";
-   /* GdkFont *small_font; */
-   GdkColor color;
-   GdkColormap *colormap;
-   /* long char_set;*/
-#ifdef USE_DB3
+#ifdef ENABLE_DATEBK
    int ret;
-   int category;
    int cat_bit;
+   int db3_type;
    long use_db3_tags;
+   struct db4_struct db4;
+#endif
+#ifdef ENABLE_GTK2
+   GObject   *text_buffer;
 #endif
 
    a_list = NULL;
    text = day_texts;
 
-   /*
-   get_pref(PREF_CHAR_SET, &char_set, NULL);
-   if (char_set==CHAR_SET_1250) {
-       small_font = gdk_fontset_load("-misc-fixed-medium-r-*-*-*-100-*-*-*-iso8859-2");
-   } else {
-       small_font = gdk_fontset_load("-misc-fixed-medium-r-*-*-*-100-*-*-*-*-*");
-   }
-   */
-   color.red = 0xAAAA;
-   color.green = 0xAAAA;
-   color.blue = 0xAAAA;
-
-   colormap = gtk_widget_get_colormap(text[0]);
-   gdk_color_alloc(colormap, &color);
-
    memcpy(&date, date_in, sizeof(struct tm));
 
    get_pref(PREF_FDOW, &fdow, &svalue);
@@ -191,14 +197,14 @@ int display_weeks_appts(struct tm *date_in, GtkWidget **day_texts)
    }
 
    for (i=0; i<8; i++, add_days_to_date(&date, 1)) {
-      strftime(short_date, 30, svalue, &date);
-      g_snprintf(str, 80, "%s %s\n", _(days[(i + fdow)%7]), short_date);
-      str[80]='\0';
-      gtk_text_insert(GTK_TEXT(glob_week_texts[i]), NULL, NULL, &color, str, -1);
+      strftime(short_date, sizeof(short_date), svalue, &date);
+      strftime(str_dow, sizeof(str_dow), "%A", &date);
+      g_snprintf(str, sizeof(str), "%s %s", str_dow, short_date);
+      gtk_label_set_text(GTK_LABEL(glob_dow_labels[i]), str);
    }
 
    /* Get all of the appointments */
-   get_days_appointments2(&a_list, NULL, 2, 2, 2);
+   get_days_appointments2(&a_list, NULL, 2, 2, 2, NULL);
 
    /* iterate through eight days */
    memcpy(&date, date_in, sizeof(struct tm));
@@ -206,36 +212,38 @@ int display_weeks_appts(struct tm *date_in, GtkWidget **day_texts)
 
    for (n=0; n<8; n++, add_days_to_date(&date, 1)) {
       for (temp_al = a_list; temp_al; temp_al=temp_al->next) {
-#ifdef USE_DB3
+#ifdef ENABLE_DATEBK
 	 get_pref(PREF_USE_DB3, &use_db3_tags, NULL);
 	 if (use_db3_tags) {
-	    ret = db3_is_float(&(temp_al->ma.a), &category);
-	    jpilot_logf(LOG_DEBUG, "category = 0x%x\n", category);
-	    cat_bit=1<<category;
+	    ret = db3_parse_tag(temp_al->ma.a.note, &db3_type, &db4);
+	    jp_logf(JP_LOG_DEBUG, "category = 0x%x\n", db4.category);
+	    cat_bit=1<<db4.category;
 	    if (!(cat_bit & datebook_category)) {
-	       jpilot_logf(LOG_DEBUG, "skipping rec not in this category\n");
+	       jp_logf(JP_LOG_DEBUG, "skipping rec not in this category\n");
 	       continue;
 	    }
 	 }
 #endif
 	 if (isApptOnDate(&(temp_al->ma.a), &date)) {
 	    if (temp_al->ma.a.event) {
-	       desc[0]='\0';
+	       strcpy(desc, "*");
 	    } else {
 	       get_pref_time_no_secs(datef);
-	       strftime(desc, 20, datef, &(temp_al->ma.a.begin));
+	       strftime(desc, sizeof(desc), datef, &(temp_al->ma.a.begin));
+	       strcat(desc, " ");
 	    }
-	    strcat(desc, " ");
 	    if (temp_al->ma.a.description) {
 	       strncat(desc, temp_al->ma.a.description, 70);
 	       desc[62]='\0';
 	    }
 	    remove_cr_lfs(desc);
 	    strcat(desc, "\n");
-	    /* gtk_text_insert(GTK_TEXT(text[n]),
-			    small_font, NULL, NULL, desc, -1);*/
-	    gtk_text_insert(GTK_TEXT(text[n]),
-			    NULL, NULL, NULL, desc, -1);
+#ifdef ENABLE_GTK2
+	    text_buffer = G_OBJECT(gtk_text_view_get_buffer(GTK_TEXT_VIEW(text[n])));
+	    gtk_text_buffer_insert_at_cursor(GTK_TEXT_BUFFER(text_buffer),desc,-1);
+#else
+	    gtk_text_insert(GTK_TEXT(text[n]), NULL, NULL, NULL, desc, -1);
+#endif
 	 }
       }
    }
@@ -256,6 +264,7 @@ void weekview_gui(struct tm *date_in)
    long fdow;
    int i;
    char title[200];
+   unsigned long w, h;
 
    if (window) {
       return;
@@ -263,12 +272,18 @@ void weekview_gui(struct tm *date_in)
 
    memcpy(&glob_week_date, date_in, sizeof(struct tm));
 
-   window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-   gtk_window_set_default_size(GTK_WINDOW(window), 800, 600);
+   get_pref(PREF_WEEKVIEW_WIDTH, &w, NULL);
+   get_pref(PREF_WEEKVIEW_HEIGHT, &h, NULL);
+
+   g_snprintf(title, sizeof(title), "%s %s", PN, _("Weekly View"));
+   window = gtk_widget_new(GTK_TYPE_WINDOW,
+			   "type", GTK_WINDOW_TOPLEVEL,
+			   "title", title,
+			   NULL);
+
+   gtk_window_set_default_size(GTK_WINDOW(window), w, h);
+
    gtk_container_set_border_width(GTK_CONTAINER(window), 10);
-   g_snprintf(title, 200, "%s %s", PN, _("Weekly View"));
-   title[199]='\0';
-   gtk_window_set_title(GTK_WINDOW(window), title);
 
    gtk_signal_connect(GTK_OBJECT(window), "destroy",
                       GTK_SIGNAL_FUNC(cb_destroy), window);
@@ -329,11 +344,28 @@ void weekview_gui(struct tm *date_in)
    sub_days_from_date(&glob_week_date, (7 - fdow + glob_week_date.tm_wday)%7);
 
    for (i=0; i<8; i++) {
+      glob_dow_labels[i] = gtk_label_new("");
+      gtk_misc_set_alignment(GTK_MISC(glob_dow_labels[i]), 0.0, 0.5);
+#ifdef ENABLE_GTK2
+      glob_week_texts[i] = gtk_text_view_new();
+      glob_week_text_buffers[i] = G_OBJECT(gtk_text_view_get_buffer(GTK_TEXT_VIEW(glob_week_texts[i])));
+      gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(glob_week_texts[i]), FALSE);
+      gtk_text_view_set_editable(GTK_TEXT_VIEW(glob_week_texts[i]), FALSE);
+      gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(glob_week_texts[i]), GTK_WRAP_WORD);
+      gtk_container_set_border_width(GTK_CONTAINER(glob_week_texts[i]), 1);
+      gtk_text_buffer_create_tag(GTK_TEXT_BUFFER(glob_week_text_buffers[i]), 
+				 "gray_background", "background", "gray", 
+			         NULL);
+#else
       glob_week_texts[i] = gtk_text_new(NULL, NULL);
+#endif
+      gtk_widget_set_usize(GTK_WIDGET(glob_week_texts[i]), 10, 10);
       if (i>3) {
-	 gtk_box_pack_start(GTK_BOX(vbox_right), glob_week_texts[i], FALSE, FALSE, 0);
+	 gtk_box_pack_start(GTK_BOX(vbox_right), glob_dow_labels[i], FALSE, FALSE, 0);
+	 gtk_box_pack_start(GTK_BOX(vbox_right), glob_week_texts[i], TRUE, TRUE, 0);
       } else {
-	 gtk_box_pack_start(GTK_BOX(vbox_left), glob_week_texts[i], FALSE, FALSE, 0);
+	 gtk_box_pack_start(GTK_BOX(vbox_left), glob_dow_labels[i], FALSE, FALSE, 0);
+	 gtk_box_pack_start(GTK_BOX(vbox_left), glob_week_texts[i], TRUE, TRUE, 0);
       }
    }
 
